/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id: Common.java,v 1.3 2003/05/19 16:27:22 wiznogood Exp $
 */

/*
 *
 * @author    Dragan Radeka & Nenad Vico
 * @since     LBS1.8
 * @version   $Revision: 1.3 $
 *
 */
package org.enhydra.dods;

import java.io.File;
import java.io.FileInputStream;
import java.io.FilenameFilter;
import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;

import org.enhydra.dods.wizard.DirectoryNameFilter;
import org.enhydra.xml.XMLConfig;
import org.enhydra.xml.XMLDocumentFactory;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;

public class Common {

    /**
     * dods configuration property
     */
    static protected Properties dods_conf;

    /**
     * last used database read from doml file
     */
    static protected String dbase;

    /**
     * dods configuration
     */
    static protected XMLConfig dodsConf;

    /**
     * dods vendor configuration
     */
    static protected HashMap dodsVendorConfs;

    /**
     * last used project root
     */
    static protected String projRoot;

    /**
     * last used doml file
     */
    static protected String domlfile;

    /**
     * last used template set
     */
    static protected String templateSet;

    static {
        init();
    }

    static private void init() {
        String dodsHome = System.getProperty("DODS_HOME");
        if (dodsHome == null) {
            dodsHome = System.getProperty("enhydra.home");
            if (dodsHome != null)
                dodsHome = dodsHome + File.separator + "dods";
            else
                System.out.println("JVM can not find neither DODS HOME nor ENHYDRA HOME!");
        }
        String dodsFile = dodsHome+File.separator+"build"+File.separator+"conf"+File.separator+"dodsConf.xml";
        Document doc = XMLDocumentFactory.parse(dodsFile);
        dodsConf = XMLConfig.newXMLConfigInstance(doc);
        dodsVendorConfs = new HashMap();
        XMLConfig vendor = null;
        String vendorName = null;
        String vendorFile = null;
        NodeList vendors = dodsConf.getSubElementsByTagName("Database/Vendor");
        for (int i=0; i<vendors.getLength(); i++) {
            vendor = (XMLConfig)vendors.item(i);
            vendorName = vendor.getAttribute("name");
            vendorFile = vendor.getText();
            doc = XMLDocumentFactory.parse(vendorFile);
            XMLConfig dodsVendorConf = XMLConfig.newXMLConfigInstance(doc);
            dodsVendorConfs.put(vendorName,dodsVendorConf);
//            System.out.println(vendorName+" = "+dodsVendorConf);
        }
    }

    /**
     * Get dods conf property.
     *
     * @param key Key of the property.
     * @param database Used database.
     *
     * @return Dods conf property.
     */
    static public String getDodsConfProperty(String key, String database) {
        XMLConfig vendor = (XMLConfig)dodsVendorConfs.get(database);
        if (vendor != null)
            return vendor.getText(key);
        return null;
    }

    /**
     * Show dods.config properties.
     */
    static public void showDodsConf() {
        StringBuffer ret = new StringBuffer();
        Iterator iter = dods_conf.entrySet().iterator();
        while(iter.hasNext()) {
            Map.Entry elm = (Map.Entry)iter.next();
            ret.append(elm.getKey()).append("=").append(elm.getValue()).append("\n");
        }
    }

    /**
     * Get path to dods-ejen.properties.
     *
     * @return Path to dods-ejen.properties.
     */
    public static String getDodsEjenPropertyFilename() {
        final String SYS_USER_HOME = "user.home";
        final String DIR_ENHYDRA = ".enhydra";
        final String PROPERTY_FILENAME = "dods-ejen.properties";

        StringBuffer buf = new StringBuffer();

        buf.append(System.getProperties().getProperty(SYS_USER_HOME));
        buf.append(File.separator);
        buf.append(DIR_ENHYDRA);
        buf.append(File.separator);
        buf.append(PROPERTY_FILENAME);
        return buf.toString();

    }

    /**
     * Get used doml file.
     * @return Used doml file.
     */
    public static String getDomlFile() {

        String strDoml = null;
        strDoml=System.getProperty("DOML_FILE");
        String domlFile=null;
        if(strDoml==null) {
            File current = new File(".");
            String[] files = current.list();
            out:
            for (int i=0; i< files.length; i++) {
                if(files[i].toLowerCase().endsWith(".doml")){
                    domlFile=files[i];
                    break out;
                }
                else
                    domlFile=null;
            }
            strDoml=domlFile;
        }
        return strDoml;
    }

    /**
     * Get used doml file name.
     * @return Used doml file name.
     */
    public static String getDomlFileName() {

        String domlFileName=getDomlFile();
        String fileSep = System.getProperty("file.separator");
        if (System.getProperty("os.name").toLowerCase().startsWith("win")) {
                if (fileSep != "/") {
                        domlFileName = domlFileName.replace('\\', '/');
                    }
        }
        return domlFileName.substring(domlFileName.lastIndexOf("/")+1);
    }

    /**
     * Get file name from used doml file name.
     * @return file name from used doml file name.
     */
    public static String getFileName() {

        String fileName = getDomlFileName();
        int index = fileName.lastIndexOf(".");
        if (index != -1)
            fileName = fileName.substring(0,index);
        return fileName;
    }

    /**
     * Get used databse vendor.
     * @return used databse vendor.
     */
    public static String getDatabaseVendor() {
        String database = System.getProperty("DATABASE_VENDOR");
        return database;
    }

    /**
     * Set used doml file.
     * @param domlfile Used doml file.
     */
    public static void setDomlFile(String doml) {
        domlfile = doml;
    }

    /**
     * Get used project root.
     * @return Project root.
     */
    public static String getProjectRoot() {
        String projectRoot = null;
        projectRoot=System.getProperty("PROJECT_ROOT");
        if(projectRoot==null)
            projectRoot=".";
        return projectRoot;
    }

    /**
     * Set used project root.
     * @parma Project root.
     */
    public static void setProjectRoot(String projectRoot) {
        projRoot = projectRoot;
    }

    /**
     * Get used template set.
     * @return template set.
     */
    public static String getTemplateSet() {
        String tempSet=System.getProperty("TEMPLATESET");
        if(tempSet==null)
            tempSet="standard";
        if(tempSet.equals("multidb"))
            tempSet="standard";
        if(tempSet.equals("webdocwf"))
            tempSet="webdocwf";
        if(tempSet.equals("multidb_webdocwf"))
            tempSet="webdocwf";
        return tempSet;
    }

    /**
     * Set used template set.
     * @param set Template set.
     */
    public static void setTemplateSet(String set) {
        templateSet = set;
    }

    /**
     * Get used extensions (template set).
     * @return extensions (template set).
     */
    public static String getExtensions() {
        String tempSet = null;
        tempSet=System.getProperty("TEMPLATESET");
        if(tempSet==null)
            tempSet="standard";
        return tempSet;
    }

    /**
     * Get force parameter.
     * @return Force parameter.
     */
    public static String getForce() {
        String force=System.getProperty("FORCE");
        return force;
    }


    /**
     * Get install properties file.
     * @return Get dods.properties file.
     */
    public static String getInstallProperties() {

        String dodsHome=null;
        try {
            Properties properties = new Properties();
            String strFileSep = System.getProperty("file.separator");

            dodsHome = System.getProperty("DODS_HOME")+strFileSep+"build"+strFileSep+"dods.properties";
            if(System.getProperty("os.name").toLowerCase().startsWith("win")) {
                if (dodsHome != "/")
                    dodsHome = dodsHome.replace('/', '\\');
            }
            if(dodsHome==null) {
            dodsHome = System.getProperty("ENHYDRA_DIR")+strFileSep+"dods.properties";
            if(System.getProperty("os.name").toLowerCase().startsWith("win")) {
                if (dodsHome != "/")
                    dodsHome = dodsHome.replace('/', '\\');
            }
            }

        }   catch(NullPointerException nullpointerexception)    {
        //throw new BuildException("Could not load the version information.");
        }
        return dodsHome;
    }


    /**
     * Get enhydra root.
     * @return Enhydra root.
     */
    public static String getDODSRoot() {

        String enhHome=null;
        try {
            String strFileSep = System.getProperty("file.separator");

            enhHome = System.getProperty("DODS_HOME");
            if(System.getProperty("os.name").toLowerCase().startsWith("win")) {
                if (enhHome != "/")
                    enhHome = enhHome.replace('\\', '/');
            }
            if(enhHome==null) {
                String strInstall = getInstallProperties();
                if(System.getProperty("os.name").toLowerCase().startsWith("win")) {
                    if (System.getProperty("file.separator") != "/")
                        strInstall = strInstall.replace('/', '\\');
                }
                FileInputStream fisInstallProp = new FileInputStream(strInstall);
                Properties properties = new Properties();
                properties.load(fisInstallProp);
                enhHome = properties.getProperty("enhydra.root");
                if(System.getProperty("os.name").toLowerCase().startsWith("win")) {
                    if (enhHome != "/")
                        enhHome = enhHome.replace('/', '\\');
                }

            }
        }   catch(NullPointerException nullpointerexception)    {
           // throw new BuildException("Could not load the version information.");
        }   catch(IOException ioexception) {
            //throw new BuildException("Could not load the version information:" + ioexception.getMessage());
        }
        return enhHome;
    }



    /**
     * Get install properties file.
     * @return Install properties file.
     */
    public static String getInstallPropertiesParam() {

        String enhydraRoot = null;
        String dodsConf = null;
        String strReturn = null;
        try {
            Properties properties = new Properties();
            String strFileSep = System.getProperty("file.separator");

            String strInstall = getInstallProperties();
            if(System.getProperty("os.name").toLowerCase().startsWith("win")) {
                if (strFileSep != "/")
                    strInstall = strInstall.replace('/', '\\');
            }
            FileInputStream fisInstallProp = new FileInputStream(strInstall);
            properties.load(fisInstallProp);

            enhydraRoot = properties.getProperty("enhydra.root");
            if(System.getProperty("os.name").toLowerCase().startsWith("win")) {
                if (enhydraRoot != "/")
                    enhydraRoot = enhydraRoot.replace('/', '\\');
            }
            dodsConf = properties.getProperty("dods.conf");
            if(System.getProperty("os.name").toLowerCase().startsWith("win")) {
                if (dodsConf != "/")
                    dodsConf = dodsConf.replace('/', '\\');
                        }
            fisInstallProp.close();

        }   catch(IOException ioexception) {
            //throw new BuildException("Could not load the version information:" + ioexception.getMessage());
        }   catch(NullPointerException nullpointerexception) {
            //throw new BuildException("Could not load the version information.");
        }
                strReturn = "<property name=\"enhydra.root\" value=\"" + enhydraRoot + "\"/>\n";
                strReturn = strReturn + "<property name=\"dods.conf\" value=\"" + dodsConf + "\"/>\n";
        return strReturn;
    }



    /**
     * Get template directory.
     * @return Template directory.
     */
    public static String getTemplateDir() {

        String templateDir=null;
        try {
            Properties properties = new Properties();
            String strFileSep = System.getProperty("file.separator");

            String strInstall = getInstallProperties();
            if(System.getProperty("os.name").toLowerCase().startsWith("win")) {
                if (strFileSep != "/")
                    strInstall = strInstall.replace('/', '\\');
            }

            FileInputStream fisDodsEjen = new FileInputStream(strInstall);
            properties.load(fisDodsEjen);
            fisDodsEjen.close();
            templateDir = properties.getProperty("template.dir");

            if(System.getProperty("os.name").toLowerCase().startsWith("win")) {
                if (templateDir != "/")
                    templateDir = templateDir.replace('/', '\\');
            }
        }   catch(IOException ioexception) {
            //throw new BuildException("Could not load the version information:" + ioexception.getMessage());
        }   catch(NullPointerException nullpointerexception) {
            // throw new BuildException("Could not load the version information.");
        }
        return templateDir;
    }


    /**
     * Get all template set directories.
     * @return all template set directories.
     */
    public static HashSet getAllTemplateSets() {

        HashSet dirs = new HashSet();
        try {
            File templateDirs = new File(Common.getTemplateDir());
            String[] temp = templateDirs.list((FilenameFilter) new DirectoryNameFilter());
            dirs.add("standard");
            dirs.add("multidb");
            for (int i=0;i<temp.length;i++) {
                dirs.add(temp[i]);
                if (temp[i].equals("webdocwf"))
                    dirs.add("multidb_webdocwf");
            }

        }   catch(NullPointerException nullpointerexception) {
            // throw new BuildException("Could not load the version information.");
        }
        return dirs;
    }




    /**
     *  Transform specify name in capital letters.
     *
     *  @param name Class member name (column table name).
     *
     *  @return Capitalized name.
     */
    static public String capitalizeName(String name) { //throws Exception {

        String otherLetters = name.substring(1);
        return name.toUpperCase(java.util.Locale.ENGLISH).substring(0,1).concat(otherLetters);
    }

    /**
     *  Transform name in upper case name.
     *
     *  @param name Class member name (column table name).
     *
     *  @return upper case name.
     */
    static public String upperCaseName(String name) {
        return name.toUpperCase();
    }

    public static void main(String[] args) {
        showDodsConf();
        System.out.println("Database.OidJdbcType = "+getDodsConfProperty("Database.OidJdbcType","Standard"));
    }

}