/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 */

/*
 *
 * @author    Tanja Jovanovic
 * @version   2.0  07.01.2003.
 *
 */
package org.enhydra.dods.cache;

import java.util.Map;

import com.lutris.dods.builder.generator.dataobject.GenericDO;

/**
 *
 * This class contains data and mechanisms needed for caching data objects and queries.
 *
 */
public interface QueryCache extends CacheAdministration {

    /**
     * Constant for maximal number of DO objects in the cache.
     * Used if application's configuration file doesn't contain information
     * about maximal cache size.
     */
    public final int DEFAULT_MAX_CACHE_SIZE = 10000;

    /**
     * Constant for maximal number of Query objects in the simple query cache.
     */
    public final int DEFAULT_MAX_SIMPLE_QUERY_CACHE_SIZE = 1000;

    /**
     * Constant for maximal number of Query objects in the complex query cache.
     */
    public final int DEFAULT_MAX_COMPLEX_QUERY_CACHE_SIZE = 100;

    /**
     * Returns cache content.
     *
     * @return cache content as <code>Map</code> of DO objects.
     */
    public Map getCacheContent();

    /**
     * @return true if multi caching is used, otherwise false.
     */
    public boolean isMulti();

    /**
     * @param newMulti new value for multi
     */
    public void setMulti(boolean newMulti);
   
    /**
     * Checks wheather cache reconfiguration needs to be done.
     *
     * @return true if cache reconfiguration needs to be done, otherwise false.
     */
    public boolean toReconfigure();

    /**
     *
     *
     */
    public GenericDO addDO(GenericDO newDO);

    /**
     * Remove DO from cache.
     *
     * @param cacheHandle Key of the object that will be removed.
     *
     * @return removed DO object.
     */
    public GenericDO removeDO(GenericDO DO);

    /**
     * Remove DO from cache.
     *
     * @param handle String representation of object Id that will be removed.
     *
     * @return removed DO object.
     */
    public GenericDO removeDO(String handle);

    /**
     *
     */
    public GenericDO updateDO(GenericDO DO);

    /**
     * Deletes DO from simpleQCache and complexQCache
     */
    public GenericDO deleteDO(GenericDO DO);

    public QueryCacheItem newQueryCacheItemInstance(String dbName);

    public QueryCacheItem addSimpleQuery(QueryCacheItem queryItem);

    public QueryCacheItem removeSimpleQuery(QueryCacheItem queryItem);

    public QueryCacheItem addComplexQuery(QueryCacheItem queryItem);

    public QueryCacheItem removeComplexQuery(QueryCacheItem queryItem);

    /**
     * @param handle
     * @return
     */
    public GenericDO getDOByHandle(String handle);

    public QueryResult getSimpleQueryResults(String dbName, String query);

    public QueryResult getComplexQueryResults(String dbName, String query);

    public QueryResult getQueryResults(String dbName, String query);

}

