 /*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 */

 /*
 *
 * @author    Tanja Jovanovic
 * @version   1.0  23.04.2003.
 *
 */


package org.enhydra.dods.cache;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.LinkedHashSet;

import com.lutris.dods.builder.generator.dataobject.GenericDO;

/**
 * 
 * This class is for storing one query and its necessary data, for query array. 
 *
 */
public interface QueryCacheItem {

    /**
     * Returns query id.
     * 
     * @return query id
     */
    public String getQueryId();

    /**
     * Sets unique representation of query object.
     * 
     * @param queryId <code>String</code> that unique represent query.
     */
    public void setQueryId(String queryId);
    
    /**
     * Returns OIds (array of data objects which are the results of the query).
     *
     * @return query array of data objects which are the results of the query
     */
    public HashSet getOIds ();

    /**
     * Sets OIds (array of data objects which are the results of the query).
     *
     * @parma OIds query array of data objects which are the results of the query.
     */
    public void setOIds (LinkedHashSet OIds);

    /**
     * Returns time needed for query execution. 
     * 
     * @return time needed for query execution
     */
    public int getTime ();

    /**
     * Sets time needed for query execution. 
     * 
     * @param time time needed for query execution
     */
    public void setTime(int time);
    
    /**
     * Returns array of conditions conds.
     * 
     * @return array of conditions
     */
    public ArrayList getConds();
    
    /**
     * Sets array of conditions.
     * 
     * @param conds array of conditions
     */
    public void setConds(ArrayList conds);

    /**
     * Add condition to query.
     * 
     * @param cond <code>Condition</code> that will be added to query.
     */
    public void addCond(Condition cond);

    /**
     * Returns query database.
     *  
     * @return query database
     */
    public String getOriginDatabase();
    
    /**
     * Checks whether data object obj satisfies conditions of this query.
     *
     * @param  obj dataobject for which are checked conditions of this query
     * @return true if data object obj satisfies conditions of this query,
     * otherwise false.
     */
    public boolean checkConditions(GenericDO obj);
    
    /**
     * Inserts data object obj (or updates it if already exists) in array DOs,
     * if it satisfies this query.
     *
     * @param obj data object which may be inserted (or updated) in array DOs 
     *  
     */
    public void update(GenericDO obj);
    
    /**
     * Removes data object obj from array DOs, if present.
     *
     * @param obj data object which will be removed from array DOs 
     */
    public void delete(GenericDO obj); 
    
    /**
     * Adds data object obj to array DOs.
     *
     * @param obj data object which will be added to array DOs 
     */
    public void add(GenericDO obj);
 
}
