<?xml version="1.0" encoding="iso-8859-1"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xslt"
                xmlns:common="org.enhydra.dods.Common"
                xmlns:miu="org.enhydra.dods.xslt.XSLTUtil"
                version="1.0">
    <xsl:output method="text" encoding="iso-8859-1"/>
    <xsl:template name="memberStuff">
    /**
     * Set the <xsl:value-of select="@name"/> to query, with a QueryBuilder comparison operator.
     *
     * @param x The <xsl:value-of select="@name"/> of the <xsl:value-of select="/TABLE/TABLE_NAME"/> to query.
     * @param cmp_op QueryBuilder comparison operator to use.
     *
     * @exception DataObjectException If a database access error occurs.
     * @exception QueryException If comparison operator is inappropriate
     * (e.g. CASE_SENSITIVE_CONTAINS on an integer field).
     */
    <xsl:if test="USED_FOR_QUERY='true'">public</xsl:if><xsl:if test="USED_FOR_QUERY='false'">private</xsl:if> void setQuery<xsl:value-of select="common:capitalizeName(@name)"/>(<xsl:value-of select="miu:adjustJavaType(JAVA_TYPE)"/> x, String cmp_op )
    throws DataObjectException, QueryException {
        // WebDocWf extension for extended wildcard support
        // The following lines have been added:

        Condition cond = new Condition(<xsl:value-of select="/TABLE/CLASS_NAME"/>DO.COLUMN_<xsl:value-of select="common:upperCaseName(@name)"/>,x,cmp_op);
        queryItem.addCond(cond);
        if (cmp_op.equals(QueryBuilder.CASE_INSENSITIVE_MATCH) || cmp_op.equals(QueryBuilder.CASE_SENSITIVE_MATCH) || cmp_op.equals(QueryBuilder.USER_CASE_SENSITIVE_MATCH) || cmp_op.equals(QueryBuilder.USER_CASE_INSENSITIVE_MATCH)) {
            hitDb = true;
        } else {
            // end of WebDocWf extension for extended wildcard support


            if (<xsl:value-of select="/TABLE/CLASS_NAME"/>DO.cache != null &amp;&amp; <xsl:value-of select="/TABLE/CLASS_NAME"/>DO.cache.isFull()) {
                // Remove from cacheHits any DOs that do not meet this
                // setQuery requirement.
                for ( Iterator iter = (new HashSet(cacheHits.values())).iterator(); iter.hasNext();) {
                    <xsl:value-of select="/TABLE/CLASS_NAME"/>DO DO = (<xsl:value-of select="/TABLE/CLASS_NAME"/>DO)iter.next();
                    <xsl:value-of select="miu:adjustJavaType(JAVA_TYPE)"/> m = DO.get<xsl:value-of select="common:capitalizeName(@name)"/>();
                    if(!QueryBuilder.compare( m, x, cmp_op )) {
                        try {
                            String handle = DO.getHandle();
                            cacheHits.remove(handle);
                        } catch (DatabaseManagerException e) {
                            throw new DataObjectException("Error in loading data object's handle.");
                        }
                    }
                }
            }
        }

        // Also prepares the SQL needed to query the database
        // in case there is no cache, or the query involves other tables.
        // WebDocWf patch for correct queries in fully cached objects
        // The following line has been put under comments:
        // if ( partOrLru || hitDb )
        // end of WebDocWf patch for correct queries in fully cached objects

        builder.addWhere( <xsl:value-of select="/TABLE/CLASS_NAME"/>DO.<xsl:value-of select="common:capitalizeName(@name)"/>, x, cmp_op );
    }

    /**
     * Set the <xsl:value-of select="@name"/> to query, with a QueryBuilder comparison operator.
     *
     * @param x The <xsl:value-of select="@name"/> of the <xsl:value-of select="/TABLE/TABLE_NAME"/> to query.
     *
     * @exception DataObjectException If a database access error occurs.
     * @exception QueryException If comparison operator is inappropriate
     * (e.g. CASE_SENSITIVE_CONTAINS on an integer field).
     */
    <xsl:if test="USED_FOR_QUERY='true'">public</xsl:if><xsl:if test="USED_FOR_QUERY='false'">private</xsl:if> void setQuery<xsl:value-of select="common:capitalizeName(@name)"/>(<xsl:value-of select="miu:adjustJavaType(JAVA_TYPE)"/> x)
    throws DataObjectException, QueryException {
         setQuery<xsl:value-of select="common:capitalizeName(@name)"/>(x, QueryBuilder.EQUAL);
    }


    /**
     * Add <xsl:value-of select="@name"/> to the ORDER BY clause.
     * NOTE: The DO cache does not yet support the Order By operation.
     * Using the addOrderBy method forces the query to hit the database.
     *
     * @param direction_flag  True for ascending order, false for descending
     */
    public void addOrderBy<xsl:value-of select="common:capitalizeName(@name)"/>(boolean direction_flag) {
        hitDb = true;
        builder.addOrderByColumn(&quot;<xsl:value-of select="@name"/>&quot;, (direction_flag) ? "ASC" : "DESC");
    }


    /**
     * Add <xsl:value-of select="@name"/> to the ORDER BY clause.  This convenience
     * method assumes ascending order.
     * NOTE: The DO cache does not yet support the Order By operation.
     * Using the addOrderBy method forces the query to hit the database.
     */
    public void addOrderBy<xsl:value-of select="common:capitalizeName(@name)"/>() {
        hitDb = true;
        builder.addOrderByColumn(&quot;<xsl:value-of select="@name"/>&quot;,"ASC");
    }


<xsl:if test="JAVA_TYPE='String'">
    /**
     * Set the <xsl:value-of select="@name"/> to query with a user wildcard string
     *
     * @param x The <xsl:value-of select="@name"/> of the <xsl:value-of select="/TABLE/TABLE_NAME"/> to query with user wildcards
     *
     * @exception DataObjectException If a database access error occurs.
     *
     * @deprecated Use comparison operators instead
     *
     * WebDocWf extension
     *
     */
    <xsl:if test="USED_FOR_QUERY='true'">public</xsl:if><xsl:if test="USED_FOR_QUERY='false'">private</xsl:if> void setUserMatch<xsl:value-of select="common:capitalizeName(@name)"/>( <xsl:value-of select="miu:adjustJavaType(JAVA_TYPE)"/> x )
    throws DataObjectException, QueryException {
        <xsl:value-of select="miu:adjustJavaType(JAVA_TYPE)"/> y = convertUserSearchValue( x );
        setDBMatch<xsl:value-of select="common:capitalizeName(@name)"/>( y );
    }

    /**
     * Set the <xsl:value-of select="@name"/> to query with a DB wildcard string
     *
     * @param x The <xsl:value-of select="@name"/> of the <xsl:value-of select="/TABLE/TABLE_NAME"/> to query with DB wildcards
     *
     * @exception DataObjectException If a database access error occurs.
     *
     * @deprecated Use comparison operators instead
     *
     * WebDocWf extension
     *
     */
    <xsl:if test="USED_FOR_QUERY='true'">public</xsl:if><xsl:if test="USED_FOR_QUERY='false'">private</xsl:if> void setDBMatch<xsl:value-of select="common:capitalizeName(@name)"/>(<xsl:value-of select="miu:adjustJavaType(JAVA_TYPE)"/> x )
    throws DataObjectException, QueryException {
        if (containsWildcards(x) || builder.getUserStringAppendWildcard()) {
            builder.addMatchClause( <xsl:value-of select="/TABLE/CLASS_NAME"/>DO.<xsl:value-of select="common:capitalizeName(@name)"/>, x );
            hitDb = true;
        } else
            setQuery<xsl:value-of select="common:capitalizeName(@name)"/>( x, QueryBuilder.EQUAL );
    }

</xsl:if>
    // end of WebDocWf extension for extended wildcard support
</xsl:template>
</xsl:stylesheet>

