<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xslt"
                xmlns:stu="org.ejen.ext.StringUtil"
                version="1.0">
    <xsl:output method="text" encoding="iso-8859-1"/>
<xsl:template name="fileFooter9">
<xsl:value-of select="STUBS_FOR_COMMIT_ASSERTIONS"/>
    /**
     * Modifies the DO within its table.
     * Performs recursive commit/delete on referenced DOs;
     * all operations occur within a single transaction
     * to allow rollback in the event of error.
     * Only the creator of the transaction releases it.
     *
     * @param dbt The transaction object used for this operation.
     * @param delete True if doing a delete, otherwise false (for insert/update).
     *
     * @exception com.lutris.appserver.server.sql.DatabaseManagerException If a Transaction can not be created.
     * @exception com.lutris.appserver.server.sql.DBRowUpdateException If a version error occurs.
     * @exception RefAssertionException Thrown by okTo method.
     * @exception java.sql.SQLException If any SQL errors occur.
     * 
     * WebDocWf extension
     */
    protected void modifyDO( DBTransaction dbt, boolean delete)
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        modifyDO( dbt, delete, true);
    }

    /**
     * Modifies the DO within its table.
     * Performs recursive commit/delete on referenced DOs;
     * all operations occur within a single transaction
     * to allow rollback in the event of error.
     * Only the creator of the transaction releases it.
     *
     * @param dbt The transaction object used for this operation.
     * @param delete True if doing a delete, otherwise false (for insert/update).
     * @param references True if references should be saved
     *
     * @exception com.lutris.appserver.server.sql.DatabaseManagerException If a Transaction can not be created.
     * @exception com.lutris.appserver.server.sql.DBRowUpdateException If a version error occurs.
     * @exception RefAssertionException Thrown by okTo method.
     * @exception java.sql.SQLException If any SQL errors occur.
     * 
     * WebDocWf extension
     */
    protected void modifyDO( DBTransaction dbt, boolean delete, boolean references )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        if ( delete )
            okToDelete();
        else
            okToCommit();
        boolean ownTransaction = false;
        try {
            if ( null == dbt ) {
                DatabaseManager dbm = DODS.getDatabaseManager();
                dbt = dbm.createTransaction(getOriginDatabase());      // create a transaction
                dbt.setDatabaseName(getOriginDatabase());
                ownTransaction = true;
            } else {
<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">
                if ( !dbt.getDatabaseName().equals(getOriginDatabase()) ) {
                    throw new DatabaseManagerException("Logical database of transaction is different from DO's.");
                }
</xsl:if>
                // WebDocWf fix for circular references
                // The following lines have been inserted
                CoreDO foundDO = dbt.getDO(this);
                if (foundDO!=null) 
                    return;
            }
            // end of WebDocWf fix for circular references
            if ( null == dbt )
                throw new DatabaseManagerException("DatabaseManager.createTransaction returned null." );
            if ( delete ) {
                // Code to perform cascading deletes is generated here
                // if cascading deletes are not supported by the database.      
</xsl:template>
</xsl:stylesheet>
