<?xml version="1.0" encoding="iso-8859-1"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:dodst="org.enhydra.dods.xslt.DODSTag"
                xmlns:common="org.enhydra.dods.Common"
                version="1.0">
        <xsl:output method="text" encoding="iso-8859-1"/>
<xsl:template name="fileHeader">
<xsl:value-of select="dodst:initColumn()"/>
/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * contains WebDocWf extensions
 *
 *-----------------------------------------------------------------------------
 * <xsl:value-of select="PACKAGE"/>/<xsl:value-of select="CLASS_NAME"/>DO.java
 *-----------------------------------------------------------------------------
 */

package <xsl:value-of select="translate(PACKAGE,'/','.')"/>;

import java.io.*;
import java.sql.*;
import java.math.*;
import java.util.Hashtable;
import java.util.Collection;
import java.util.Properties;
import java.io.FileInputStream;

import org.enhydra.dods.DODS;
import com.lutris.util.Config;
import com.lutris.util.ConfigException;
import com.lutris.appserver.server.sql.*;
import com.lutris.dods.builder.generator.dataobject.*;
import com.lutris.dods.builder.generator.query.*;
import org.enhydra.dods.cache.LRUCache;
import org.enhydra.dods.cache.Condition;
import org.enhydra.dods.cache.QueryCacheImpl;
import org.enhydra.dods.cache.QueryCache;
import org.enhydra.dods.cache.QueryResult;
import org.enhydra.dods.cache.CacheAdministration;
import org.enhydra.dods.statistics.Statistics;
import org.enhydra.dods.statistics.TableStatistics;
import org.enhydra.dods.statistics.QueryCacheStatistics;

// WebDocWf extension for DODS row instance security and genric store
import org.webdocwf.dods.access.*;
import org.webdocwf.util.genericstore.GenericStoreDO;
import java.util.Vector;
import java.util.HashSet;
import java.util.Iterator;
// end of WebDocWf extension for DODS row instance security and genric store

/**
 * Data core class, used to set and retrieve the <xsl:value-of select="CLASS_NAME"/>DO information.
 *
 * @version $Revision: 1.6 $
 * @author  <xsl:value-of select="AUTHOR"/>
 * @since   <xsl:value-of select="PROJECT_NAME"/>
 */
<xsl:if test="IS_ABSTRACT='true'">abstract</xsl:if> public class <xsl:value-of select="CLASS_NAME"/>DO extends com.lutris.dods.builder.generator.dataobject.GenericDO implements <xsl:value-of select="CLASS_NAME"/>DOI, java.io.Serializable {
    /**
     * Static final data members name the table and columns for this DO.
     * By using these members with an instance of the QueryBuilder class,
     * an application can perform straight SQL queries while retaining
     * compile-time checking of table and column usage.
     *
     * Example:  List the Cities containing Persons named Bob:
     *
     *    Using straight SQL with QueryBuilder:
     *      Pro: code runs faster because you create fewer objects
     *      Con: code is less clear
     *
     *         Vector fields = new Vector();
     *         fields.addElement( AddressDO.City );
     *         QueryBuilder qb = new QueryBuilder( fields );
     *         qb.addWhere( PersonDO.FirstName, "Bob" );
     *         qb.addWhere( PersonDO.PrimaryKey, AddressDO.Person );
     *         RDBRow row;
     *         while ( null != ( row = qb.getNextRow() ) ) {
     *             String city = row.get( AddressDO.City ).getString();
     *         }
     *
     *    Using Query/DO classes:
     *      Pro: code is (often) clearer
     *      Con: code runs slower because you create more objects
     *
     *         PersonQuery pq = new PersonQuery();
     *         pq.setQueryFirstName( "Bob" );
     *         PersonDO[] bobs = pq.getDOArray();
     *         for ( int i = 0; i &lt; bobs.length; i++ ) {
     *             AddressQuery aq = new AddressQuery();
     *             aq.setQueryPerson( bobs[i] );
     *             AddressDO addr = aq.getNextDO();
     *             String city = addr.getCity();
     *         }
     */
    static public final RDBTable  table = new RDBTable( &quot;<xsl:value-of select="TABLE_NAME"/>&quot; );

    /**
     * Name of the logical database for which DO object was created
     */
    protected String originDatabase = null;

    /**
     * Return the name of the logical database for which DO object was created.
     *
     * @return origin logical database name.
     *
     */
    public String getOriginDatabase() {
        return originDatabase;
    }

    /**
     * Return <xsl:value-of select="TABLE_NAME"/> as the name of the table in the database
     * which contains <xsl:value-of select="CLASS_NAME"/>DO objects.
     * This method overrides CoreDO.getTableName()
     * and is used by CoreDO.executeUpdate() during error handling.
     *
     * @return The database table name.
     *
     * @see CoreDO
     * @author Jay Gunter
     */
    public String getTableName() {
        return &quot;<xsl:value-of select="TABLE_NAME"/>&quot;;
    }
<xsl:if test="DO_IS_OID_BASED='true'">
    /**
     * static final RDBColumn PrimaryKey for use with QueryBuilder.
     * See example above.
     */
    static public final RDBColumn PrimaryKey = new RDBColumn( table, getPrimaryKeyName() );

    /* RDBColumns for <xsl:value-of select="CLASS_NAME"/>DO attributes are defined below. */
</xsl:if>

    /* Using a DO (and its Query class) to access a VIEW instead of a TABLE:
     *
     * A DO (and its Query class) can be used to access a VIEW
     * instead of a TABLE.  The Data Object is created as usual in DODS,
     * but the "create table" SQL command for that DO is not used.
     * Instead, you substitute a "create view" command to create a
     * virtual table in the database; this is often done to provide
     * convenient access to a collection of tables joined together.
     *
     * A VIEW usually does not return "oid" and "version" columns;
     * often (but now always) a VIEW is defined to return the "oid" column
     * for one of the tables joined together in the definition of the VIEW.
     *
     * If notUsingOId is true, <xsl:value-of select="CLASS_NAME"/>DO.createExisting(ResultSet)
     * will NOT invoke the GenericDO(ResultSet) constructor
     * so to avoid attempting to extract the "oid" and "version" columns
     * from the ResultSet.
     */
<xsl:if test="DO_IS_OID_BASED='false'">    static protected final boolean notUsingOId = true;
</xsl:if>
<xsl:if test="DO_IS_OID_BASED='true'">    static protected final boolean notUsingOId = false;
</xsl:if>

    /**
     * A DO class contains a reference to a DataStruct class.
     * This reference can be null (when the data for the DO
     * has not yet been retrieved from the database),
     * allowing a DO object to be a lightweight placeholder
     * until its data is needed.
     */
    private <xsl:value-of select="CLASS_NAME"/>DataStruct data = null;

    /**
     * Return information whether the data for this object has been marked read-only.
     *
     * @return True if the data for this object has been marked read-only.
     *
     */
    public boolean isReadOnly() {
        if ( null == data )
            return false;
        return data.readOnly;
    }

    /**
     * Sets DO's data.
     */
     public void setData (Object data) {
        this.data = (<xsl:value-of select="CLASS_NAME"/>DataStruct)data;
     }

    /**
     * Returns DO's data.
     */
     public Object getData () {
        return data;
     }

    /**
     * Protected constructor. Only derived classes should call it.
     *
     * @param is_view
     *
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     */
    protected <xsl:value-of select="CLASS_NAME"/>DO ( boolean is_view )
    throws ObjectIdException, DatabaseManagerException {
        super( is_view );
        originDatabase = getLogicalDBName();
    }
<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">
    /**
     * Protected constructor. Only derived classes should call it.
     *
     * @param dbName Logical name of the database from which
     * <xsl:value-of select="CLASS_NAME"/>DO object will be created.
     * @param is_view
     *
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     */
    protected <xsl:value-of select="CLASS_NAME"/>DO (String dbName,  boolean is_view)
    throws ObjectIdException, DatabaseManagerException {
        super(dbName,is_view);
        useLogicalDatabase(dbName);
<xsl:if test="DO_IS_OID_BASED='false'">
        data = new <xsl:value-of select="CLASS_NAME"/>DataStruct ();
</xsl:if>
        originDatabase = dbName;
    }
</xsl:if>
    /**
     * Protected constructor.  Only derived classes should call it.
     *
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     */
    protected <xsl:value-of select="CLASS_NAME"/>DO ()
    throws ObjectIdException, DatabaseManagerException {
        super( notUsingOId );
<xsl:if test="DO_IS_OID_BASED='false'">
        data = new <xsl:value-of select="CLASS_NAME"/>DataStruct ();
</xsl:if>
        originDatabase = getLogicalDBName();
    }
<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">
    /**
     * Protected constructor. Only derived classes should call it.
     *
     * @param dbName Logical name of the database from which
     * <xsl:value-of select="CLASS_NAME"/>DO object will be created.
     *
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     */
    protected <xsl:value-of select="CLASS_NAME"/>DO (String dbName)
    throws ObjectIdException, DatabaseManagerException {
        super(dbName,notUsingOId);
        useLogicalDatabase(dbName);
<xsl:if test="DO_IS_OID_BASED='false'">
        data = new <xsl:value-of select="CLASS_NAME"/>DataStruct ();
</xsl:if>
        originDatabase = dbName;
    }
</xsl:if>

    /**
     * isLoaded()
     * Returns information whether object's data is loaded from database.
     * @return true if the data for this object has been retrieved
     * from the database.
     */
    public boolean isLoaded() {
        return ( null != data );
    }
<xsl:if test="DO_IS_OID_BASED='true'">
    /**
     * Load the fields for the DO from the database.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    public void loadData()
    throws SQLException, ObjectIdException, DataObjectException {
        if ( null == data ) {
            data = new <xsl:value-of select="CLASS_NAME"/>DataStruct ();
        }
<xsl:if test="IS_ABSTRACT='false'">
        ObjectId id = getOId();
        if ( null == id )
            return;
        if ( ! isPersistent() )   // DO from createVirgin
            return;

        // DO from createExisting.  Complain if no record in database.
        <xsl:value-of select="CLASS_NAME"/>Query query;
<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">
        try {
            if (originDatabase != null)
                query = new <xsl:value-of select="CLASS_NAME"/>Query (originDatabase);
            else</xsl:if>
                query = new <xsl:value-of select="CLASS_NAME"/>Query ();
<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">
        } catch (DatabaseManagerException e) {
            throw new DataObjectException("INTERNAL ERROR: unexpected DatabaseManagerException" );
        }
</xsl:if>
        query.setQueryOId( id );
        query.requireUniqueInstance();
        <xsl:value-of select="CLASS_NAME"/>DO obj;
        try {
            obj = query.getNextDO();
            if ( null == obj )
                throw new DataObjectException(&quot;<xsl:value-of select="CLASS_NAME"/>DO DO not found for id=&quot; + id );
            makeIdentical(obj);
            setVersion(    obj.getVersion() );
            setNewVersion( obj.getVersion() );
        } catch ( NonUniqueQueryException e ) {
            throw new ObjectIdException( "Duplicate ObjectId" );
        }
</xsl:if>
    }
</xsl:if>

<xsl:if test="DO_IS_OID_BASED='true'">
    /**
     * Load the actual DO data if necessary.
     * Called by get/set methods.
     *
     * @exception DataObjectException If a data access error occurs.
     */
    protected void checkLoad()
    throws DataObjectException {
        if ( null == data )
            try {
                loadData();
            } catch ( Exception e ) {
                throw new DataObjectException(&quot;Unable to load data for <xsl:value-of select="CLASS_NAME"/>DO id=&quot; + getOId() +
                                              &quot;, error = &quot; + e.getMessage() );
            }
    }
</xsl:if>
    /**
     * Protected constructor used by createExisting(ObjectId) above.
     *
     * @param id The ObjectId for the object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   Should never see this exception since GenericDO.ctor(ObjectId)
     *   never accesses the database.
     */
    protected <xsl:value-of select="CLASS_NAME"/>DO( ObjectId id )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException {
        super( id );
        originDatabase = getLogicalDBName();
    }
<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">
    /**
     * Protected constructor used by createExisting(dbName,ObjectId) above.
     *
     * @param dbName Logical name of the database from which
     * <xsl:value-of select="CLASS_NAME"/>DO object will be created.
     * @param id The ObjectId for the object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   Should never see this exception since GenericDO.ctor(ObjectId)
     *   never accesses the database.
     */
    protected <xsl:value-of select="CLASS_NAME"/>DO(String dbName, ObjectId id )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException {
        super(dbName,id);
        useLogicalDatabase(dbName);
        originDatabase = dbName;
    }
</xsl:if>
<xsl:if test="IS_ABSTRACT='false'">

    /**
     * Represents table and cache (if there is caching) statistics.
     */
    protected static Statistics statistics;

    /**
     * Get table statistics.
     *
     * @return Table statistics.
     */
    public static Statistics getStatistics() {
        if (cache != null) {
            ((QueryCacheStatistics)statistics).setCacheType(cache.getCacheType());
            ((QueryCacheStatistics)statistics).setMaxCacheSize(cache.getMaxCacheSize(false));
            ((QueryCacheStatistics)statistics).setCacheSize(cache.getCacheSize());
            ((QueryCacheStatistics)statistics).setComplexQueryCacheSize(cache.getComplexQueryCacheSize());
            ((QueryCacheStatistics)statistics).setMaxComplexQueryCacheSize(cache.getMaxComplexQueryCacheSize(false));
            ((QueryCacheStatistics)statistics).setSimpleQueryCacheSize(cache.getSimpleQueryCacheSize());
            ((QueryCacheStatistics)statistics).setMaxSimpleQueryCacheSize(cache.getMaxSimpleQueryCacheSize(false));
            statistics.stopTime();
        }
        return statistics;
    }

    /**
     * Refresh table statistics.
     */
    public static void refreshStatistics() {
        if (cache != null)
            statistics = new QueryCacheStatistics(&quot;<xsl:value-of select="TABLE_NAME"/>&quot;);
        else
            statistics = new TableStatistics(&quot;<xsl:value-of select="TABLE_NAME"/>&quot;);
    }
<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">
    /**
     * This set contains list of logical databases used by this <xsl:value-of select="CLASS_NAME"/>DO.
     */
    protected static HashSet usedDatabase;

    /**
     * Add logical database in set of used logical databases. Set of this databases is important for
     * refreshing cache in full caching mode. Tables from this databases will be refreshed.
     * This method returns information whether database already exists in set usedDatabase.
     *
     * @return True if database does not already exist in database set, otherwise false.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     */
    public static synchronized boolean useLogicalDatabase(String database)
    throws DatabaseManagerException, ObjectIdException {
        if (usedDatabase == null)
            usedDatabase = new HashSet();
        boolean result = usedDatabase.add(database);
        if (result &amp;&amp; cache != null &amp;&amp; cache.isFull()) {
            try {
                <xsl:value-of select="CLASS_NAME"/>Query query = new <xsl:value-of select="CLASS_NAME"/>Query (database);
                query.hitDatabase();
                <xsl:value-of select="CLASS_NAME"/>DO obj;
                while ( null != ( obj = query.getNextDO() ) ) {
                    addToCache( obj );
                    obj.data.readOnly = true;
                }
            } catch (DataObjectException e) {
                throw new DatabaseManagerException("INTERNAL ERROR: unexpected DataObjectException" );
            } catch ( NonUniqueQueryException ex ) {
                // Since we do not call query.requireUniqueInstance()
                // this should never happen.
                throw new ObjectIdException( "Duplicate ObjectId" );
            }
        }
        return result;
    }

    /**
     * Get all used logical databases.
     *
     * @return Array that contains names of all used logical databases.
     *
     */
    public static String[] getUsedLogicalDatabases() {
        if (usedDatabase != null)
            return (String[])usedDatabase.toArray();
        return null;
    }
</xsl:if>
<xsl:if test="not(TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf')">
    /**
     * Get all used logical databases.
     *
     * @return Array that contains names of all used logical databases.
     *
     */
    public static String[] getUsedLogicalDatabases() {
        String[] str = { getLogicalDBName() };
        return str;
    }</xsl:if>


    protected static QueryCacheImpl cache; // cache for <xsl:value-of select="CLASS_NAME"/>DO

    /**
     * Read cache configuration from application configuration file:
     * cache size for <xsl:value-of select="translate(PACKAGE,'/','.')"/>.<xsl:value-of select="TABLE_NAME"/> table or default cache size.
     */
    public static void readCacheConfiguration() {
        Config tableConfig = null;
        Config cacheConfig = null;
        int maxSize = 0;
        try {
            cacheConfig = (Config)DODS.getDatabaseManager().getConfig().getSection("DB.Cache");
            if (cacheConfig != null) {
                try {
                    tableConfig = cacheConfig.getConfig(&quot;<xsl:value-of select="TABLE_NAME"/>&quot;);
                } catch (ConfigException ex) {
                }
                boolean notFound = false;
                if (tableConfig != null) {
                    try {
                        maxSize = tableConfig.getInt("maxCacheSize");
                    } catch (ConfigException ex) {
                        notFound = true;
                    }
                } else
                    notFound = true;
                if ( notFound ) {
                    try {
                        maxSize = cacheConfig.getInt("defaultMaxCacheSize");
                        notFound = false;
                    } catch (ConfigException ex) {
                        notFound = true;
                    }
                }
                if ( notFound ) {
                    maxSize = QueryCache.DEFAULT_MAX_CACHE_SIZE;
                }
            } else
                maxSize = QueryCache.DEFAULT_MAX_CACHE_SIZE;

        } catch ( Exception e ) {
            maxSize = QueryCache.DEFAULT_MAX_CACHE_SIZE;
        }
        if (maxSize != 0) {
            if (cache == null) {
                cache = new QueryCacheImpl(maxSize);<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">
                cache.setMulti(true);</xsl:if>
            } else
                cache.setMaxCacheSize(maxSize);
            cache.readConfiguration(tableConfig,cacheConfig);
        } else
            cache = null;

//        System.out.println("Cache size for "+&quot;<xsl:value-of select="translate(PACKAGE,'/','.')"/>.<xsl:value-of select="TABLE_NAME"/>&quot;+" = "+getMaxCacheSize(true));
    }

    /**
     * Get name of the table that is cached.
     *
     * @return Name of the table that is cached.
     */
    public static String getCacheDodsTableName() {
        return &quot;<xsl:value-of select="TABLE_NAME"/>&quot;;
    }

    /**
     * Get maximum cache size.
     *
     * @return Maximum cache size.
     */
    public static int getMaxCacheSize() {
        if (cache == null)
            return 0;
        return cache.getMaxCacheSize();
    }

    /**
     * Sets maximum cache size.
     *
     * @param maxSize Maximum cache size.
     */
    public static synchronized void setMaxCacheSize(int maxSize) {
        if (maxSize == 0) {
            cache = null;
            refreshStatistics();
            return;
        }
        if (maxSize != 0 &amp;&amp; cache == null) {
            cache = new QueryCacheImpl(maxSize);<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">
            cache.setMulti(true);</xsl:if>
            try {
                refreshCache();
            } catch (Exception e) {
                e.printStackTrace();
            }
        } else {
            cache.setMaxCacheSize(maxSize);
        }
    }

    /**
     * Returns size of currently used cache.
     *
     * @return size of currently used cache.
     */
    public static int getCacheSize() {
        if (cache != null)
            return cache.getCacheSize();
        return 0;
    }

    /**
     * Returns <xsl:value-of select="TABLE_NAME"/> table cache.
     *
     * @return <xsl:value-of select="TABLE_NAME"/> table cache.
     */
    public static CacheAdministration getCache() {
        return cache;
    }

    /**
     * Queries all rows in table, and for each row
     * creates a DO instance in the cache.
     * For these DOs, data.readOnly = true,
     * which causes set methods to throw an exception.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    public static void refreshCache()
    throws java.sql.SQLException, DatabaseManagerException, ObjectIdException, DataObjectException {
        if (cache != null) {
            cache.refresh();
            String querySnt = cache.getInitialQueryCache();
            if (querySnt != null) {
                <xsl:value-of select="CLASS_NAME"/>Query query;
<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">
                if (usedDatabase != null) {
                    try {
                        String database;
                        for (Iterator i = usedDatabase.iterator(); i.hasNext();) {
                            database = (String)i.next();
                            query = new <xsl:value-of select="CLASS_NAME"/>Query (database);
                            query.hitDatabase();
                            if (!querySnt.equals("*") ) {
                                QueryBuilder builder = query.getQueryBuilder();
                                builder.addWhere(querySnt);
                            }
                            <xsl:value-of select="CLASS_NAME"/>DO obj;
                            while ( null != ( obj = query.getNextDO() ) ) {
                                obj.data.readOnly = true;
                            }
                        }
                    } catch (DatabaseManagerException e) {
                        throw new DataObjectException("INTERNAL ERROR: unexpected DatabaseManagerException" );
                    } catch ( NonUniqueQueryException ex ) {
                        // Since we do not call query.requireUniqueInstance()
                        // this should never happen.
                        throw new ObjectIdException( "Duplicate ObjectId" );
                    }
                } else {
</xsl:if>
                    query = new <xsl:value-of select="CLASS_NAME"/>Query ();
                    query.hitDatabase();
                    if (!querySnt.equals("*") ) {
                        QueryBuilder builder = query.getQueryBuilder();
                        builder.addWhere(querySnt);
                    }
                    <xsl:value-of select="CLASS_NAME"/>DO obj;
                    try {
                        if (cache.isFull())
                            while ( null != ( obj = query.getNextDO() ) )
                            obj.data.readOnly = true;
                        else
                            query.getNextDO();
                    } catch ( NonUniqueQueryException ex ) {
                        // Since we do not call query.requireUniqueInstance()
                        // this should never happen.
                        throw new ObjectIdException( "Duplicate ObjectId" );
                    }
<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">
                }</xsl:if>
            }
            refreshStatistics();
        }
    }

    /**
     * Add DO to cache.
     * If DO already exists in cache, just the data member is replaced,
     * so that application references to the DO remain valid.
     * If there  is no caching newDO object is returned.
     *
     * param newDO Data object that will be added to cache.
     *
     * return Data object added to cache.
     */
    public static synchronized <xsl:value-of select="CLASS_NAME"/>DO addToCache( <xsl:value-of select="CLASS_NAME"/>DO newDO ) {
        if (cache == null)
            return newDO;
        <xsl:value-of select="CLASS_NAME"/>DO ret = (<xsl:value-of select="CLASS_NAME"/>DO)cache.addDO(newDO);
        if (ret == null)
            return newDO;
        return ret;
    }

    /**
     * Add DO to cache.
     * If DO already exists in cache, just the data member is replaced,
     * so that application references to the DO remain valid.
     * This method overides method addToCache of the class CoreDO.
     */
    public void addToCache() {
        addToCache(this);
    }

    /**
     */
    public static synchronized <xsl:value-of select="CLASS_NAME"/>DO updateCache( <xsl:value-of select="CLASS_NAME"/>DO newDO ) {
        if (cache == null)
            return newDO;
        <xsl:value-of select="CLASS_NAME"/>DO ret = (<xsl:value-of select="CLASS_NAME"/>DO)cache.updateDO(newDO);
        if (ret == null)
            return newDO;
        return ret;
    }

    /**
     */
    public void updateCache() {
        updateCache(this);
    }

    /**
     */
    public static synchronized <xsl:value-of select="CLASS_NAME"/>DO deleteFromCache( <xsl:value-of select="CLASS_NAME"/>DO DO ) {
        if (cache != null)
            cache.deleteDO(DO);
        return DO;
    }
    /**
     * Remove DO from cache.
     *
     * @param dbName Logical name of the database from which
     * <xsl:value-of select="CLASS_NAME"/>DO object will be removed.
     * @param handle handle of Data object which will be re moved.
     */
    public static synchronized void removeFromCache(String dbName, String handle) {
        if (cache != null) {
            String cacheHandle = ""+handle;
<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">
            cacheHandle = dbName+cacheHandle;</xsl:if>
            cache.removeDO( cacheHandle );
        }
    }

    /**
     */
    public void deleteFromCache() {
        deleteFromCache(this);
    }


    /**
     * Remove DO from cache.
     *
     * @param DO Data object which will be removed.
     *
     * @return Removed DO or null if DO object doesn't exist in the cache.
     */
    public static synchronized <xsl:value-of select="CLASS_NAME"/>DO removeFromCache( <xsl:value-of select="CLASS_NAME"/>DO DO ) {
        <xsl:value-of select="CLASS_NAME"/>DO oldDO = null;
        if (cache != null) {
            try {
                oldDO = <xsl:value-of select="CLASS_NAME"/>DO.createVirgin(<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">DO.getOriginDatabase()</xsl:if>);
                oldDO.setOId(DO.getOId());
                oldDO.setVersion(DO.getVersion());
                oldDO.setData(DO.getData());
                cache.removeDO(DO);
            } catch (Exception e) {
            }
        }
        return oldDO;
    }

    /**
     * Remove DO from cache.
     * This method overides method evict() of the class CoreDO.
     */
    public void evict() {
        if (!isPersistent())
            removeFromCache(this);
    }

    /**
     * Remove DOs from cache.
     *
     * @param DOs Array of DOs which will be removed from cache.
     */
    public static void evict(<xsl:value-of select="CLASS_NAME"/>DO[] DOs) {
        for (int i=0; i&lt;DOs.length; i++)
            DOs[i].evict();
    }

    /**
     * Remove DOs from cache.
     *
     * @param dbName Logical name of the database from which
     * <xsl:value-of select="CLASS_NAME"/>DOs object will be removed.
     * @param handles array of DOs handles that will be removed from cache.
     */
    public static void evict(String dbName, String[] handles) {
        if (handles!=null) {
            for (int i=0; i&lt;handles.length; i++)
                removeFromCache(dbName, handles[i]);
        }
    }

    /**
     * Retrieve contents of the cache as an collection of DO objects.
     *
     * @return Collection of DO objects retrieved from the cache.
     */
    public static Collection getCacheContents() {
        if (cache!=null)
            return cache.getCache().values();
        return null;
    }

    /**
     * Get cache type.
     *
     * @return Cache type.
     */
    public static String getCacheType() {
        if (cache != null)
            return cache.getCacheType();
        return "none";
    }

    /**
     * The cache is used automatically by the class.
     * Callers of the create() methods do not know whether
     * the returned DO instance is shared or not.
     * This prevents the existence of multiple instances in memory
     * of the same database object.
     * The cache is instantiated when the first DO instance is created.
     */
    /**
     * Class that contains unchanging (static) data from the database
     * will have a cache of DOs representing the entire contents of the table.
     */
    static {
        try {
            refreshStatistics();
            readCacheConfiguration();
            refreshCache();
        } catch ( Exception e ) {
            // cannot throw from static block
        }
    }

    /**
     * This method is invoked whenever object needs to be loaded from database.
     *
     * @exception DataObjectException If a data access error occurs.
     */
    public void refresh() throws DataObjectException {
        try {
            loadData();
        } catch ( Exception e ) {
            throw new DataObjectException(&quot;Unable to load data for <xsl:value-of select="CLASS_NAME"/>DO id=&quot; + getOId() +
                                          &quot;, error = &quot; + e.getMessage() );
        }
    }

    /**
     * This method is invoked whenever objects needs to be loaded from database.
     *
     * @param DOs Array of DOs which will be red from database.
     *
     * @exception DataObjectException If a data access error occurs.
     */
    public static void refresh(<xsl:value-of select="CLASS_NAME"/>DO[] DOs) throws DataObjectException {
        for (int i=0; i&lt;DOs.length; i++)
            DOs[i].refresh();
    }

    /**
     * createVirgin()
     * Creates a DO that has no ObjectId or data.
     * Such a DO is used to insert a new database entry
     * after its data has been set.
     *
     * @return Created data object.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     */
    public static <xsl:value-of select="CLASS_NAME"/>DO createVirgin()
    throws DatabaseManagerException, ObjectIdException {
        return new <xsl:value-of select="CLASS_NAME"/>DO ();
    }

</xsl:if>
<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">
    /**
     * createVirgin(String)
     * Creates a DO that has no ObjectId or data.
     * Such a DO is used to insert a new database entry
     * after its data has been set.
     *
     * @param dbName Logical name of the database from which
     * <xsl:value-of select="CLASS_NAME"/>DO object will be created.
     *
     * @return Created data object.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     */
    public static <xsl:value-of select="CLASS_NAME"/>DO createVirgin(String dbName)
    throws DatabaseManagerException, ObjectIdException {
        return new <xsl:value-of select="CLASS_NAME"/>DO (dbName);
    }
</xsl:if>
<xsl:if test="DO_IS_OID_BASED='true'">
    /**
     * createExisting( BigDecimal )
     *
     * Factory method creates a <xsl:value-of select="CLASS_NAME"/>DO object by searching for it
     * in the database using the passed BigDecimal value as the primary key.
     *
     * Creates a DO that represents an existing entry in the database.
     * Such a DO is used to examine and possibly update such an entry.
     * createExisting() is called only from the code that retrieves
     * an ObjectId from a ResultSet (database query result).
     * createExisting() is protected because no other DO or BO should ever
     * need to call it.
     * FIX unfortunately the createExisting(BigDecimal) form *does* need
     * to be public because it is called by the public ctors of other DOs.
     * For example,
     *    AaaDO contains a ref to a BbbDO,
     *    so there is a method AaaDO.setBbb(BbbDO).
     *    In the ctor AaaDO(ResultSet), we have the call
     *       setBbb( BbbDO.createExisting( rs.getBigDecimal( &quot;bbb&quot;, 0 )));
     * Since AaaDO is not in the same package as BbbDO,
     * BbbDO.createExisting(BigDecimal) must be public, not protected.
     * Java needs the C++ 'friend' idea.
     *
     * @param bd The BigDecimal representation of the ObjectId for the object.
     *
     * @return Created <xsl:value-of select="CLASS_NAME"/>DO object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    public static <xsl:value-of select="CLASS_NAME"/>DO createExisting( BigDecimal bd )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException {
        if ( null == bd )
            return null;
        return createExisting( new ObjectId( bd ) );
    }
<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">
    /**
     * createExisting( String, BigDecimal )
     *
     * Factory method creates a <xsl:value-of select="CLASS_NAME"/>DO object by searching for it
     * in the database using the passed BigDecimal value as the primary key.
     *
     * Creates a DO that represents an existing entry in the database.
     * Such a DO is used to examine and possibly update such an entry.
     * createExisting() is called only from the code that retrieves
     * an ObjectId from a ResultSet (database query result).
     * createExisting() is protected because no other DO or BO should ever
     * need to call it.
     * FIX unfortunately the createExisting(BigDecimal) form *does* need
     * to be public because it is called by the public ctors of other DOs.
     * For example,
     *    AaaDO contains a ref to a BbbDO,
     *    so there is a method AaaDO.setBbb(BbbDO).
     *    In the ctor AaaDO(ResultSet), we have the call
     *       setBbb( BbbDO.createExisting( rs.getBigDecimal( &quot;bbb&quot;, 0 )));
     * Since AaaDO is not in the same package as BbbDO,
     * BbbDO.createExisting(BigDecimal) must be public, not protected.
     * Java needs the C++ 'friend' idea.
     *
     * @param dbName Logical name of the database from which
     * <xsl:value-of select="CLASS_NAME"/>DO object will be created.
     * @param bd The BigDecimal representation of the ObjectId for the object.
     *
     * @return Created <xsl:value-of select="CLASS_NAME"/>DO object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    public static <xsl:value-of select="CLASS_NAME"/>DO createExisting( String dbName, BigDecimal bd )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException {
        if ( null == bd )
            return null;
        return createExisting( dbName,new ObjectId( bd ) );
    }
</xsl:if>
    /**
     * createExisting( String )
     *
     * The createExisting method is used to create a &lt;CODE&gt;<xsl:value-of select="CLASS_NAME"/>DO&lt;/CODE&gt;
     * from a string handle.
     *
     * @param handle String representation of the ObjectId for the object.
     *
     * @return Created <xsl:value-of select="CLASS_NAME"/>DO object.
     */
    public static <xsl:value-of select="CLASS_NAME"/>DO createExisting( String handle ) {
        <xsl:value-of select="CLASS_NAME"/>DO ret = null;
        try {
            BigDecimal bd = new BigDecimal( handle );
            ret = createExisting( bd );
        } catch ( Exception e ) {
        }
        return ret;
    }
<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">
    /**
     * createExisting( String, String )
     *
     * The createExisting method is used to create a &lt;CODE&gt;<xsl:value-of select="CLASS_NAME"/>DO&lt;/CODE&gt;
     * from a string handle.
     *
     * @param handle String representation of the ObjectId for the object.
     * @param dbName Logical name of the database from which
     * <xsl:value-of select="CLASS_NAME"/>DO object will be created.
     *
     * @return Created <xsl:value-of select="CLASS_NAME"/>DO object.
     */
    public static <xsl:value-of select="CLASS_NAME"/>DO createExisting( String dbName, String handle ) {
        <xsl:value-of select="CLASS_NAME"/>DO ret = null;
        try {
            BigDecimal bd = new BigDecimal( handle );
            ret = createExisting(dbName,bd);
        } catch ( Exception e ) {
        }
        return ret;
    }
</xsl:if>
    /**
     * createExisting( ObjectId )
     *
     * Factory method creates a <xsl:value-of select="CLASS_NAME"/>DO object by searching for it
     * in the database using the passed ObjectID value as the primary key.
     *
     * @param id The ObjectId for the object.
     *
     * @return Created <xsl:value-of select="CLASS_NAME"/>DO object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected static <xsl:value-of select="CLASS_NAME"/>DO createExisting( ObjectId id )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException {
        if ( null == id )
            return null;
        <xsl:value-of select="CLASS_NAME"/>DO ret = null;
<xsl:if test="not(CACHING='none' or DO_IS_OID_BASED='false')">
        String cacheHandle = id.toString();
<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">
        cacheHandle = getLogicalDBName()+"."+cacheHandle;
</xsl:if>
        ret = findCachedObjectByHandle( cacheHandle );
        if ( null != ret )
            return ret;
</xsl:if>
        ret = new <xsl:value-of select="CLASS_NAME"/>DO( id );
        ret.setPersistent( true );  // mark DO as persistent (preexisting)
        if ( ! <xsl:value-of select="IS_LAZY_LOADING"/> ) { // If not lazy-loading, fetch DO data now.
            ret.loadData();
        } else {
            statistics.incrementLazyLoadingNum();
        }

        // unset the GenericDO.dirty flag.
        ret.markClean();
        return ret;
    }
<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">
    /**
     * createExisting( String, ObjectId )
     *
     * Factory method creates a <xsl:value-of select="CLASS_NAME"/>DO object by searching for it
     * in the database using the passed ObjectID value as the primary key.
     *
     * @param dbName Logical name of the database from which
     * <xsl:value-of select="CLASS_NAME"/>DO object will be created.
     * @param id The ObjectId for the object.
     *
     * @return Created <xsl:value-of select="CLASS_NAME"/>DO object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected static <xsl:value-of select="CLASS_NAME"/>DO createExisting( String dbName, ObjectId id )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException {
        if ( null == id )
            return null;
        <xsl:value-of select="CLASS_NAME"/>DO ret = null;
        String cacheHandle = dbName+"."+id.toString();
        ret = findCachedObjectByHandle( cacheHandle );
        if ( null != ret )
            return ret;
        ret = new <xsl:value-of select="CLASS_NAME"/>DO( dbName,id );
        ret.setPersistent( true );  // mark DO as persistent (preexisting)
        if ( ! <xsl:value-of select="IS_LAZY_LOADING"/> ) { // If not lazy-loading, fetch DO data now.
            ret.loadData();
        } else {
            statistics.incrementLazyLoadingNum();
        }
        // unset the GenericDO.dirty flag.
        ret.markClean();
        return ret;
    }
</xsl:if>
</xsl:if>
    /**
     * createExisting( ResultSet )
     *
     * Factory method used to create an instance of this class to
     * represent a Data Object already existing in the database.
     *
     * @param rs The ResultSet returned by the Query class for
     * an existing Data Object stored in the database.
     *
     * @return Created <xsl:value-of select="CLASS_NAME"/>DO object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected static <xsl:value-of select="CLASS_NAME"/>DO createExisting( ResultSet rs )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException {
        if ( null == rs )
            return null;
        <xsl:value-of select="CLASS_NAME"/>DO ret = null;
        if ( notUsingOId ) {
            ret = new <xsl:value-of select="CLASS_NAME"/>DO ();
            ret.initFromResultSet( rs );
        } else {
            ret = new <xsl:value-of select="CLASS_NAME"/>DO ( rs );
        }
//        ret = addToCache( ret );
        return ret;
    }
<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">
    /**
     * createExisting( String, ResultSet )
     *
     * Factory method used to create an instance of this class to
     * represent a Data Object already existing in the database.
     *
     * @param dbName Logical name of the database from which
     * <xsl:value-of select="CLASS_NAME"/>DO object will be created.
     * @param rs The ResultSet returned by the Query class for
     * an existing Data Object stored in the database.
     *
     * @return Created <xsl:value-of select="CLASS_NAME"/>DO object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected static <xsl:value-of select="CLASS_NAME"/>DO createExisting( String dbName, ResultSet rs )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException {
        if ( null == rs )
            return null;
        <xsl:value-of select="CLASS_NAME"/>DO ret = null;
        if ( notUsingOId ) {
            ret = new <xsl:value-of select="CLASS_NAME"/>DO (dbName);
            ret.initFromResultSet( rs );
        } else {
            ret = new <xsl:value-of select="CLASS_NAME"/>DO ( dbName, rs );
        }
        // WebDocWf fix for thread safe lazyloading
        // The following lines have been put under comments:

        // Uncommented by Tanja Jovanovic. Date: 10.11.2002.
        <xsl:if test="not(CACHING='none' or DO_IS_OID_BASED='false')">
//            ret = addToCache( ret );
        </xsl:if>
        // end of WebDocWf fix for thread safe lazyloading
        return ret;
    }
</xsl:if>
<xsl:if test="DO_IS_OID_BASED='true'">
    /**
     * createExisting( RDBRow )
     *
     * Factory method creates a <xsl:value-of select="CLASS_NAME"/>DO object by searching for it
     * in the database using the <xsl:value-of select="CLASS_NAME"/>DO.PrimaryKey value
     * in the passed RDBRow.
     *
     * @param RDBRow A row returned by QueryBuilder.getNextRow().
     *
     * @return Created <xsl:value-of select="CLASS_NAME"/>DO object.
     *
     * @exception DataObjectException
     *   If the RDBRow does not contain a <xsl:value-of select="CLASS_NAME"/>DO.PrimaryKey.
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected static <xsl:value-of select="CLASS_NAME"/>DO createExisting( RDBRow row )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException {
        if ( null == row )
            return null;
        RDBColumnValue pk = null;
        try {
            pk = row.get( <xsl:value-of select="CLASS_NAME"/>DO.PrimaryKey );
            return createExisting( pk );
        } catch ( Exception e ) {
            throw new DataObjectException(&quot;Cannot create <xsl:value-of select="CLASS_NAME"/>DO, row does not &quot; +
                                          &quot;contain <xsl:value-of select="CLASS_NAME"/>DO primary key.&quot; );
        }
    }
<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">
    /**
     * createExisting( String, RDBRow )
     *
     * Factory method creates a <xsl:value-of select="CLASS_NAME"/>DO object by searching for it
     * in the database using the <xsl:value-of select="CLASS_NAME"/>DO.PrimaryKey value
     * in the passed RDBRow.
     *
     * @param dbName Logical name of the database from which
     * <xsl:value-of select="CLASS_NAME"/>DO object will be created.
     * @param RDBRow A row returned by QueryBuilder.getNextRow().
     *
     * @return Created <xsl:value-of select="CLASS_NAME"/>DO object.
     *
     * @exception DataObjectException
     *   If the RDBRow does not contain a <xsl:value-of select="CLASS_NAME"/>DO.PrimaryKey.
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected static <xsl:value-of select="CLASS_NAME"/>DO createExisting( String dbName, RDBRow row )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException {
        if ( null == row )
            return null;
        RDBColumnValue pk = null;
        try {
            pk = row.get( <xsl:value-of select="CLASS_NAME"/>DO.PrimaryKey );
            return createExisting( dbName,pk );
        } catch ( Exception e ) {
            throw new DataObjectException(&quot;Cannot create <xsl:value-of select="CLASS_NAME"/>DO, row does not &quot; +
                                          &quot;contain <xsl:value-of select="CLASS_NAME"/>DO primary key.&quot; );
        }
    }
</xsl:if>
    /**
     * createExisting( RDBColumnValue )
     *
     * Factory method creates a <xsl:value-of select="CLASS_NAME"/>DO object by searching for it
     * in the database using the passed <xsl:value-of select="CLASS_NAME"/>DO.PrimaryKey.
     *
     * @param RDBColumnValue a PrimaryKey column value from a row
     * that was returned by QueryBuilder.getNextRow().
     *
     * @return Created <xsl:value-of select="CLASS_NAME"/>DO object.
     *
     * @exception DataObjectException
     *   If the RDBColumnValue does not contain a <xsl:value-of select="CLASS_NAME"/>DO.PrimaryKey.
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected static <xsl:value-of select="CLASS_NAME"/>DO createExisting( RDBColumnValue pk )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException {
        if ( null == pk )
            return null;
        if ( ! pk.equals( <xsl:value-of select="CLASS_NAME"/>DO.PrimaryKey ) )
            throw new DataObjectException(&quot;Cannot create <xsl:value-of select="CLASS_NAME"/>DO, &quot; +
                                          &quot;RDBColumnValue is not <xsl:value-of select="CLASS_NAME"/>DO.PrimaryKey.&quot; );
        BigDecimal bd = null;
        try {
            bd = pk.getBigDecimal();
        } catch ( Exception e ) {
            throw new DataObjectException(&quot;Cannot create <xsl:value-of select="CLASS_NAME"/>DO, bad primary key.&quot; );
        }
        if ( null == bd )
            return null;
        return createExisting( bd );
    }
<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">
    /**
     * createExisting( String, RDBColumnValue )
     *
     * Factory method creates a <xsl:value-of select="CLASS_NAME"/>DO object by searching for it
     * in the database using the passed <xsl:value-of select="CLASS_NAME"/>DO.PrimaryKey.
     *
     * @param dbName Logical name of the database from which
     * <xsl:value-of select="CLASS_NAME"/>DO object will be created.
     * @param RDBColumnValue a PrimaryKey column value from a row
     * that was returned by QueryBuilder.getNextRow().
     *
     * @return Created <xsl:value-of select="CLASS_NAME"/>DO object.
     *
     * @exception DataObjectException
     *   If the RDBColumnValue does not contain a <xsl:value-of select="CLASS_NAME"/>DO.PrimaryKey.
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected static <xsl:value-of select="CLASS_NAME"/>DO createExisting( String dbName, RDBColumnValue pk )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException {
        if ( null == pk )
            return null;
        if ( ! pk.equals( <xsl:value-of select="CLASS_NAME"/>DO.PrimaryKey ) )
            throw new DataObjectException(&quot;Cannot create <xsl:value-of select="CLASS_NAME"/>DO, &quot; +
                                          &quot;RDBColumnValue is not <xsl:value-of select="CLASS_NAME"/>DO.PrimaryKey.&quot; );
        BigDecimal bd = null;
        try {
            bd = pk.getBigDecimal();
        } catch ( Exception e ) {
            throw new DataObjectException(&quot;Cannot create <xsl:value-of select="CLASS_NAME"/>DO, bad primary key.&quot; );
        }
        if ( null == bd )
            return null;
        return createExisting( dbName,bd );
    }
</xsl:if>
</xsl:if>
    /**
     * Creates a DO that has no ObjectId
     * but has a copy of an existing DO's data.
     * Such a DO is used to insert a new database entry
     * that is largely similar to an existing entry.
     *
     * @param data The data struct to copy values from.
     *
     * @return Created <xsl:value-of select="CLASS_NAME"/>DO object.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     */
    public static <xsl:value-of select="CLASS_NAME"/>DO createCopy( <xsl:value-of select="CLASS_NAME"/>DataStruct data )
    throws DatabaseManagerException, ObjectIdException {
        <xsl:value-of select="CLASS_NAME"/>DO ret = new <xsl:value-of select="CLASS_NAME"/>DO ();
        ret.data = ( <xsl:value-of select="CLASS_NAME"/>DataStruct ) data.duplicate();
        return ret;
    }
<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">
    /**
     * Creates a DO that has no ObjectId
     * but has a copy of an existing DO's data.
     * Such a DO is used to insert a new database entry
     * that is largely similar to an existing entry.
     *
     * @param dbName Logical name of the database from which
     * <xsl:value-of select="CLASS_NAME"/>DO object will be created.
     * @param data The data struct to copy values from.
     *
     * @return Created <xsl:value-of select="CLASS_NAME"/>DO object.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     */
    public static <xsl:value-of select="CLASS_NAME"/>DO createCopy( String dbName, <xsl:value-of select="CLASS_NAME"/>DataStruct data )
    throws DatabaseManagerException, ObjectIdException {
        <xsl:value-of select="CLASS_NAME"/>DO ret = new <xsl:value-of select="CLASS_NAME"/>DO (dbName);
        ret.data = ( <xsl:value-of select="CLASS_NAME"/>DataStruct ) data.duplicate();
        return ret;
    }
</xsl:if>
    /**
     * Creates a DO that has no ObjectId
     * but has a copy of an existing DO's data.
     * Such a DO is used to insert a new database entry
     * that is largely similar to an existing entry.
     *
     * @param orig The original DO to copy.
     *
     * @return Created <xsl:value-of select="CLASS_NAME"/>DO object.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     */
    public static <xsl:value-of select="CLASS_NAME"/>DO createCopy( <xsl:value-of select="CLASS_NAME"/>DO orig )
    throws DatabaseManagerException, ObjectIdException {
        <xsl:value-of select="CLASS_NAME"/>DO ret = new <xsl:value-of select="CLASS_NAME"/>DO ();
        if ( null != orig.data ) {
            ret.data = ( <xsl:value-of select="CLASS_NAME"/>DataStruct ) orig.data.duplicate();
        }
        return ret;
    }
<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">
    /**
     * Creates a DO that has no ObjectId
     * but has a copy of an existing DO's data.
     * Such a DO is used to insert a new database entry
     * that is largely similar to an existing entry.
     *
     * @param dbName Logical name of the database from which
     * <xsl:value-of select="CLASS_NAME"/>DO object will be created.
     * @param orig The original DO to copy.
     *
     * @return Created <xsl:value-of select="CLASS_NAME"/>DO object.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object's id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     */
    public static <xsl:value-of select="CLASS_NAME"/>DO createCopy( String dbName, <xsl:value-of select="CLASS_NAME"/>DO orig )
    throws DatabaseManagerException, ObjectIdException {
        <xsl:value-of select="CLASS_NAME"/>DO ret = new <xsl:value-of select="CLASS_NAME"/>DO (dbName);
        if ( null != orig.data ) {
            ret.data = ( <xsl:value-of select="CLASS_NAME"/>DataStruct ) orig.data.duplicate();
        }
        return ret;
    }
</xsl:if>
<xsl:if test="DO_IS_OID_BASED='true'">
    /**
     * Causes the DO to refresh itself from the database
     * the next time a set or get method is called.
     */
    public void reload() {
        data = null;
    }
</xsl:if>
<xsl:if test="DO_IS_OID_BASED='true'">
    /**
     * The methods &lt;CODE&gt;
     *     getHandle
     *     hasMatchingHandle
<xsl:if test="DO_IS_OID_BASED='true'">
     *     findCachedObjectByHandle
</xsl:if>
     * &lt;/CODE&gt; are used by Presentation Objects that need to populate
     * HTML select lists with Data Objects as options.
     * The &lt;CODE&gt;getHandle()&lt;/CODE&gt; method is used
     * to set the value for each option,
     * and the &lt;CODE&gt;hasMatchingHandle()&lt;CODE&gt;
<xsl:if test="DO_IS_OID_BASED='false'">
     * and &lt;CODE&gt;findCachedObjectByHandle()&lt;CODE&gt;
</xsl:if>
     * methods are used to lookup the Data Object when the selection has
     * been made.
     *
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     *
     * @return id of this DO as a string
     *   If an object's id can't be allocated for this object.
     */
    public String  getHandle()
    throws DatabaseManagerException {
        String ret = null;
        if ( null == getOId() )
               throw new DatabaseManagerException( "ID not set " );
        ret = getOId().toString();
        return ret;
    }

    /**
     * Returns cache handle.
     *
     * @return cache handle.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     */
    public String  getCacheHandle()
    throws DatabaseManagerException {
        String ret = getHandle();
<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">
        ret = getOriginDatabase() + "." + ret;
</xsl:if>
        return ret;
    }
</xsl:if>

<xsl:if test="DO_IS_OID_BASED='true'">
   /**
     * Compare string version of the id of this DO and handle.
     *
     * @param handle
     *   &lt;CODE&gt;String&lt;/CODE&gt; version of DO id.
     *
     * @return boolean
     *   True if the string version of the id of this DO matches passed handle.
     *
     * @see getHandle
     */
    public boolean hasMatchingHandle( String handle ) {
        boolean ret = false;
        if ( null == getOId() )
           return false;
        else {
           String thisHnadle = getOId().toString();
           ret = thisHnadle.equals( handle );
        }
        return ret;
    }
</xsl:if>
   /**
     * Get data object with key cacheHandle from the cache.
     *
     * @param cacheHandle
     *   &lt;CODE&gt;String&lt;/CODE&gt; version of
<xsl:if test="TEMPLATE_SET='multidb' or TEMPLATE_SET='multidb_webdocwf'">
     * concatenation of: name of the data object's database, followed by '.', followed by
</xsl:if>
     * data object's id.
     *
     * @return &lt;CODE&gt;<xsl:value-of select="CLASS_NAME"/>DO&lt;/CODE&gt;
     *   Object if one is found in cache, otherwise null.
     *
     * @see getHandle
     */
    public static <xsl:value-of select="CLASS_NAME"/>DO findCachedObjectByHandle( String cacheHandle ){
        if ( null == cacheHandle )
            return null;
        if ( null == cache )
            return null;
        return ( <xsl:value-of select="CLASS_NAME"/>DO ) cache.getDOByHandle( cacheHandle );
    }

    /**
     * Assigns the DataStruct of an existing DO to this DO.
     * Does not duplicate data. Just assigns the reference.
     *
     * @param orig The original DO.
     *
     */
    protected void makeIdentical( <xsl:value-of select="CLASS_NAME"/>DO orig ) {
        super.makeIdentical(orig);
        data = orig.data;
    }

    /**
     * getVersion makes the protected method public in CoreDO.
     *
     * @return Data object's version.
     */
    public int getVersion() {
        return super.getVersion();
    }

    // WebDocWf extension for writeable fully cached objects
    /**
     * Mark the object as read-only.
     *
     * WebDocWf extension
     *
     */
    public void makeReadOnly() {
        if ( null != data )
            data.readOnly=true;
    }

    /**
     * Mark the object as read-write.
     *
     * WebDocWf extension
     */
    public void makeReadWrite() {
        if ( null != data )
            data.readOnly=false;
    }
    // end of WebDocWf extension for writeable fully cached objects
</xsl:template>
</xsl:stylesheet>