/*-----------------------------------------------------------------------------
 * Enhydra Java Application Server
 * Copyright 1997-2000 Lutris Technologies, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *      This product includes Enhydra software developed by Lutris
 *      Technologies, Inc. and its contributors.
 * 4. Neither the name of Lutris Technologies nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY LUTRIS TECHNOLOGIES AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL LUTRIS TECHNOLOGIES OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 * contains WebDocWf extensions
 * 
 *-----------------------------------------------------------------------------
 * C:/develop/gen/org/webdocwf/util/genericstore/data/GenericObjectDO.java
 *-----------------------------------------------------------------------------
 */

package org.webdocwf.util.genericstore.data;

import java.math.BigDecimal;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Vector;

import org.enhydra.dods.DODS;
import org.webdocwf.dods.access.AccessEvalException;
import org.webdocwf.dods.access.AccessException;
import org.webdocwf.dods.access.AccessRightException;
import org.webdocwf.util.genericstore.GenericStoreDO;

import com.lutris.appserver.server.sql.CoreDO;
import com.lutris.appserver.server.sql.DBConnection;
import com.lutris.appserver.server.sql.DBRowUpdateException;
import com.lutris.appserver.server.sql.DBTransaction;
import com.lutris.appserver.server.sql.DatabaseManager;
import com.lutris.appserver.server.sql.DatabaseManagerException;
import com.lutris.appserver.server.sql.ObjectId;
import com.lutris.appserver.server.sql.ObjectIdException;
import com.lutris.dods.builder.generator.query.DataObjectException;
import com.lutris.dods.builder.generator.query.NonUniqueQueryException;
import com.lutris.dods.builder.generator.query.QueryException;
import com.lutris.dods.builder.generator.query.RDBColumn;
import com.lutris.dods.builder.generator.query.RDBColumnValue;
import com.lutris.dods.builder.generator.query.RDBRow;
import com.lutris.dods.builder.generator.query.RDBTable;
import com.lutris.dods.builder.generator.query.RefAssertionException;
// end of WebDocWf extension for DODS row instance security and genric store

/**
 * Data core class, used to set, retrieve the GenericObjectDO information.
 *
 * @version $Revision: 1.2 $
 * @author  administrator
 * @since   org
 */
 public class GenericObjectDO extends org.webdocwf.util.genericstore.GenericStoreDO implements GenericObjectDOI, java.io.Serializable {

    /**
     * static final data members name the table and columns for this DO.
     * By using these members with an instance of the QueryBuilder class,
     * an application can perform straight SQL queries while retaining
     * compile-time checking of table and column usage.
     *
     * Example:  List the Cities containing Persons named Bob:
     *
     *    Using straight SQL with QueryBuilder:
     *      Pro: code runs faster because you create fewer objects
     *      Con: code is less clear
     *
     *         Vector fields = new Vector();
     *         fields.addElement( AddressDO.City );
     *         QueryBuilder qb = new QueryBuilder( fields );
     *         qb.addWhere( PersonDO.FirstName, "Bob" );
     *         qb.addWhere( PersonDO.PrimaryKey, AddressDO.Person );
     *         RDBRow row;
     *         while ( null != ( row = qb.getNextRow() ) ) {
     *             String city = row.get( AddressDO.City ).getString();
     *         }
     *
     *    Using Query/DO classes:
     *      Pro: code is (often) clearer
     *      Con: code runs slower because you create more objects
     *
     *         PersonQuery pq = new PersonQuery();
     *         pq.setQueryFirstName( "Bob" );
     *         PersonDO[] bobs = pq.getDOArray();
     *         for ( int i = 0; i < bobs.length; i++ ) {
     *             AddressQuery aq = new AddressQuery();
     *             aq.setQueryPerson( bobs[i] );
     *             AddressDO addr = aq.getNextDO();
     *             String city = addr.getCity();
     *         }
     */
    static public final RDBTable  table = new RDBTable( "GENERICOBJECT" );

    /**
     * Return GENERICOBJECT as the name of the table in the database
     * which contains GenericObjectDO objects.
     * This method overrides CoreDO.getTableName()
     * and is used by CoreDO.executeUpdate() during error handling.
     *
     * @return the database table name
     * @see CoreDO
     * @author Jay Gunter
     */
// WebDocWf enhancement for public tablename
// The following line has been changed
    public String getTableName() {
// before: protected String getTableName() {
	return "GENERICOBJECT";
    }

   /**
    * static final RDBColumn PrimaryKey for use with QueryBuilder.
    * See example above.
    */
    static public final RDBColumn PrimaryKey = new RDBColumn( table,
					      getPrimaryKeyName() );
    /* RDBColumns for GenericObjectDO attributes are defined below. */

    /* Using a DO (and its Query class) to access a VIEW instead of a TABLE:
     *
     * A DO (and its Query class) can be used to access a VIEW
     * instead of a TABLE.  The Data Object is created as usual in DODS,
     * but the "create table" SQL command for that DO is not used.
     * Instead, you substitute a "create view" command to create a
     * virtual table in the database; this is often done to provide
     * convenient access to a collection of tables joined together.
     *
     * A VIEW usually does not return "oid" and "version" columns;
     * often (but now always) a VIEW is defined to return the "oid" column
     * for one of the tables joined together in the definition of the VIEW.
     * If the isView flag is true, GenericObjectDO.createExisting(ResultSet)
     * will NOT invoke the GenericDO(ResultSet) constructor
     * so to avoid attempting to extract the "oid" and "version" columns
     * from the ResultSet.
     */
    static protected final boolean isView = false;

    /**
     * A DO class contains a reference to a DataStruct class.
     * This reference can be null (when the data for the DO
     * has not yet been retrieved from the database),
     * allowing a DO object to be a lightweight placeholder
     * until its data is needed.
     */
    private GenericObjectDataStruct data = null;

    /**
     * isReadOnly()
     * Returns true if the data for this object has been marked read-only.
     */
    public boolean isReadOnly() {
	if ( null == data ) return false;
	return data.readOnly;
    }

    /**
     * Protected constructor.  Only derived classes should call it.
     *
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     */
    protected GenericObjectDO ( boolean is_view )
    throws ObjectIdException, DatabaseManagerException {
        super( is_view );
    }

    /**
     * Protected constructor.  Only derived classes should call it.
     *
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     */
    protected GenericObjectDO ()
    throws ObjectIdException, DatabaseManagerException {
        super( isView );
    }

    /**
     * isLoaded()
     * Returns true if the data for this objects has been retrieved
     * from the database.
     */
    public boolean isLoaded() {
	return ( null != data );
    }

    /**
     * loadData()
     * Load the fields for the DO from the database.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    public void loadData()
    throws SQLException, ObjectIdException, DataObjectException
    {
	if ( null == data ) {
	    
	    data = new GenericObjectDataStruct ();
	}

	ObjectId id = getOId();
	if ( null == id )
	    return;
	if ( ! isPersistent() )   // DO from createVirgin
	    return;

	// DO from createExisting.  Complain if no record in database.
	GenericObjectQuery query = new GenericObjectQuery ();
	query.setQueryOId( id );
	query.requireUniqueInstance();
	GenericObjectDO obj;
	try {
	    obj = query.getNextDO();
	    if ( null == obj )
		throw new DataObjectException(
		    "GenericObjectDO DO not found for id=" + id );
	    makeIdentical(obj);
	    setVersion(    obj.getVersion() );
	    setNewVersion( obj.getVersion() );

	} catch ( NonUniqueQueryException e ) {
	    throw new ObjectIdException( "Duplicate ObjectId" );
	}
// WebDocWf fix for thread safe lazy loading
// The following lines have been added:
// end of WebDocWf fix for thread safe lazy loading

    }
    /**
     * Load the actual DO data if necessary.
     * Called by get/set methods.
     *
     * @exception DataObjectException If a data access error occurs.
     */
    protected void checkLoad()
    throws DataObjectException {
	if ( null == data )
	    try {
		loadData();
	    } catch ( Exception e ) {
		throw new DataObjectException(
		    "Unable to load data for GenericObjectDO id=" + getOId() +
		    ", error = " + e.getMessage() );
	    }
    }

    /**
     * Protected constructor used by createExisting(ObjectId) above.
     *
     * @param id The ObjectId for the object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   Should never see this exception since GenericDO.ctor(ObjectId)
     *   never accesses the database.
     */
    protected GenericObjectDO( ObjectId id )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException
    {
	super( id );
    }




    /**
     * createVirgin()
     * Creates a DO that has no ObjectId or data.
     * Such a DO is used to insert a new database entry
     * after its data has been set.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     */
    public static GenericObjectDO createVirgin()
    throws DatabaseManagerException, ObjectIdException {
	return new GenericObjectDO ();
    }
// WebDocWf extension for DODS row instance security
// The following lines have been added:
    /**
     * createVirgin()
     * Creates a DO that has no ObjectId or data.
     * Such a DO is used to insert a new database entry
     * after its data has been set.
     *
     * @param usr The user for security check
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception AccessException
     *   The user is not allowed to create new instances
     *
     * WebDocWf extension
     *
     */
    public static GenericObjectDO createVirgin( org.webdocwf.dods.access.User usr )
    throws DatabaseManagerException, ObjectIdException, AccessException {
        assertDOCreateAccess( usr );
	return createVirgin();
    }
    /**
     * Ensure that the given user is allowed to create new instances
     *
     * @param usr The user for security check
     *
     * @exception AccessException
     *   The user is not allowed to create new instances
     *
     * WebDocWf extension
     *
     */
    public static void assertDOCreateAccess( org.webdocwf.dods.access.User usr )
    throws AccessException {
	if (! hasDOCreateAccess( usr )) throw new AccessRightException("No access !", usr, "Get", null, "org.webdocwf.util.genericstore.data.GenericObjectDO", null, null, null, null, null);
    }
    /**
     * Check whether the given user is allowed to create new instances
     *
     * @param usr The user for security check
     * @return Whether the given user is allowed to create new instances
     *
     * WebDocWf extension
     *
     */
    public static boolean hasDOCreateAccess( org.webdocwf.dods.access.User usr )
    throws AccessEvalException
    {
      	return usr.hasDOCreateAccess( "org.webdocwf.util.genericstore.data.GenericObjectDO" );
    }
// end of WebDocWf extension for DODS row instance security

// WebDocWf extension for generic store
// The following lines have been addded
    /**
     * Create an instance for an existing row
     * This is a strange hack for having an instance method
     * Maybe there is a better solution ?
     *
     * WebDocWf extension
     *
     */
    public GenericStoreDO createForExisting( BigDecimal bd )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException
    {
	return createExisting( bd );
    }
// end of WebDocWf extension for generic store

    /**
     * createExisting( BigDecimal )
     *
     * Factory method creates a GenericObjectDO object by searching for it
     * in the database using the passed BigDecimal value as the primary key.
     *
     * Creates a DO that represents an existing entry in the database.
     * Such a DO is used to examine and possibly update such an entry.
     * createExisting() is called only from the code that retrieves
     * an ObjectId from a ResultSet (database query result).
     * createExisting() is protected because no other DO or BO should ever
     * need to call it.
     * FIX unfortunately the createExisting(BigDecimal) form *does* need
     * to be public because it is called by the public ctors of other DOs.
     * For example,
     *    AaaDO contains a ref to a BbbDO,
     *    so there is a method AaaDO.setBbb(BbbDO).
     *    In the ctor AaaDO(ResultSet), we have the call
     *       setBbb( BbbDO.createExisting( rs.getBigDecimal( "bbb", 0 )));
     * Since AaaDO is not in the same package as BbbDO,
     * BbbDO.createExisting(BigDecimal) must be public, not protected.
     * Java needs the C++ 'friend' idea.
     *
     * @param bd The BigDecimal representation of the ObjectId for the object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    public static GenericObjectDO createExisting( BigDecimal bd )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException
    {
	if ( null == bd )
	    return null;
	return createExisting( new ObjectId( bd ) );
    }
// WebDocWf extension for DODS row instance security
// The following lines have been added:
    /**
     * createExisting( BigDecimal )
     *
     * Factory method creates a GenericObjectDO object by searching for it
     * in the database using the passed BigDecimal value as the primary key.
     *
     * @param bd The BigDecimal representation of the ObjectId for the object.
     * @param usr The user for security check
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     * @exception AccessException
     *   The user is not allowed to read the instance existance
     *   
     * WebDocWf extension
     *   
     */
    public static GenericObjectDO createExisting( BigDecimal bd, org.webdocwf.dods.access.User usr )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException, AccessException
    {
	if ( null == bd )
	    return null;
	GenericObjectDO ret = createExisting( new ObjectId( bd ) );
	ret.assertDOGetAccess( usr );
	return ret;
    }
    /**
     * Check whether the given user is allowed to read the DO existance
     * Static function to be called without having the object
     *
     * @param bd The BigDecimal representation of the ObjectId for the object.
     * @param usr The user for security check
     * @return Whether the given user is allowed to read the DO existance
     *   
     * WebDocWf extension
     *   
     */
    public static boolean hasDOGetAccess( BigDecimal bd, org.webdocwf.dods.access.User usr ) 
    throws AccessEvalException
    {
	GenericObjectDO newDO;
	try {
	  newDO = createExisting( new ObjectId( bd ) );
	} catch (Exception e)
	{
	  throw new AccessEvalException("Error in hasDoGetAccess/createExisting !", e, usr, "Get", null, "org.webdocwf.util.genericstore.data.GenericObjectDO", null, null, null, null, null);
	}
	return usr.hasDOGetAccess( newDO );
    }
// end of WebDocWf extension for DODS row instance security

    /**
     * The createExisting method is used to create a <CODE>GenericObjectDO</CODE>
     * from a string handle.
     */
    public static GenericObjectDO createExisting( String handle ) {
	GenericObjectDO ret = null;
        try {
            BigDecimal bd = new BigDecimal( handle );
	    ret = createExisting( bd );
        } catch ( Exception e ) {
        }
	return ret;
    }
// WebDocWf extension for DODS row instance security
// The following lines have been added:
    /**
     * The createExisting method is used to create a <CODE>GenericObjectDO</CODE>
     * from a string handle.
     *   
     * WebDocWf extension
     *   
     */
    public static GenericObjectDO createExisting( String handle, org.webdocwf.dods.access.User usr )
    throws AccessException
    {
	GenericObjectDO ret = createExisting( handle );
        if (ret!=null)
        {
          ret.assertDOGetAccess( usr );
        }
	return ret;
    }
    /**
     * Check whether the given user is allowed to read the DO existance
     * Static function to be called without having the object
     *
     * @param handle The string representation of the ObjectId for the object.
     * @param usr The user for security check
     * @return Whether the given user is allowed to read the DO existance
     *   
     * WebDocWf extension
     *   
     */
    public static boolean hasDOGetAccess( String handle, org.webdocwf.dods.access.User usr ) throws AccessEvalException {
	GenericObjectDO newDO;
	try {
	  newDO = createExisting( handle );
	} catch (Exception e)
	{
	  throw new AccessEvalException("Error in hasDoGetAccess/createExisting !", e, usr, "Get", null, "org.webdocwf.util.genericstore.data.GenericObjectDO", null, null, null, null, null);
	}
	return usr.hasDOGetAccess( newDO );
    }
// end of WebDocWf extension for DODS row instance security

    /**
     * createExisting( ObjectId )
     *
     * Factory method creates a GenericObjectDO object by searching for it
     * in the database using the passed ObjectID value as the primary key.
     *
     * @param id The ObjectId for the object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected static GenericObjectDO createExisting( ObjectId id )
    throws SQLException, ObjectIdException, DataObjectException, DatabaseManagerException
    {
	if ( null == id )
	    return null;
	GenericObjectDO ret = null;
	ret = new GenericObjectDO( id );
	ret.setPersistent( true );  // mark DO as persistent (preexisting)
	if ( ! false ) // If not lazy-loading, fetch DO data now.
	    ret.loadData();
// WebDocWf fix for thread safe lazyloading
// The following lines have been put under comments:
// end of WebDocWf fix for thread safe lazyloading
	// unset the GenericDO.dirty flag.
	ret.markClean();
	return ret;
    }

    /**
     * createExisting( ResultSet )
     *
     * Factory method used to create an instance of this class to
     * represent a Data Object already existing in the database.
     *
     * @param rs The ResultSet returned by the Query class for
     * an existing Data Object stored in the database.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected static GenericObjectDO createExisting( ResultSet rs )
    throws SQLException, ObjectIdException, DataObjectException
		, DatabaseManagerException
    {
	if ( null == rs )
	    return null;
	GenericObjectDO ret = null;
	if ( isView ) {
	    ret = new GenericObjectDO ();
	    ret.initFromResultSet( rs );
	} else {
	    ret = new GenericObjectDO ( rs );
	}
// WebDocWf fix for thread safe lazyloading
// The following lines have been put under comments:
// end of WebDocWf fix for thread safe lazyloading
	return ret;
    }

    /**
     * createExisting( RDBRow )
     *
     * Factory method creates a GenericObjectDO object by searching for it
     * in the database using the GenericObjectDO.PrimaryKey value
     * in the passed RDBRow.
     *
     * @param RDBRow A row returned by QueryBuilder.getNextRow().
     *
     * @exception DataObjectException
     *   If the RDBRow does not contain a GenericObjectDO.PrimaryKey.
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected static GenericObjectDO createExisting( RDBRow row )
    throws SQLException, ObjectIdException, DataObjectException
		, DatabaseManagerException
    {
	if ( null == row )
	    return null;
        RDBColumnValue pk = null;
        try {
	    pk = row.get( GenericObjectDO.PrimaryKey );
	    return createExisting( pk );
        } catch ( Exception e ) {
	    throw new DataObjectException(
		"Cannot create GenericObjectDO, row does not " +
		"contain GenericObjectDO primary key." );
        }
    }

    /**
     * createExisting( RDBColumnValue )
     *
     * Factory method creates a GenericObjectDO object by searching for it
     * in the database using the passed GenericObjectDO.PrimaryKey.
     *
     * @param RDBColumnValue a PrimaryKey column value from a row
     * that was returned by QueryBuilder.getNextRow().
     *
     * @exception DataObjectException
     *   If the RDBColumnValue does not contain a GenericObjectDO.PrimaryKey.
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected static GenericObjectDO createExisting( RDBColumnValue pk )
    throws SQLException, ObjectIdException, DataObjectException
		, DatabaseManagerException
    {
	if ( null == pk )
	    return null;
	if ( ! pk.equals( GenericObjectDO.PrimaryKey ) )
	    throw new DataObjectException(
		"Cannot create GenericObjectDO, " +
		"RDBColumnValue is not GenericObjectDO.PrimaryKey." );
	BigDecimal bd = null;
        try {
	    bd = pk.getBigDecimal();
        } catch ( Exception e ) {
	    throw new DataObjectException(
		"Cannot create GenericObjectDO, bad primary key." );
        }
	if ( null == bd )
            return null;
	return createExisting( bd );
    }

    /**
     * createCopy()
     * Creates a DO that has no ObjectId
     * but has a copy of an existing DO's data.
     * Such a DO is used to insert a new database entry
     * that is largely similar to an existing entry.
     *
     * @param data The data struct to copy values from.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     */
    public static GenericObjectDO createCopy( GenericObjectDataStruct data )
    throws DatabaseManagerException, ObjectIdException {
	GenericObjectDO ret = new GenericObjectDO ();
	ret.data = ( GenericObjectDataStruct ) data.duplicate();
	return ret;
    }

    /**
     * createCopy()
     * Creates a DO that has no ObjectId
     * but has a copy of an existing DO's data.
     * Such a DO is used to insert a new database entry
     * that is largely similar to an existing entry.
     *
     * @param orig The original DO to copy.
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     */
    public static GenericObjectDO createCopy( GenericObjectDO orig )
    throws DatabaseManagerException, ObjectIdException {
	GenericObjectDO ret = new GenericObjectDO ();
	if ( null != orig.data ) {
	    ret.data = ( GenericObjectDataStruct ) orig.data.duplicate();
	}
	return ret;
    }
// WebDocWf extension for DODS row instance security
// The following lines have been added:
    /**
     * createCopy()
     * Creates a DO that has no ObjectId
     * but has a copy of an existing DO's data.
     * Such a DO is used to insert a new database entry
     * that is largely similar to an existing entry.
     *
     * @param orig The original DO to copy.
     * @param usr The user for security check
     *
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception AccessException
     *   The user is not allowed to make a copy
     *
     * WebDocWf extension
     *
     */
    public static GenericObjectDO createCopy( GenericObjectDO orig, org.webdocwf.dods.access.User usr )
    throws DatabaseManagerException, ObjectIdException, AccessException {
        orig.assertDOCopyAccess ( usr );
	GenericObjectDO ret = createCopy ( orig );
	return ret;
    }
// end of WebDocWf extension for DODS row instance security

    /**
     * reload()
     * Causes the DO to refresh itself from the database
     * the next time a set or get method is called.
     */
    public void reload() {
	data = null;
    }


    /**
     * The methods <CODE>
     *     getHandle
     *     hasMatchingHandle
     * </CODE> are used by Presentation Objects that need to populate
     * HTML select lists with Data Objects as options.
     * The <CODE>getHandle()</CODE> method is used
     * to set the value for each option,
     * and the <CODE>hasMatchingHandle()<CODE>
     * methods are used to lookup the Data Object when the selection has
     * been made.
     *
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     *
     * @return id of this DO as a string
     *   If an object id can't be allocated for this object.
     */
    public        String  getHandle()
    throws DatabaseManagerException {
	String ret = null;
	    if ( null == getOId() )
		   throw new DatabaseManagerException( "ID not set" );
	    ret = getOId().toString();
        return ret;
    }

   /**
     * hasMatchingHandle
     *
     * @param handle
     *   <CODE>String</CODE> version of DO id
     * @return boolean
     *   True if the string version of the id of this DO matches passed handle
     */
    public        boolean hasMatchingHandle( String handle ) {
	boolean ret = false;
	    if ( null == getOId() )
		   ret = false;
	    else
		   ret = getOId().toString().equals( handle );
        return ret;
    }



    /**
     * makeIdentical()
     *
     * Assigns the DataStruct of an existing DO to this DO.
     * Does not duplicate data. Just assigns the reference.
     *
     * @param orig The original DO.
     *
     */
    protected void makeIdentical( GenericObjectDO orig ) {
	super.makeIdentical(orig);
	data = orig.data;
    }
// WebDocWf extension for writeable fully cached obejcts
    /**
     * makeReadOnly()
     * Mark the object as readonly
     * 
     * WebDocWf extension
     * 
     */
    public void makeReadOnly() {
	if ( null != data ) data.readOnly=true;
    }
    /**
     * Mark the object as readwrite
     * 
     * WebDocWf extension
     * 
     */
    /**
     * makeReadWrite()
     */
    public void makeReadWrite() {
	if ( null != data ) data.readOnly=false;
    }
// end of WebDocWf extension for writeable fully cached obejcts

////////////////////////// data member EXTERNALOID

   
   /* static final RDBColumn EXTERNALOID for use with QueryBuilder.
    * See RDBColumn PrimaryKey at the top of this file for usage example.
    */
   static public final RDBColumn EXTERNALOID = 
		new RDBColumn( table, "EXTERNALOID", true );

   
   

   /**
    * Get EXTERNALOID of the GENERICOBJECT
    *
    * @return EXTERNALOID of the GENERICOBJECT
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public java.math.BigDecimal getEXTERNALOID () 
   throws DataObjectException {
   
      checkLoad();
   
      return data.EXTERNALOID;
   }
// WebDocWf extension for DODS row instance security
// The following lines have been added
   /**
    * Get EXTERNALOID of the GENERICOBJECT
    *
    * @param usr The user for security check
    * @return EXTERNALOID of the GENERICOBJECT
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    * @exception AccessException
    *   The user is not allowed to read the attribute(value)
    *
    * WebDocWf extension
    *
    */
   public java.math.BigDecimal getEXTERNALOID ( org.webdocwf.dods.access.User usr ) 
   throws DataObjectException, AccessException {
      assertDOGetAttrEXTERNALOIDAccess( usr );
      java.math.BigDecimal ret = getEXTERNALOID();
      assertDOGetDOValueAccess( "org.webdocwf.util.genericstore.data.GenericObjectDO:EXTERNALOID", ret, usr );
      return ret;
   }
   /**
    * Ensure that the given user is allowed to read the attribute
    *
    * @param usr The user for security check
    *
    * @exception AccessException
    *   The user is not allowed to read the attribute(value)
    *
    * WebDocWf extension
    *
    */
   public void assertDOGetAttrEXTERNALOIDAccess( org.webdocwf.dods.access.User usr )
   throws AccessException {
	if (!hasDOGetAttrEXTERNALOIDAccess( usr )) throw new AccessRightException( "No access", usr, "GetAttr", this, null, "EXTERNALOID", null, null, "java.math.BigDecimal", null);
   }

   /**
    * Check whether the given user is allowed to read the attribute
    *
    * @param usr The user for security check
    * @return Whether the given user is allowed to read the attribute
    *
    * WebDocWf extension
    *
    */
   public boolean hasDOGetAttrEXTERNALOIDAccess( org.webdocwf.dods.access.User usr)
   throws AccessEvalException {
	return hasDOGetAttrAccess( "org.webdocwf.util.genericstore.data.GenericObjectDO:EXTERNALOID", data.EXTERNALOID, usr );
   }
   /**
    * Check whether the given user is allowed to read object in a pointer attribute
    *
    * @param usr The user for security check
    * @return Whether the given user is allowed to read the attribute
    *
    * WebDocWf extension
    *
    */
   public boolean hasDOGetDOValueEXTERNALOIDAccess ( org.webdocwf.dods.access.User usr ) 
   throws AccessEvalException {
	java.math.BigDecimal val;
	try {
        val = getEXTERNALOID();
	} catch (Exception e)
	{
	  throw new AccessEvalException("Error in hasDOGetDOValueEXTERNALOIDAccess/getEXTERNALOID !", e, usr, "GetAttr", this, null, "EXTERNALOID", null, null, "java.math.BigDecimal", null);
	}
      return hasDOGetDOValueAccess(val, usr);
   }
// end of WebDocWf extension for DODS row instance security

   
   /**
    * Set EXTERNALOID of the GENERICOBJECT
    *
    * @param EXTERNALOID of the GENERICOBJECT
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setEXTERNALOID ( java.math.BigDecimal EXTERNALOID )
   throws DataObjectException {
      checkLoad();
      data.EXTERNALOID =  markNewValue(
	data.EXTERNALOID, EXTERNALOID  );
   }
// WebDocWf extension for DODS row instance security
// The following lines have been added
   /**
    * Set EXTERNALOID of the GENERICOBJECT
    *
    * @param EXTERNALOID of the GENERICOBJECT
    * @param usr The user for security check
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    * @exception AccessException
    *   The user is not allowed to read the attribute(value)
    *
    * WebDocWf extension
    *
    */
   public void setEXTERNALOID ( java.math.BigDecimal EXTERNALOID, org.webdocwf.dods.access.User usr ) 
   throws DataObjectException, AccessException {
      assertDOSetAttrEXTERNALOIDAccess( EXTERNALOID, usr );
      setEXTERNALOID( EXTERNALOID );
   }
   /**
    * Ensure that the given user is allowed to update the attribute
    *
    * @param EXTERNALOID of the GENERICOBJECT
    * @param usr The user for security check
    *
    * @exception AccessException
    *   The user is not allowed to read the attribute(value)
    *
    * WebDocWf extension
    *
    */
   public void assertDOSetAttrEXTERNALOIDAccess( java.math.BigDecimal EXTERNALOID, org.webdocwf.dods.access.User usr )
   throws AccessException {
	if (!hasDOSetAttrEXTERNALOIDAccess( EXTERNALOID, usr )) throw new AccessRightException( "No access", usr, "SetAttr", this, null, "EXTERNALOID", null, null, "java.math.BigDecimal", null);
   }
   /**
    * Check whether the given user is allowed to update the attribute
    *
    * @param EXTERNALOID of the GENERICOBJECT
    * @param usr The user for security check
    * @return Whether the given user is allowed to update the attribute
    *
    * WebDocWf extension
    *
    */
   public boolean hasDOSetAttrEXTERNALOIDAccess( java.math.BigDecimal EXTERNALOID, org.webdocwf.dods.access.User usr)
   throws AccessEvalException {
	try {
	checkLoad();
	} catch (Exception e)
	{
	  throw new AccessEvalException("Error in hasDOSetAttrEXTERNALOIDAccess/checkLoad !", e, usr, "SetAttr", this, null, "EXTERNALOID", null, null, "java.math.BigDecimal", null);
	}
	return hasDOSetAttrAccess( "org.webdocwf.util.genericstore.data.GenericObjectDO:EXTERNALOID", data.EXTERNALOID,EXTERNALOID,usr );
   }
// end of WebDocWf extension for DODS row instance security
   


////////////////////////// data member TYPEDEFINITIONOID

   
   /* static final RDBColumn TYPEDEFINITIONOID for use with QueryBuilder.
    * See RDBColumn PrimaryKey at the top of this file for usage example.
    */
   static public final RDBColumn TYPEDEFINITIONOID = 
		new RDBColumn( table, "TYPEDEFINITIONOID", true );

   
   

   /**
    * Get TYPEDEFINITIONOID of the GENERICOBJECT
    *
    * @return TYPEDEFINITIONOID of the GENERICOBJECT
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public org.webdocwf.util.genericstore.data.TypeDefinitionDO getTYPEDEFINITIONOID () 
   throws DataObjectException {
   
      checkLoad();
   
      return data.TYPEDEFINITIONOID;
   }
// WebDocWf extension for DODS row instance security
// The following lines have been added
   /**
    * Get TYPEDEFINITIONOID of the GENERICOBJECT
    *
    * @param usr The user for security check
    * @return TYPEDEFINITIONOID of the GENERICOBJECT
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    * @exception AccessException
    *   The user is not allowed to read the attribute(value)
    *
    * WebDocWf extension
    *
    */
   public org.webdocwf.util.genericstore.data.TypeDefinitionDO getTYPEDEFINITIONOID ( org.webdocwf.dods.access.User usr ) 
   throws DataObjectException, AccessException {
      assertDOGetAttrTYPEDEFINITIONOIDAccess( usr );
      org.webdocwf.util.genericstore.data.TypeDefinitionDO ret = getTYPEDEFINITIONOID();
      assertDOGetDOValueAccess( "org.webdocwf.util.genericstore.data.GenericObjectDO:TYPEDEFINITIONOID", ret, usr );
      return ret;
   }
   /**
    * Ensure that the given user is allowed to read the attribute
    *
    * @param usr The user for security check
    *
    * @exception AccessException
    *   The user is not allowed to read the attribute(value)
    *
    * WebDocWf extension
    *
    */
   public void assertDOGetAttrTYPEDEFINITIONOIDAccess( org.webdocwf.dods.access.User usr )
   throws AccessException {
	if (!hasDOGetAttrTYPEDEFINITIONOIDAccess( usr )) throw new AccessRightException( "No access", usr, "GetAttr", this, null, "TYPEDEFINITIONOID", null, null, "org.webdocwf.util.genericstore.data.TypeDefinitionDO", null);
   }

   /**
    * Check whether the given user is allowed to read the attribute
    *
    * @param usr The user for security check
    * @return Whether the given user is allowed to read the attribute
    *
    * WebDocWf extension
    *
    */
   public boolean hasDOGetAttrTYPEDEFINITIONOIDAccess( org.webdocwf.dods.access.User usr)
   throws AccessEvalException {
	return hasDOGetAttrAccess( "org.webdocwf.util.genericstore.data.GenericObjectDO:TYPEDEFINITIONOID", data.TYPEDEFINITIONOID, usr );
   }
   /**
    * Check whether the given user is allowed to read object in a pointer attribute
    *
    * @param usr The user for security check
    * @return Whether the given user is allowed to read the attribute
    *
    * WebDocWf extension
    *
    */
   public boolean hasDOGetDOValueTYPEDEFINITIONOIDAccess ( org.webdocwf.dods.access.User usr ) 
   throws AccessEvalException {
	org.webdocwf.util.genericstore.data.TypeDefinitionDO val;
	try {
        val = getTYPEDEFINITIONOID();
	} catch (Exception e)
	{
	  throw new AccessEvalException("Error in hasDOGetDOValueTYPEDEFINITIONOIDAccess/getTYPEDEFINITIONOID !", e, usr, "GetAttr", this, null, "TYPEDEFINITIONOID", null, null, "org.webdocwf.util.genericstore.data.TypeDefinitionDO", null);
	}
      return hasDOGetDOValueAccess(val, usr);
   }
// end of WebDocWf extension for DODS row instance security

   
   /**
    * Set TYPEDEFINITIONOID of the GENERICOBJECT
    *
    * @param TYPEDEFINITIONOID of the GENERICOBJECT
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    */
   public void setTYPEDEFINITIONOID ( org.webdocwf.util.genericstore.data.TypeDefinitionDO TYPEDEFINITIONOID )
   throws DataObjectException {
      checkLoad();
      data.TYPEDEFINITIONOID = (org.webdocwf.util.genericstore.data.TypeDefinitionDO) markNewValue(
	data.TYPEDEFINITIONOID, TYPEDEFINITIONOID  );
   }
// WebDocWf extension for DODS row instance security
// The following lines have been added
   /**
    * Set TYPEDEFINITIONOID of the GENERICOBJECT
    *
    * @param TYPEDEFINITIONOID of the GENERICOBJECT
    * @param usr The user for security check
    *
    * @exception DataObjectException
    *   If the object is not found in the database.
    * @exception AccessException
    *   The user is not allowed to read the attribute(value)
    *
    * WebDocWf extension
    *
    */
   public void setTYPEDEFINITIONOID ( org.webdocwf.util.genericstore.data.TypeDefinitionDO TYPEDEFINITIONOID, org.webdocwf.dods.access.User usr ) 
   throws DataObjectException, AccessException {
      assertDOSetAttrTYPEDEFINITIONOIDAccess( TYPEDEFINITIONOID, usr );
      setTYPEDEFINITIONOID( TYPEDEFINITIONOID );
   }
   /**
    * Ensure that the given user is allowed to update the attribute
    *
    * @param TYPEDEFINITIONOID of the GENERICOBJECT
    * @param usr The user for security check
    *
    * @exception AccessException
    *   The user is not allowed to read the attribute(value)
    *
    * WebDocWf extension
    *
    */
   public void assertDOSetAttrTYPEDEFINITIONOIDAccess( org.webdocwf.util.genericstore.data.TypeDefinitionDO TYPEDEFINITIONOID, org.webdocwf.dods.access.User usr )
   throws AccessException {
	if (!hasDOSetAttrTYPEDEFINITIONOIDAccess( TYPEDEFINITIONOID, usr )) throw new AccessRightException( "No access", usr, "SetAttr", this, null, "TYPEDEFINITIONOID", null, null, "org.webdocwf.util.genericstore.data.TypeDefinitionDO", null);
   }
   /**
    * Check whether the given user is allowed to update the attribute
    *
    * @param TYPEDEFINITIONOID of the GENERICOBJECT
    * @param usr The user for security check
    * @return Whether the given user is allowed to update the attribute
    *
    * WebDocWf extension
    *
    */
   public boolean hasDOSetAttrTYPEDEFINITIONOIDAccess( org.webdocwf.util.genericstore.data.TypeDefinitionDO TYPEDEFINITIONOID, org.webdocwf.dods.access.User usr)
   throws AccessEvalException {
	try {
	checkLoad();
	} catch (Exception e)
	{
	  throw new AccessEvalException("Error in hasDOSetAttrTYPEDEFINITIONOIDAccess/checkLoad !", e, usr, "SetAttr", this, null, "TYPEDEFINITIONOID", null, null, "org.webdocwf.util.genericstore.data.TypeDefinitionDO", null);
	}
	return hasDOSetAttrAccess( "org.webdocwf.util.genericstore.data.GenericObjectDO:TYPEDEFINITIONOID", data.TYPEDEFINITIONOID,TYPEDEFINITIONOID,usr );
   }
// end of WebDocWf extension for DODS row instance security
   

    /**
     * Protected constructor.
     *
     * @param rs
     *   Result set from which to obtain product data.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception com.lutris.appserver.server.sql.ObjectIdException
     *   If an object id can't be allocated for this object.
     * @exception DatabaseManagerException
     *   If a connection to the database cannot be established, etc.
     * @exception SQLException
     *   If the database rejects the SQL generated to retrieve data
     *   for this object, or if the table contains a bad foreign key, etc.
     */
    protected GenericObjectDO(ResultSet rs)
            throws SQLException, ObjectIdException, DataObjectException
 	    , DatabaseManagerException
    {
        super(rs);
	initFromResultSet( rs );
//	super.init( rs );
    }

    /**
     * initFromResultSet initializes the data members from GENERICOBJECT.
     * This code was separated from the ResultSet constructor
     * so that createExisting(ResultSet) could handle VIEWs.
     */
    private void initFromResultSet( ResultSet rs )
            throws SQLException, ObjectIdException, DataObjectException
 	    , DatabaseManagerException
    {
	// Constructing a DO from a ResultSet means we definitely need the 
	// DataStruct ready for the setXxx methods invoked below.
	data = new GenericObjectDataStruct ();
 
	// writeMemberStuff uses the ResultSetExtraction.template
	// to build up the value for this tag:
	// the value is a series of calls to the DO set methods.

	
	setEXTERNALOID( 
	    
		rs.getBigDecimal( 
// WebDocWf bug fix
// The following line has been changed:
			"EXTERNALOID" )
// before: "EXTERNALOID" <BIG_DECIMAL_SCALE> )
// end of WebDocWf bug fix
	    
	);
	
	
	setTYPEDEFINITIONOID( 
	    org.webdocwf.util.genericstore.data.TypeDefinitionDO.createExisting( 
		rs.getBigDecimal( 
// WebDocWf bug fix
// The following line has been changed:
			"TYPEDEFINITIONOID" )
// before: "TYPEDEFINITIONOID" , 0 )
// end of WebDocWf bug fix
	     )
	);
	

 
        markClean();
    }        


    

    

    private int[] param = null;

    /**
     * Prepares the statement used to insert this object
     * into the database.
     * @param conn the database connection.
     * @return the insert statement.
     * @exception java.sql.SQLException if an error occurs.
     */
    public PreparedStatement getInsertStatement(DBConnection conn)
            throws SQLException {
	/* 
	 * It would probably be better to have CoreDO implement
	 * 	void addToCache(CoreDO DO) {}
	 * and have each DO that has caching enabled override it as
	 *      void addToCache(CoreDO DO) { cache.put( DO.getOId(), DO ); }
	 * and change CoreDO to invoke addToCache()
	 * when it invokes getInsertStatement().
	 */

        ObjectId oid;

        PreparedStatement stmt = conn.prepareStatement( 
	    "insert into GENERICOBJECT ( EXTERNALOID, TYPEDEFINITIONOID, " + getOIdColumnName() + ", " + getVersionColumnName() + " ) values ( ?, ?, ?, ? )" );

	param = new int[1]; param[0] = 1;
	// writeMemberStuff uses the JDBCsetCalls.template
	// to build up the value for this tag:
	// the value is a series of calls to setPrepStmtParam_TYPE methods.
	// Those methods are defined in GenericDO.
	try {
	    setPrepStmtParam_java_math_BigDecimal( stmt, param,
		getEXTERNALOID() );
	    setPrepStmtParam_DO( stmt, param,
		getTYPEDEFINITIONOID() );


	    /* The ObjecId/Version columns form the primary key.  */
	    setPrepStmtParam_BigDecimal( stmt, param, getOId().toBigDecimal() );
	    setPrepStmtParam_int(        stmt, param, getNewVersion() );

	} catch ( Exception e ) {
	    throw new SQLException( "Data Object error: " + e.getMessage() );
	}

        return stmt;
    }

    /**
     * Prepares the statement used to update this object
     * in the database.
     * @param conn the database connection
     * @return the update statement.
     * @exception java.sql.SQLException if an error occurs.
     */
    public PreparedStatement getUpdateStatement(DBConnection conn)
            throws SQLException {

        ObjectId oid;

        PreparedStatement stmt = conn.prepareStatement(
	    "update GENERICOBJECT set " + getVersionColumnName() + " = ?, EXTERNALOID = ?, TYPEDEFINITIONOID = ? " +
	    "where " + getOIdColumnName() + " = ? and " + getVersionColumnName() + " = ?" );

	param = new int[1]; param[0] = 1;
	//int[] param = {1};
	// writeMemberStuff builds up the value for this tag:
	// the value is a series of calls to setPrepStmtParam_TYPE methods.
	// Those methods are defined below.
	try {
	    setPrepStmtParam_int( stmt, param, getNewVersion() );
	    setPrepStmtParam_java_math_BigDecimal( stmt, param,
		getEXTERNALOID() );
	    setPrepStmtParam_DO( stmt, param,
		getTYPEDEFINITIONOID() );


	    /* When updating a persistent object, the UPDATE_WHERE_CLAUSE tag
	     * used to build the SQL WHERE clause (above) uses the 
	     * DO's ObjectId and Version to locate the correct row in
	     * the database table.
	     */
	    setPrepStmtParam_BigDecimal( stmt, param, getOId().toBigDecimal() );
	    setPrepStmtParam_int(        stmt, param, getVersion() );

	} catch ( Exception e ) {
	    throw new SQLException( "Data Object error: " + e.getMessage() );
	}

        return stmt;
    }

    /**
     * Prepares the statement used to delete this object
     * from the database.
     * @param conn the database connection
     * @return the delete statement.
     * @exception java.sql.SQLException if an error occurs.
     */
    public PreparedStatement getDeleteStatement(DBConnection conn)
            throws SQLException {

        String sql =
            "delete from GENERICOBJECT \n" +
            "where " + getOIdColumnName() + " = ?";
        PreparedStatement stmt = conn.prepareStatement(sql);
        stmt.setBigDecimal(1, getOId().toBigDecimal());
        return stmt;
    }

    

    /*
     * toString - for debugging
     */
/*
    public String toString(){
	String str = "GenericObjectDO:";
	ObjectId oid = getOId();
	String id = "virgin";
	if ( null != oid ) 
	    id = oid.toString();
	str += " OID=" + id;
	if ( null != data ) 
	    str = str + "\n" + indent + "EXTERNALOID=" + data.EXTERNALOID
+ "\n" + indent + "TYPEDEFINITIONOID=" + ( null == data.TYPEDEFINITIONOID ? null  : data.TYPEDEFINITIONOID.toString( indentCount + 1 ) )
;
        return str + "; " + super.toString();
    }
*/

    /*
     * toString - for debugging
     */
    public String toString(){
        return toString( 1 );
    }
    public String toString( int indentCount ){
        String indent = "";
        for ( int i = 0; i < indentCount; i++ ) {
            indent += ". ";
        }
        String str = indent + "GenericObjectDO:";
        ObjectId oid = getOId();
        String id = "virgin";
        if ( null != oid )
            id = oid.toString();
        str += " OID=" + id;
        if ( null != data )
            str = str + "\n" + indent + "EXTERNALOID=" + data.EXTERNALOID
+ "\n" + indent + "TYPEDEFINITIONOID=" + ( null == data.TYPEDEFINITIONOID ? null  : data.TYPEDEFINITIONOID.toString( indentCount + 1 ) )
;
        return str + "\n" + indent + "SUPER=" + super.toString( indentCount );
        //return str;
    }

    
    /**
     * Get array of AttributeBoolValueDO objects that refer to this DO.
     *
     * @return array of AttributeBoolValueDO objects.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     */
    public org.webdocwf.util.genericstore.data.AttributeBoolValueDO[] getAttributeBoolValueDOArray () 
    throws DataObjectException, QueryException {
	org.webdocwf.util.genericstore.data.AttributeBoolValueDO[] ret = null;
	try {
	    org.webdocwf.util.genericstore.data.AttributeBoolValueQuery q = new org.webdocwf.util.genericstore.data.AttributeBoolValueQuery();
	    q.setQueryGENERICOBJECTOID( this );
	    ret = q.getDOArray();
	} catch ( NonUniqueQueryException e ) { 
	    throw new DataObjectException( 
		"INTERNAL ERROR: unexpected NonUniqueQueryException" );
	} finally {
	    if ( null == ret )
		ret = new org.webdocwf.util.genericstore.data.AttributeBoolValueDO[ 0 ];
	}
	return ret;
    }

    /**
     * Get the single AttributeBoolValueDO object
     * that refers to this DO.
     *
     * @return AttributeBoolValueDO object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     * @exception NonUniqueQueryException
     *   If more than one AttributeBoolValueDO object was found.
     */
    public org.webdocwf.util.genericstore.data.AttributeBoolValueDO getAttributeBoolValueDO () 
    throws DataObjectException, QueryException, NonUniqueQueryException {
	org.webdocwf.util.genericstore.data.AttributeBoolValueQuery q = new org.webdocwf.util.genericstore.data.AttributeBoolValueQuery();
	q.setQueryGENERICOBJECTOID( this );
	q.requireUniqueInstance();
	return q.getNextDO();
    }

    /**
     * Add (set & commit) a AttributeBoolValueDO object that refers to this DO.
     *
     * @param referrer AttributeBoolValueDO to be set to point to this DO and committed.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void addAttributeBoolValueDO( org.webdocwf.util.genericstore.data.AttributeBoolValueDO referrer )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
// WebDocWf extension for DODS row instance security
// The following line has been changed:
        addAttributeBoolValueDO( referrer, (DBTransaction)null );
// before:        addAttributeBoolValueDO( referrer, null );
// end of WebDocWf extension for DODS row instance security
    }
 
 
    /**
     * Add (set & commit) a AttributeBoolValueDO object that refers to this DO.
     *
     * @param referrer AttributeBoolValueDO to be set to point to this DO and committed.
     *
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void addAttributeBoolValueDO( org.webdocwf.util.genericstore.data.AttributeBoolValueDO referrer, DBTransaction tran )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        referrer.setGENERICOBJECTOID( this );
        referrer.commit( tran );
    }

 
    /**
     * Remove (delete) a AttributeBoolValueDO object that refers to this DO.
     *
     * @param referrer AttributeBoolValueDO to be deleted.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void removeAttributeBoolValueDO( org.webdocwf.util.genericstore.data.AttributeBoolValueDO referrer )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
// WebDocWf extension for DODS row instance security
// The following line has been changed:
        removeAttributeBoolValueDO( referrer, (DBTransaction)null );
// before:        removeAttributeBoolValueDO( referrer, null );
// end of WebDocWf extension for DODS row instance security
    }
 
 
    /**
     * Remove (delete) a AttributeBoolValueDO object that refers to this DO.
     *
     * @param referrer AttributeBoolValueDO to be deleted.
     *
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void removeAttributeBoolValueDO( org.webdocwf.util.genericstore.data.AttributeBoolValueDO referrer, DBTransaction tran )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
	GenericObjectDO referred = referrer.getGENERICOBJECTOID();
	String referredHandle = referred.getHandle();
	String mydoHandle = this.getHandle();
	if ( null == referredHandle || null == mydoHandle || 
	     ( ! referredHandle.equals( mydoHandle ) ) ) {
	    throw new DataObjectException( "Object " + referrer +
		" does not refer to object " + this +
		", cannot be removed this way." );
	}
        referrer.delete( tran );
    }
    // WebDocWF extension for DODS row instance security
    // The following lines have been added:
    /**
     * Get array of AttributeBoolValueDO objects that refer to this DO.
     *
     * @param usr The user for security checks
     * @return array of AttributeBoolValueDO objects.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     * @exception AccessException
     *   The user is not allowed to create a query for 
     *   org.webdocwf.util.genericstore.data.AttributeBoolValue or 
     *   to access the foreign key attribute GENERICOBJECTOID.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public org.webdocwf.util.genericstore.data.AttributeBoolValueDO[] getAttributeBoolValueDOArray ( org.webdocwf.dods.access.User usr ) 
    throws DataObjectException, QueryException, AccessException {
	org.webdocwf.util.genericstore.data.AttributeBoolValueDO[] ret = null;
	try {
	    org.webdocwf.util.genericstore.data.AttributeBoolValueQuery q = new org.webdocwf.util.genericstore.data.AttributeBoolValueQuery( usr );
	    q.setQueryGENERICOBJECTOID( this, usr );
	    ret = q.getDOArray();
	} catch ( NonUniqueQueryException e ) { 
	    throw new DataObjectException( 
		"INTERNAL ERROR: unexpected NonUniqueQueryException" );
	} finally {
	    if ( null == ret )
		ret = new org.webdocwf.util.genericstore.data.AttributeBoolValueDO[ 0 ];
	}
	return ret;
    }
    /**
     * Get the single AttributeBoolValueDO object
     * that refers to this DO.
     *
     * @param usr The user for security checks
     * @return AttributeBoolValueDO object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     * @exception NonUniqueQueryException
     *   If more than one AttributeBoolValueDO object was found.
     * @exception AccessException
     *   The user is not allowed to create a query for 
     *   org.webdocwf.util.genericstore.data.AttributeBoolValue or 
     *   to access the foreign key attribute GENERICOBJECTOID.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public org.webdocwf.util.genericstore.data.AttributeBoolValueDO getAttributeBoolValueDO ( org.webdocwf.dods.access.User usr ) 
    throws DataObjectException, QueryException, NonUniqueQueryException, AccessException {
	org.webdocwf.util.genericstore.data.AttributeBoolValueQuery q = new org.webdocwf.util.genericstore.data.AttributeBoolValueQuery( usr );
	q.setQueryGENERICOBJECTOID( this, usr );
	q.requireUniqueInstance();
	return q.getNextDO();
    }
   /**
     * Add (set & commit) a AttributeBoolValueDO object that refers to this DO.
     *
     * @param referrer AttributeBoolValueDO to be set to point to this DO and committed.
     * @param usr The user for security checks
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void addAttributeBoolValueDO( org.webdocwf.util.genericstore.data.AttributeBoolValueDO referrer, org.webdocwf.dods.access.User usr )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException, AccessException {
        addAttributeBoolValueDO( referrer, null, usr );
    }
 
 
    /**
     * Add (set & commit) a AttributeBoolValueDO object that refers to this DO.
     *
     * @param referrer AttributeBoolValueDO to be set to point to this DO and committed.
     * @param usr The user for security checks
     *
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void addAttributeBoolValueDO( org.webdocwf.util.genericstore.data.AttributeBoolValueDO referrer, DBTransaction tran, org.webdocwf.dods.access.User usr )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException, AccessException {
        referrer.setGENERICOBJECTOID( this, usr );
        referrer.commit( tran );
    }

 
    /**
     * Remove (delete) a AttributeBoolValueDO object that refers to this DO.
     *
     * @param referrer AttributeBoolValueDO to be deleted.
     * @param usr The user for security checks
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void removeAttributeBoolValueDO( org.webdocwf.util.genericstore.data.AttributeBoolValueDO referrer, org.webdocwf.dods.access.User usr )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException, AccessException {
        removeAttributeBoolValueDO( referrer, null, usr );
    }
 
 
    /**
     * Remove (delete) a AttributeBoolValueDO object that refers to this DO.
     *
     * @param referrer AttributeBoolValueDO to be deleted.
     * @param usr The user for security checks
     *
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void removeAttributeBoolValueDO( org.webdocwf.util.genericstore.data.AttributeBoolValueDO referrer, DBTransaction tran, org.webdocwf.dods.access.User usr )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException, AccessException {
	GenericObjectDO referred = referrer.getGENERICOBJECTOID( usr );
	String referredHandle = referred.getHandle( usr );
	String mydoHandle = this.getHandle( usr );
	if ( null == referredHandle || null == mydoHandle || 
	     ( ! referredHandle.equals( mydoHandle ) ) ) {
	    throw new DataObjectException( "Object " + referrer +
		" does not refer to object " + this +
		", cannot be removed this way." );
	}
        referrer.delete( tran, usr );
    }
// end of WebDocWf extension for DODS row instance security

    // WebDocWF extension for DODS row counter
    // The following lines have been added:
    /**
     * Get the number of AttributeBoolValueDOs that refer to this DO.
     * via GENERICOBJECTOID
     *
     * @return The number of objects that refer to this DO
     *
     * @exception DataObjectException If object is not found in the database.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public int getAttributeBoolValueDOArrayCount () 
    throws DataObjectException, QueryException, NonUniqueQueryException, SQLException, DatabaseManagerException {
	int ret = 0;
    org.webdocwf.util.genericstore.data.AttributeBoolValueQuery q = new org.webdocwf.util.genericstore.data.AttributeBoolValueQuery();
    q.setQueryGENERICOBJECTOID( this );
    ret = q.getCount();
	return ret;
    }

    /**
     * Get the number of AttributeBoolValueDOs that refer to this DO.
     * via GENERICOBJECTOID
     *
     * @param usr The user for security checks
     * @return The number of objects that refer to this DO
     *
     * @exception DataObjectException If object is not found in the database.
     * @exception AccessException
     *   The user is not allowed to create a query for 
     *   org.webdocwf.util.genericstore.data.AttributeBoolValue or 
     *   to access the foreign key attribute GENERICOBJECTOID.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public int getAttributeBoolValueDOArrayCount ( org.webdocwf.dods.access.User usr ) 
    throws DataObjectException, QueryException, NonUniqueQueryException, SQLException, DatabaseManagerException, AccessException {
	int ret = 0;
    org.webdocwf.util.genericstore.data.AttributeBoolValueQuery q = new org.webdocwf.util.genericstore.data.AttributeBoolValueQuery(usr);
    q.setQueryGENERICOBJECTOID( this, usr );
    ret = q.getCount();
	return ret;
    }
    // end of WebDocWF extension for DODS row counter

    /**
     * Get array of AttributeDateValueDO objects that refer to this DO.
     *
     * @return array of AttributeDateValueDO objects.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     */
    public org.webdocwf.util.genericstore.data.AttributeDateValueDO[] getAttributeDateValueDOArray () 
    throws DataObjectException, QueryException {
	org.webdocwf.util.genericstore.data.AttributeDateValueDO[] ret = null;
	try {
	    org.webdocwf.util.genericstore.data.AttributeDateValueQuery q = new org.webdocwf.util.genericstore.data.AttributeDateValueQuery();
	    q.setQueryGENERICOBJECTOID( this );
	    ret = q.getDOArray();
	} catch ( NonUniqueQueryException e ) { 
	    throw new DataObjectException( 
		"INTERNAL ERROR: unexpected NonUniqueQueryException" );
	} finally {
	    if ( null == ret )
		ret = new org.webdocwf.util.genericstore.data.AttributeDateValueDO[ 0 ];
	}
	return ret;
    }

    /**
     * Get the single AttributeDateValueDO object
     * that refers to this DO.
     *
     * @return AttributeDateValueDO object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     * @exception NonUniqueQueryException
     *   If more than one AttributeDateValueDO object was found.
     */
    public org.webdocwf.util.genericstore.data.AttributeDateValueDO getAttributeDateValueDO () 
    throws DataObjectException, QueryException, NonUniqueQueryException {
	org.webdocwf.util.genericstore.data.AttributeDateValueQuery q = new org.webdocwf.util.genericstore.data.AttributeDateValueQuery();
	q.setQueryGENERICOBJECTOID( this );
	q.requireUniqueInstance();
	return q.getNextDO();
    }

    /**
     * Add (set & commit) a AttributeDateValueDO object that refers to this DO.
     *
     * @param referrer AttributeDateValueDO to be set to point to this DO and committed.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void addAttributeDateValueDO( org.webdocwf.util.genericstore.data.AttributeDateValueDO referrer )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
// WebDocWf extension for DODS row instance security
// The following line has been changed:
        addAttributeDateValueDO( referrer, (DBTransaction)null );
// before:        addAttributeDateValueDO( referrer, null );
// end of WebDocWf extension for DODS row instance security
    }
 
 
    /**
     * Add (set & commit) a AttributeDateValueDO object that refers to this DO.
     *
     * @param referrer AttributeDateValueDO to be set to point to this DO and committed.
     *
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void addAttributeDateValueDO( org.webdocwf.util.genericstore.data.AttributeDateValueDO referrer, DBTransaction tran )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        referrer.setGENERICOBJECTOID( this );
        referrer.commit( tran );
    }

 
    /**
     * Remove (delete) a AttributeDateValueDO object that refers to this DO.
     *
     * @param referrer AttributeDateValueDO to be deleted.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void removeAttributeDateValueDO( org.webdocwf.util.genericstore.data.AttributeDateValueDO referrer )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
// WebDocWf extension for DODS row instance security
// The following line has been changed:
        removeAttributeDateValueDO( referrer, (DBTransaction)null );
// before:        removeAttributeDateValueDO( referrer, null );
// end of WebDocWf extension for DODS row instance security
    }
 
 
    /**
     * Remove (delete) a AttributeDateValueDO object that refers to this DO.
     *
     * @param referrer AttributeDateValueDO to be deleted.
     *
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void removeAttributeDateValueDO( org.webdocwf.util.genericstore.data.AttributeDateValueDO referrer, DBTransaction tran )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
	GenericObjectDO referred = referrer.getGENERICOBJECTOID();
	String referredHandle = referred.getHandle();
	String mydoHandle = this.getHandle();
	if ( null == referredHandle || null == mydoHandle || 
	     ( ! referredHandle.equals( mydoHandle ) ) ) {
	    throw new DataObjectException( "Object " + referrer +
		" does not refer to object " + this +
		", cannot be removed this way." );
	}
        referrer.delete( tran );
    }
    // WebDocWF extension for DODS row instance security
    // The following lines have been added:
    /**
     * Get array of AttributeDateValueDO objects that refer to this DO.
     *
     * @param usr The user for security checks
     * @return array of AttributeDateValueDO objects.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     * @exception AccessException
     *   The user is not allowed to create a query for 
     *   org.webdocwf.util.genericstore.data.AttributeDateValue or 
     *   to access the foreign key attribute GENERICOBJECTOID.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public org.webdocwf.util.genericstore.data.AttributeDateValueDO[] getAttributeDateValueDOArray ( org.webdocwf.dods.access.User usr ) 
    throws DataObjectException, QueryException, AccessException {
	org.webdocwf.util.genericstore.data.AttributeDateValueDO[] ret = null;
	try {
	    org.webdocwf.util.genericstore.data.AttributeDateValueQuery q = new org.webdocwf.util.genericstore.data.AttributeDateValueQuery( usr );
	    q.setQueryGENERICOBJECTOID( this, usr );
	    ret = q.getDOArray();
	} catch ( NonUniqueQueryException e ) { 
	    throw new DataObjectException( 
		"INTERNAL ERROR: unexpected NonUniqueQueryException" );
	} finally {
	    if ( null == ret )
		ret = new org.webdocwf.util.genericstore.data.AttributeDateValueDO[ 0 ];
	}
	return ret;
    }
    /**
     * Get the single AttributeDateValueDO object
     * that refers to this DO.
     *
     * @param usr The user for security checks
     * @return AttributeDateValueDO object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     * @exception NonUniqueQueryException
     *   If more than one AttributeDateValueDO object was found.
     * @exception AccessException
     *   The user is not allowed to create a query for 
     *   org.webdocwf.util.genericstore.data.AttributeDateValue or 
     *   to access the foreign key attribute GENERICOBJECTOID.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public org.webdocwf.util.genericstore.data.AttributeDateValueDO getAttributeDateValueDO ( org.webdocwf.dods.access.User usr ) 
    throws DataObjectException, QueryException, NonUniqueQueryException, AccessException {
	org.webdocwf.util.genericstore.data.AttributeDateValueQuery q = new org.webdocwf.util.genericstore.data.AttributeDateValueQuery( usr );
	q.setQueryGENERICOBJECTOID( this, usr );
	q.requireUniqueInstance();
	return q.getNextDO();
    }
   /**
     * Add (set & commit) a AttributeDateValueDO object that refers to this DO.
     *
     * @param referrer AttributeDateValueDO to be set to point to this DO and committed.
     * @param usr The user for security checks
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void addAttributeDateValueDO( org.webdocwf.util.genericstore.data.AttributeDateValueDO referrer, org.webdocwf.dods.access.User usr )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException, AccessException {
        addAttributeDateValueDO( referrer, null, usr );
    }
 
 
    /**
     * Add (set & commit) a AttributeDateValueDO object that refers to this DO.
     *
     * @param referrer AttributeDateValueDO to be set to point to this DO and committed.
     * @param usr The user for security checks
     *
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void addAttributeDateValueDO( org.webdocwf.util.genericstore.data.AttributeDateValueDO referrer, DBTransaction tran, org.webdocwf.dods.access.User usr )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException, AccessException {
        referrer.setGENERICOBJECTOID( this, usr );
        referrer.commit( tran );
    }

 
    /**
     * Remove (delete) a AttributeDateValueDO object that refers to this DO.
     *
     * @param referrer AttributeDateValueDO to be deleted.
     * @param usr The user for security checks
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void removeAttributeDateValueDO( org.webdocwf.util.genericstore.data.AttributeDateValueDO referrer, org.webdocwf.dods.access.User usr )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException, AccessException {
        removeAttributeDateValueDO( referrer, null, usr );
    }
 
 
    /**
     * Remove (delete) a AttributeDateValueDO object that refers to this DO.
     *
     * @param referrer AttributeDateValueDO to be deleted.
     * @param usr The user for security checks
     *
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void removeAttributeDateValueDO( org.webdocwf.util.genericstore.data.AttributeDateValueDO referrer, DBTransaction tran, org.webdocwf.dods.access.User usr )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException, AccessException {
	GenericObjectDO referred = referrer.getGENERICOBJECTOID( usr );
	String referredHandle = referred.getHandle( usr );
	String mydoHandle = this.getHandle( usr );
	if ( null == referredHandle || null == mydoHandle || 
	     ( ! referredHandle.equals( mydoHandle ) ) ) {
	    throw new DataObjectException( "Object " + referrer +
		" does not refer to object " + this +
		", cannot be removed this way." );
	}
        referrer.delete( tran, usr );
    }
// end of WebDocWf extension for DODS row instance security

    // WebDocWF extension for DODS row counter
    // The following lines have been added:
    /**
     * Get the number of AttributeDateValueDOs that refer to this DO.
     * via GENERICOBJECTOID
     *
     * @return The number of objects that refer to this DO
     *
     * @exception DataObjectException If object is not found in the database.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public int getAttributeDateValueDOArrayCount () 
    throws DataObjectException, QueryException, NonUniqueQueryException, SQLException, DatabaseManagerException {
	int ret = 0;
    org.webdocwf.util.genericstore.data.AttributeDateValueQuery q = new org.webdocwf.util.genericstore.data.AttributeDateValueQuery();
    q.setQueryGENERICOBJECTOID( this );
    ret = q.getCount();
	return ret;
    }

    /**
     * Get the number of AttributeDateValueDOs that refer to this DO.
     * via GENERICOBJECTOID
     *
     * @param usr The user for security checks
     * @return The number of objects that refer to this DO
     *
     * @exception DataObjectException If object is not found in the database.
     * @exception AccessException
     *   The user is not allowed to create a query for 
     *   org.webdocwf.util.genericstore.data.AttributeDateValue or 
     *   to access the foreign key attribute GENERICOBJECTOID.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public int getAttributeDateValueDOArrayCount ( org.webdocwf.dods.access.User usr ) 
    throws DataObjectException, QueryException, NonUniqueQueryException, SQLException, DatabaseManagerException, AccessException {
	int ret = 0;
    org.webdocwf.util.genericstore.data.AttributeDateValueQuery q = new org.webdocwf.util.genericstore.data.AttributeDateValueQuery(usr);
    q.setQueryGENERICOBJECTOID( this, usr );
    ret = q.getCount();
	return ret;
    }
    // end of WebDocWF extension for DODS row counter

    /**
     * Get array of AttributeEnumValueDO objects that refer to this DO.
     *
     * @return array of AttributeEnumValueDO objects.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     */
    public org.webdocwf.util.genericstore.data.AttributeEnumValueDO[] getAttributeEnumValueDOArray () 
    throws DataObjectException, QueryException {
	org.webdocwf.util.genericstore.data.AttributeEnumValueDO[] ret = null;
	try {
	    org.webdocwf.util.genericstore.data.AttributeEnumValueQuery q = new org.webdocwf.util.genericstore.data.AttributeEnumValueQuery();
	    q.setQueryGENERICOBJECTOID( this );
	    ret = q.getDOArray();
	} catch ( NonUniqueQueryException e ) { 
	    throw new DataObjectException( 
		"INTERNAL ERROR: unexpected NonUniqueQueryException" );
	} finally {
	    if ( null == ret )
		ret = new org.webdocwf.util.genericstore.data.AttributeEnumValueDO[ 0 ];
	}
	return ret;
    }

    /**
     * Get the single AttributeEnumValueDO object
     * that refers to this DO.
     *
     * @return AttributeEnumValueDO object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     * @exception NonUniqueQueryException
     *   If more than one AttributeEnumValueDO object was found.
     */
    public org.webdocwf.util.genericstore.data.AttributeEnumValueDO getAttributeEnumValueDO () 
    throws DataObjectException, QueryException, NonUniqueQueryException {
	org.webdocwf.util.genericstore.data.AttributeEnumValueQuery q = new org.webdocwf.util.genericstore.data.AttributeEnumValueQuery();
	q.setQueryGENERICOBJECTOID( this );
	q.requireUniqueInstance();
	return q.getNextDO();
    }

    /**
     * Add (set & commit) a AttributeEnumValueDO object that refers to this DO.
     *
     * @param referrer AttributeEnumValueDO to be set to point to this DO and committed.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void addAttributeEnumValueDO( org.webdocwf.util.genericstore.data.AttributeEnumValueDO referrer )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
// WebDocWf extension for DODS row instance security
// The following line has been changed:
        addAttributeEnumValueDO( referrer, (DBTransaction)null );
// before:        addAttributeEnumValueDO( referrer, null );
// end of WebDocWf extension for DODS row instance security
    }
 
 
    /**
     * Add (set & commit) a AttributeEnumValueDO object that refers to this DO.
     *
     * @param referrer AttributeEnumValueDO to be set to point to this DO and committed.
     *
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void addAttributeEnumValueDO( org.webdocwf.util.genericstore.data.AttributeEnumValueDO referrer, DBTransaction tran )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        referrer.setGENERICOBJECTOID( this );
        referrer.commit( tran );
    }

 
    /**
     * Remove (delete) a AttributeEnumValueDO object that refers to this DO.
     *
     * @param referrer AttributeEnumValueDO to be deleted.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void removeAttributeEnumValueDO( org.webdocwf.util.genericstore.data.AttributeEnumValueDO referrer )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
// WebDocWf extension for DODS row instance security
// The following line has been changed:
        removeAttributeEnumValueDO( referrer, (DBTransaction)null );
// before:        removeAttributeEnumValueDO( referrer, null );
// end of WebDocWf extension for DODS row instance security
    }
 
 
    /**
     * Remove (delete) a AttributeEnumValueDO object that refers to this DO.
     *
     * @param referrer AttributeEnumValueDO to be deleted.
     *
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void removeAttributeEnumValueDO( org.webdocwf.util.genericstore.data.AttributeEnumValueDO referrer, DBTransaction tran )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
	GenericObjectDO referred = referrer.getGENERICOBJECTOID();
	String referredHandle = referred.getHandle();
	String mydoHandle = this.getHandle();
	if ( null == referredHandle || null == mydoHandle || 
	     ( ! referredHandle.equals( mydoHandle ) ) ) {
	    throw new DataObjectException( "Object " + referrer +
		" does not refer to object " + this +
		", cannot be removed this way." );
	}
        referrer.delete( tran );
    }
    // WebDocWF extension for DODS row instance security
    // The following lines have been added:
    /**
     * Get array of AttributeEnumValueDO objects that refer to this DO.
     *
     * @param usr The user for security checks
     * @return array of AttributeEnumValueDO objects.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     * @exception AccessException
     *   The user is not allowed to create a query for 
     *   org.webdocwf.util.genericstore.data.AttributeEnumValue or 
     *   to access the foreign key attribute GENERICOBJECTOID.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public org.webdocwf.util.genericstore.data.AttributeEnumValueDO[] getAttributeEnumValueDOArray ( org.webdocwf.dods.access.User usr ) 
    throws DataObjectException, QueryException, AccessException {
	org.webdocwf.util.genericstore.data.AttributeEnumValueDO[] ret = null;
	try {
	    org.webdocwf.util.genericstore.data.AttributeEnumValueQuery q = new org.webdocwf.util.genericstore.data.AttributeEnumValueQuery( usr );
	    q.setQueryGENERICOBJECTOID( this, usr );
	    ret = q.getDOArray();
	} catch ( NonUniqueQueryException e ) { 
	    throw new DataObjectException( 
		"INTERNAL ERROR: unexpected NonUniqueQueryException" );
	} finally {
	    if ( null == ret )
		ret = new org.webdocwf.util.genericstore.data.AttributeEnumValueDO[ 0 ];
	}
	return ret;
    }
    /**
     * Get the single AttributeEnumValueDO object
     * that refers to this DO.
     *
     * @param usr The user for security checks
     * @return AttributeEnumValueDO object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     * @exception NonUniqueQueryException
     *   If more than one AttributeEnumValueDO object was found.
     * @exception AccessException
     *   The user is not allowed to create a query for 
     *   org.webdocwf.util.genericstore.data.AttributeEnumValue or 
     *   to access the foreign key attribute GENERICOBJECTOID.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public org.webdocwf.util.genericstore.data.AttributeEnumValueDO getAttributeEnumValueDO ( org.webdocwf.dods.access.User usr ) 
    throws DataObjectException, QueryException, NonUniqueQueryException, AccessException {
	org.webdocwf.util.genericstore.data.AttributeEnumValueQuery q = new org.webdocwf.util.genericstore.data.AttributeEnumValueQuery( usr );
	q.setQueryGENERICOBJECTOID( this, usr );
	q.requireUniqueInstance();
	return q.getNextDO();
    }
   /**
     * Add (set & commit) a AttributeEnumValueDO object that refers to this DO.
     *
     * @param referrer AttributeEnumValueDO to be set to point to this DO and committed.
     * @param usr The user for security checks
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void addAttributeEnumValueDO( org.webdocwf.util.genericstore.data.AttributeEnumValueDO referrer, org.webdocwf.dods.access.User usr )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException, AccessException {
        addAttributeEnumValueDO( referrer, null, usr );
    }
 
 
    /**
     * Add (set & commit) a AttributeEnumValueDO object that refers to this DO.
     *
     * @param referrer AttributeEnumValueDO to be set to point to this DO and committed.
     * @param usr The user for security checks
     *
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void addAttributeEnumValueDO( org.webdocwf.util.genericstore.data.AttributeEnumValueDO referrer, DBTransaction tran, org.webdocwf.dods.access.User usr )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException, AccessException {
        referrer.setGENERICOBJECTOID( this, usr );
        referrer.commit( tran );
    }

 
    /**
     * Remove (delete) a AttributeEnumValueDO object that refers to this DO.
     *
     * @param referrer AttributeEnumValueDO to be deleted.
     * @param usr The user for security checks
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void removeAttributeEnumValueDO( org.webdocwf.util.genericstore.data.AttributeEnumValueDO referrer, org.webdocwf.dods.access.User usr )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException, AccessException {
        removeAttributeEnumValueDO( referrer, null, usr );
    }
 
 
    /**
     * Remove (delete) a AttributeEnumValueDO object that refers to this DO.
     *
     * @param referrer AttributeEnumValueDO to be deleted.
     * @param usr The user for security checks
     *
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void removeAttributeEnumValueDO( org.webdocwf.util.genericstore.data.AttributeEnumValueDO referrer, DBTransaction tran, org.webdocwf.dods.access.User usr )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException, AccessException {
	GenericObjectDO referred = referrer.getGENERICOBJECTOID( usr );
	String referredHandle = referred.getHandle( usr );
	String mydoHandle = this.getHandle( usr );
	if ( null == referredHandle || null == mydoHandle || 
	     ( ! referredHandle.equals( mydoHandle ) ) ) {
	    throw new DataObjectException( "Object " + referrer +
		" does not refer to object " + this +
		", cannot be removed this way." );
	}
        referrer.delete( tran, usr );
    }
// end of WebDocWf extension for DODS row instance security

    // WebDocWF extension for DODS row counter
    // The following lines have been added:
    /**
     * Get the number of AttributeEnumValueDOs that refer to this DO.
     * via GENERICOBJECTOID
     *
     * @return The number of objects that refer to this DO
     *
     * @exception DataObjectException If object is not found in the database.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public int getAttributeEnumValueDOArrayCount () 
    throws DataObjectException, QueryException, NonUniqueQueryException, SQLException, DatabaseManagerException {
	int ret = 0;
    org.webdocwf.util.genericstore.data.AttributeEnumValueQuery q = new org.webdocwf.util.genericstore.data.AttributeEnumValueQuery();
    q.setQueryGENERICOBJECTOID( this );
    ret = q.getCount();
	return ret;
    }

    /**
     * Get the number of AttributeEnumValueDOs that refer to this DO.
     * via GENERICOBJECTOID
     *
     * @param usr The user for security checks
     * @return The number of objects that refer to this DO
     *
     * @exception DataObjectException If object is not found in the database.
     * @exception AccessException
     *   The user is not allowed to create a query for 
     *   org.webdocwf.util.genericstore.data.AttributeEnumValue or 
     *   to access the foreign key attribute GENERICOBJECTOID.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public int getAttributeEnumValueDOArrayCount ( org.webdocwf.dods.access.User usr ) 
    throws DataObjectException, QueryException, NonUniqueQueryException, SQLException, DatabaseManagerException, AccessException {
	int ret = 0;
    org.webdocwf.util.genericstore.data.AttributeEnumValueQuery q = new org.webdocwf.util.genericstore.data.AttributeEnumValueQuery(usr);
    q.setQueryGENERICOBJECTOID( this, usr );
    ret = q.getCount();
	return ret;
    }
    // end of WebDocWF extension for DODS row counter

    /**
     * Get array of AttributeNumberValueDO objects that refer to this DO.
     *
     * @return array of AttributeNumberValueDO objects.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     */
    public org.webdocwf.util.genericstore.data.AttributeNumberValueDO[] getAttributeNumberValueDOArray () 
    throws DataObjectException, QueryException {
	org.webdocwf.util.genericstore.data.AttributeNumberValueDO[] ret = null;
	try {
	    org.webdocwf.util.genericstore.data.AttributeNumberValueQuery q = new org.webdocwf.util.genericstore.data.AttributeNumberValueQuery();
	    q.setQueryGENERICOBJECTOID( this );
	    ret = q.getDOArray();
	} catch ( NonUniqueQueryException e ) { 
	    throw new DataObjectException( 
		"INTERNAL ERROR: unexpected NonUniqueQueryException" );
	} finally {
	    if ( null == ret )
		ret = new org.webdocwf.util.genericstore.data.AttributeNumberValueDO[ 0 ];
	}
	return ret;
    }

    /**
     * Get the single AttributeNumberValueDO object
     * that refers to this DO.
     *
     * @return AttributeNumberValueDO object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     * @exception NonUniqueQueryException
     *   If more than one AttributeNumberValueDO object was found.
     */
    public org.webdocwf.util.genericstore.data.AttributeNumberValueDO getAttributeNumberValueDO () 
    throws DataObjectException, QueryException, NonUniqueQueryException {
	org.webdocwf.util.genericstore.data.AttributeNumberValueQuery q = new org.webdocwf.util.genericstore.data.AttributeNumberValueQuery();
	q.setQueryGENERICOBJECTOID( this );
	q.requireUniqueInstance();
	return q.getNextDO();
    }

    /**
     * Add (set & commit) a AttributeNumberValueDO object that refers to this DO.
     *
     * @param referrer AttributeNumberValueDO to be set to point to this DO and committed.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void addAttributeNumberValueDO( org.webdocwf.util.genericstore.data.AttributeNumberValueDO referrer )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
// WebDocWf extension for DODS row instance security
// The following line has been changed:
        addAttributeNumberValueDO( referrer, (DBTransaction)null );
// before:        addAttributeNumberValueDO( referrer, null );
// end of WebDocWf extension for DODS row instance security
    }
 
 
    /**
     * Add (set & commit) a AttributeNumberValueDO object that refers to this DO.
     *
     * @param referrer AttributeNumberValueDO to be set to point to this DO and committed.
     *
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void addAttributeNumberValueDO( org.webdocwf.util.genericstore.data.AttributeNumberValueDO referrer, DBTransaction tran )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        referrer.setGENERICOBJECTOID( this );
        referrer.commit( tran );
    }

 
    /**
     * Remove (delete) a AttributeNumberValueDO object that refers to this DO.
     *
     * @param referrer AttributeNumberValueDO to be deleted.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void removeAttributeNumberValueDO( org.webdocwf.util.genericstore.data.AttributeNumberValueDO referrer )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
// WebDocWf extension for DODS row instance security
// The following line has been changed:
        removeAttributeNumberValueDO( referrer, (DBTransaction)null );
// before:        removeAttributeNumberValueDO( referrer, null );
// end of WebDocWf extension for DODS row instance security
    }
 
 
    /**
     * Remove (delete) a AttributeNumberValueDO object that refers to this DO.
     *
     * @param referrer AttributeNumberValueDO to be deleted.
     *
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void removeAttributeNumberValueDO( org.webdocwf.util.genericstore.data.AttributeNumberValueDO referrer, DBTransaction tran )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
	GenericObjectDO referred = referrer.getGENERICOBJECTOID();
	String referredHandle = referred.getHandle();
	String mydoHandle = this.getHandle();
	if ( null == referredHandle || null == mydoHandle || 
	     ( ! referredHandle.equals( mydoHandle ) ) ) {
	    throw new DataObjectException( "Object " + referrer +
		" does not refer to object " + this +
		", cannot be removed this way." );
	}
        referrer.delete( tran );
    }
    // WebDocWF extension for DODS row instance security
    // The following lines have been added:
    /**
     * Get array of AttributeNumberValueDO objects that refer to this DO.
     *
     * @param usr The user for security checks
     * @return array of AttributeNumberValueDO objects.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     * @exception AccessException
     *   The user is not allowed to create a query for 
     *   org.webdocwf.util.genericstore.data.AttributeNumberValue or 
     *   to access the foreign key attribute GENERICOBJECTOID.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public org.webdocwf.util.genericstore.data.AttributeNumberValueDO[] getAttributeNumberValueDOArray ( org.webdocwf.dods.access.User usr ) 
    throws DataObjectException, QueryException, AccessException {
	org.webdocwf.util.genericstore.data.AttributeNumberValueDO[] ret = null;
	try {
	    org.webdocwf.util.genericstore.data.AttributeNumberValueQuery q = new org.webdocwf.util.genericstore.data.AttributeNumberValueQuery( usr );
	    q.setQueryGENERICOBJECTOID( this, usr );
	    ret = q.getDOArray();
	} catch ( NonUniqueQueryException e ) { 
	    throw new DataObjectException( 
		"INTERNAL ERROR: unexpected NonUniqueQueryException" );
	} finally {
	    if ( null == ret )
		ret = new org.webdocwf.util.genericstore.data.AttributeNumberValueDO[ 0 ];
	}
	return ret;
    }
    /**
     * Get the single AttributeNumberValueDO object
     * that refers to this DO.
     *
     * @param usr The user for security checks
     * @return AttributeNumberValueDO object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     * @exception NonUniqueQueryException
     *   If more than one AttributeNumberValueDO object was found.
     * @exception AccessException
     *   The user is not allowed to create a query for 
     *   org.webdocwf.util.genericstore.data.AttributeNumberValue or 
     *   to access the foreign key attribute GENERICOBJECTOID.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public org.webdocwf.util.genericstore.data.AttributeNumberValueDO getAttributeNumberValueDO ( org.webdocwf.dods.access.User usr ) 
    throws DataObjectException, QueryException, NonUniqueQueryException, AccessException {
	org.webdocwf.util.genericstore.data.AttributeNumberValueQuery q = new org.webdocwf.util.genericstore.data.AttributeNumberValueQuery( usr );
	q.setQueryGENERICOBJECTOID( this, usr );
	q.requireUniqueInstance();
	return q.getNextDO();
    }
   /**
     * Add (set & commit) a AttributeNumberValueDO object that refers to this DO.
     *
     * @param referrer AttributeNumberValueDO to be set to point to this DO and committed.
     * @param usr The user for security checks
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void addAttributeNumberValueDO( org.webdocwf.util.genericstore.data.AttributeNumberValueDO referrer, org.webdocwf.dods.access.User usr )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException, AccessException {
        addAttributeNumberValueDO( referrer, null, usr );
    }
 
 
    /**
     * Add (set & commit) a AttributeNumberValueDO object that refers to this DO.
     *
     * @param referrer AttributeNumberValueDO to be set to point to this DO and committed.
     * @param usr The user for security checks
     *
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void addAttributeNumberValueDO( org.webdocwf.util.genericstore.data.AttributeNumberValueDO referrer, DBTransaction tran, org.webdocwf.dods.access.User usr )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException, AccessException {
        referrer.setGENERICOBJECTOID( this, usr );
        referrer.commit( tran );
    }

 
    /**
     * Remove (delete) a AttributeNumberValueDO object that refers to this DO.
     *
     * @param referrer AttributeNumberValueDO to be deleted.
     * @param usr The user for security checks
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void removeAttributeNumberValueDO( org.webdocwf.util.genericstore.data.AttributeNumberValueDO referrer, org.webdocwf.dods.access.User usr )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException, AccessException {
        removeAttributeNumberValueDO( referrer, null, usr );
    }
 
 
    /**
     * Remove (delete) a AttributeNumberValueDO object that refers to this DO.
     *
     * @param referrer AttributeNumberValueDO to be deleted.
     * @param usr The user for security checks
     *
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void removeAttributeNumberValueDO( org.webdocwf.util.genericstore.data.AttributeNumberValueDO referrer, DBTransaction tran, org.webdocwf.dods.access.User usr )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException, AccessException {
	GenericObjectDO referred = referrer.getGENERICOBJECTOID( usr );
	String referredHandle = referred.getHandle( usr );
	String mydoHandle = this.getHandle( usr );
	if ( null == referredHandle || null == mydoHandle || 
	     ( ! referredHandle.equals( mydoHandle ) ) ) {
	    throw new DataObjectException( "Object " + referrer +
		" does not refer to object " + this +
		", cannot be removed this way." );
	}
        referrer.delete( tran, usr );
    }
// end of WebDocWf extension for DODS row instance security

    // WebDocWF extension for DODS row counter
    // The following lines have been added:
    /**
     * Get the number of AttributeNumberValueDOs that refer to this DO.
     * via GENERICOBJECTOID
     *
     * @return The number of objects that refer to this DO
     *
     * @exception DataObjectException If object is not found in the database.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public int getAttributeNumberValueDOArrayCount () 
    throws DataObjectException, QueryException, NonUniqueQueryException, SQLException, DatabaseManagerException {
	int ret = 0;
    org.webdocwf.util.genericstore.data.AttributeNumberValueQuery q = new org.webdocwf.util.genericstore.data.AttributeNumberValueQuery();
    q.setQueryGENERICOBJECTOID( this );
    ret = q.getCount();
	return ret;
    }

    /**
     * Get the number of AttributeNumberValueDOs that refer to this DO.
     * via GENERICOBJECTOID
     *
     * @param usr The user for security checks
     * @return The number of objects that refer to this DO
     *
     * @exception DataObjectException If object is not found in the database.
     * @exception AccessException
     *   The user is not allowed to create a query for 
     *   org.webdocwf.util.genericstore.data.AttributeNumberValue or 
     *   to access the foreign key attribute GENERICOBJECTOID.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public int getAttributeNumberValueDOArrayCount ( org.webdocwf.dods.access.User usr ) 
    throws DataObjectException, QueryException, NonUniqueQueryException, SQLException, DatabaseManagerException, AccessException {
	int ret = 0;
    org.webdocwf.util.genericstore.data.AttributeNumberValueQuery q = new org.webdocwf.util.genericstore.data.AttributeNumberValueQuery(usr);
    q.setQueryGENERICOBJECTOID( this, usr );
    ret = q.getCount();
	return ret;
    }
    // end of WebDocWF extension for DODS row counter

    /**
     * Get array of AttributeObjectValueDO objects that refer to this DO.
     *
     * @return array of AttributeObjectValueDO objects.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     */
    public org.webdocwf.util.genericstore.data.AttributeObjectValueDO[] getAttributeObjectValueDOArray () 
    throws DataObjectException, QueryException {
	org.webdocwf.util.genericstore.data.AttributeObjectValueDO[] ret = null;
	try {
	    org.webdocwf.util.genericstore.data.AttributeObjectValueQuery q = new org.webdocwf.util.genericstore.data.AttributeObjectValueQuery();
	    q.setQueryGENERICOBJECTOID( this );
	    ret = q.getDOArray();
	} catch ( NonUniqueQueryException e ) { 
	    throw new DataObjectException( 
		"INTERNAL ERROR: unexpected NonUniqueQueryException" );
	} finally {
	    if ( null == ret )
		ret = new org.webdocwf.util.genericstore.data.AttributeObjectValueDO[ 0 ];
	}
	return ret;
    }

    /**
     * Get the single AttributeObjectValueDO object
     * that refers to this DO.
     *
     * @return AttributeObjectValueDO object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     * @exception NonUniqueQueryException
     *   If more than one AttributeObjectValueDO object was found.
     */
    public org.webdocwf.util.genericstore.data.AttributeObjectValueDO getAttributeObjectValueDO () 
    throws DataObjectException, QueryException, NonUniqueQueryException {
	org.webdocwf.util.genericstore.data.AttributeObjectValueQuery q = new org.webdocwf.util.genericstore.data.AttributeObjectValueQuery();
	q.setQueryGENERICOBJECTOID( this );
	q.requireUniqueInstance();
	return q.getNextDO();
    }

    /**
     * Add (set & commit) a AttributeObjectValueDO object that refers to this DO.
     *
     * @param referrer AttributeObjectValueDO to be set to point to this DO and committed.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void addAttributeObjectValueDO( org.webdocwf.util.genericstore.data.AttributeObjectValueDO referrer )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
// WebDocWf extension for DODS row instance security
// The following line has been changed:
        addAttributeObjectValueDO( referrer, (DBTransaction)null );
// before:        addAttributeObjectValueDO( referrer, null );
// end of WebDocWf extension for DODS row instance security
    }
 
 
    /**
     * Add (set & commit) a AttributeObjectValueDO object that refers to this DO.
     *
     * @param referrer AttributeObjectValueDO to be set to point to this DO and committed.
     *
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void addAttributeObjectValueDO( org.webdocwf.util.genericstore.data.AttributeObjectValueDO referrer, DBTransaction tran )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        referrer.setGENERICOBJECTOID( this );
        referrer.commit( tran );
    }

 
    /**
     * Remove (delete) a AttributeObjectValueDO object that refers to this DO.
     *
     * @param referrer AttributeObjectValueDO to be deleted.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void removeAttributeObjectValueDO( org.webdocwf.util.genericstore.data.AttributeObjectValueDO referrer )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
// WebDocWf extension for DODS row instance security
// The following line has been changed:
        removeAttributeObjectValueDO( referrer, (DBTransaction)null );
// before:        removeAttributeObjectValueDO( referrer, null );
// end of WebDocWf extension for DODS row instance security
    }
 
 
    /**
     * Remove (delete) a AttributeObjectValueDO object that refers to this DO.
     *
     * @param referrer AttributeObjectValueDO to be deleted.
     *
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void removeAttributeObjectValueDO( org.webdocwf.util.genericstore.data.AttributeObjectValueDO referrer, DBTransaction tran )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
	GenericObjectDO referred = referrer.getGENERICOBJECTOID();
	String referredHandle = referred.getHandle();
	String mydoHandle = this.getHandle();
	if ( null == referredHandle || null == mydoHandle || 
	     ( ! referredHandle.equals( mydoHandle ) ) ) {
	    throw new DataObjectException( "Object " + referrer +
		" does not refer to object " + this +
		", cannot be removed this way." );
	}
        referrer.delete( tran );
    }
    // WebDocWF extension for DODS row instance security
    // The following lines have been added:
    /**
     * Get array of AttributeObjectValueDO objects that refer to this DO.
     *
     * @param usr The user for security checks
     * @return array of AttributeObjectValueDO objects.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     * @exception AccessException
     *   The user is not allowed to create a query for 
     *   org.webdocwf.util.genericstore.data.AttributeObjectValue or 
     *   to access the foreign key attribute GENERICOBJECTOID.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public org.webdocwf.util.genericstore.data.AttributeObjectValueDO[] getAttributeObjectValueDOArray ( org.webdocwf.dods.access.User usr ) 
    throws DataObjectException, QueryException, AccessException {
	org.webdocwf.util.genericstore.data.AttributeObjectValueDO[] ret = null;
	try {
	    org.webdocwf.util.genericstore.data.AttributeObjectValueQuery q = new org.webdocwf.util.genericstore.data.AttributeObjectValueQuery( usr );
	    q.setQueryGENERICOBJECTOID( this, usr );
	    ret = q.getDOArray();
	} catch ( NonUniqueQueryException e ) { 
	    throw new DataObjectException( 
		"INTERNAL ERROR: unexpected NonUniqueQueryException" );
	} finally {
	    if ( null == ret )
		ret = new org.webdocwf.util.genericstore.data.AttributeObjectValueDO[ 0 ];
	}
	return ret;
    }
    /**
     * Get the single AttributeObjectValueDO object
     * that refers to this DO.
     *
     * @param usr The user for security checks
     * @return AttributeObjectValueDO object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     * @exception NonUniqueQueryException
     *   If more than one AttributeObjectValueDO object was found.
     * @exception AccessException
     *   The user is not allowed to create a query for 
     *   org.webdocwf.util.genericstore.data.AttributeObjectValue or 
     *   to access the foreign key attribute GENERICOBJECTOID.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public org.webdocwf.util.genericstore.data.AttributeObjectValueDO getAttributeObjectValueDO ( org.webdocwf.dods.access.User usr ) 
    throws DataObjectException, QueryException, NonUniqueQueryException, AccessException {
	org.webdocwf.util.genericstore.data.AttributeObjectValueQuery q = new org.webdocwf.util.genericstore.data.AttributeObjectValueQuery( usr );
	q.setQueryGENERICOBJECTOID( this, usr );
	q.requireUniqueInstance();
	return q.getNextDO();
    }
   /**
     * Add (set & commit) a AttributeObjectValueDO object that refers to this DO.
     *
     * @param referrer AttributeObjectValueDO to be set to point to this DO and committed.
     * @param usr The user for security checks
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void addAttributeObjectValueDO( org.webdocwf.util.genericstore.data.AttributeObjectValueDO referrer, org.webdocwf.dods.access.User usr )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException, AccessException {
        addAttributeObjectValueDO( referrer, null, usr );
    }
 
 
    /**
     * Add (set & commit) a AttributeObjectValueDO object that refers to this DO.
     *
     * @param referrer AttributeObjectValueDO to be set to point to this DO and committed.
     * @param usr The user for security checks
     *
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void addAttributeObjectValueDO( org.webdocwf.util.genericstore.data.AttributeObjectValueDO referrer, DBTransaction tran, org.webdocwf.dods.access.User usr )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException, AccessException {
        referrer.setGENERICOBJECTOID( this, usr );
        referrer.commit( tran );
    }

 
    /**
     * Remove (delete) a AttributeObjectValueDO object that refers to this DO.
     *
     * @param referrer AttributeObjectValueDO to be deleted.
     * @param usr The user for security checks
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void removeAttributeObjectValueDO( org.webdocwf.util.genericstore.data.AttributeObjectValueDO referrer, org.webdocwf.dods.access.User usr )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException, AccessException {
        removeAttributeObjectValueDO( referrer, null, usr );
    }
 
 
    /**
     * Remove (delete) a AttributeObjectValueDO object that refers to this DO.
     *
     * @param referrer AttributeObjectValueDO to be deleted.
     * @param usr The user for security checks
     *
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void removeAttributeObjectValueDO( org.webdocwf.util.genericstore.data.AttributeObjectValueDO referrer, DBTransaction tran, org.webdocwf.dods.access.User usr )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException, AccessException {
	GenericObjectDO referred = referrer.getGENERICOBJECTOID( usr );
	String referredHandle = referred.getHandle( usr );
	String mydoHandle = this.getHandle( usr );
	if ( null == referredHandle || null == mydoHandle || 
	     ( ! referredHandle.equals( mydoHandle ) ) ) {
	    throw new DataObjectException( "Object " + referrer +
		" does not refer to object " + this +
		", cannot be removed this way." );
	}
        referrer.delete( tran, usr );
    }
// end of WebDocWf extension for DODS row instance security

    // WebDocWF extension for DODS row counter
    // The following lines have been added:
    /**
     * Get the number of AttributeObjectValueDOs that refer to this DO.
     * via GENERICOBJECTOID
     *
     * @return The number of objects that refer to this DO
     *
     * @exception DataObjectException If object is not found in the database.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public int getAttributeObjectValueDOArrayCount () 
    throws DataObjectException, QueryException, NonUniqueQueryException, SQLException, DatabaseManagerException {
	int ret = 0;
    org.webdocwf.util.genericstore.data.AttributeObjectValueQuery q = new org.webdocwf.util.genericstore.data.AttributeObjectValueQuery();
    q.setQueryGENERICOBJECTOID( this );
    ret = q.getCount();
	return ret;
    }

    /**
     * Get the number of AttributeObjectValueDOs that refer to this DO.
     * via GENERICOBJECTOID
     *
     * @param usr The user for security checks
     * @return The number of objects that refer to this DO
     *
     * @exception DataObjectException If object is not found in the database.
     * @exception AccessException
     *   The user is not allowed to create a query for 
     *   org.webdocwf.util.genericstore.data.AttributeObjectValue or 
     *   to access the foreign key attribute GENERICOBJECTOID.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public int getAttributeObjectValueDOArrayCount ( org.webdocwf.dods.access.User usr ) 
    throws DataObjectException, QueryException, NonUniqueQueryException, SQLException, DatabaseManagerException, AccessException {
	int ret = 0;
    org.webdocwf.util.genericstore.data.AttributeObjectValueQuery q = new org.webdocwf.util.genericstore.data.AttributeObjectValueQuery(usr);
    q.setQueryGENERICOBJECTOID( this, usr );
    ret = q.getCount();
	return ret;
    }
    // end of WebDocWF extension for DODS row counter

    /**
     * Get array of AttributeStringValueDO objects that refer to this DO.
     *
     * @return array of AttributeStringValueDO objects.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     */
    public org.webdocwf.util.genericstore.data.AttributeStringValueDO[] getAttributeStringValueDOArray () 
    throws DataObjectException, QueryException {
	org.webdocwf.util.genericstore.data.AttributeStringValueDO[] ret = null;
	try {
	    org.webdocwf.util.genericstore.data.AttributeStringValueQuery q = new org.webdocwf.util.genericstore.data.AttributeStringValueQuery();
	    q.setQueryGENERICOBJECTOID( this );
	    ret = q.getDOArray();
	} catch ( NonUniqueQueryException e ) { 
	    throw new DataObjectException( 
		"INTERNAL ERROR: unexpected NonUniqueQueryException" );
	} finally {
	    if ( null == ret )
		ret = new org.webdocwf.util.genericstore.data.AttributeStringValueDO[ 0 ];
	}
	return ret;
    }

    /**
     * Get the single AttributeStringValueDO object
     * that refers to this DO.
     *
     * @return AttributeStringValueDO object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     * @exception NonUniqueQueryException
     *   If more than one AttributeStringValueDO object was found.
     */
    public org.webdocwf.util.genericstore.data.AttributeStringValueDO getAttributeStringValueDO () 
    throws DataObjectException, QueryException, NonUniqueQueryException {
	org.webdocwf.util.genericstore.data.AttributeStringValueQuery q = new org.webdocwf.util.genericstore.data.AttributeStringValueQuery();
	q.setQueryGENERICOBJECTOID( this );
	q.requireUniqueInstance();
	return q.getNextDO();
    }

    /**
     * Add (set & commit) a AttributeStringValueDO object that refers to this DO.
     *
     * @param referrer AttributeStringValueDO to be set to point to this DO and committed.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void addAttributeStringValueDO( org.webdocwf.util.genericstore.data.AttributeStringValueDO referrer )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
// WebDocWf extension for DODS row instance security
// The following line has been changed:
        addAttributeStringValueDO( referrer, (DBTransaction)null );
// before:        addAttributeStringValueDO( referrer, null );
// end of WebDocWf extension for DODS row instance security
    }
 
 
    /**
     * Add (set & commit) a AttributeStringValueDO object that refers to this DO.
     *
     * @param referrer AttributeStringValueDO to be set to point to this DO and committed.
     *
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void addAttributeStringValueDO( org.webdocwf.util.genericstore.data.AttributeStringValueDO referrer, DBTransaction tran )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
        referrer.setGENERICOBJECTOID( this );
        referrer.commit( tran );
    }

 
    /**
     * Remove (delete) a AttributeStringValueDO object that refers to this DO.
     *
     * @param referrer AttributeStringValueDO to be deleted.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void removeAttributeStringValueDO( org.webdocwf.util.genericstore.data.AttributeStringValueDO referrer )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
// WebDocWf extension for DODS row instance security
// The following line has been changed:
        removeAttributeStringValueDO( referrer, (DBTransaction)null );
// before:        removeAttributeStringValueDO( referrer, null );
// end of WebDocWf extension for DODS row instance security
    }
 
 
    /**
     * Remove (delete) a AttributeStringValueDO object that refers to this DO.
     *
     * @param referrer AttributeStringValueDO to be deleted.
     *
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     */
    public void removeAttributeStringValueDO( org.webdocwf.util.genericstore.data.AttributeStringValueDO referrer, DBTransaction tran )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
	GenericObjectDO referred = referrer.getGENERICOBJECTOID();
	String referredHandle = referred.getHandle();
	String mydoHandle = this.getHandle();
	if ( null == referredHandle || null == mydoHandle || 
	     ( ! referredHandle.equals( mydoHandle ) ) ) {
	    throw new DataObjectException( "Object " + referrer +
		" does not refer to object " + this +
		", cannot be removed this way." );
	}
        referrer.delete( tran );
    }
    // WebDocWF extension for DODS row instance security
    // The following lines have been added:
    /**
     * Get array of AttributeStringValueDO objects that refer to this DO.
     *
     * @param usr The user for security checks
     * @return array of AttributeStringValueDO objects.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     * @exception AccessException
     *   The user is not allowed to create a query for 
     *   org.webdocwf.util.genericstore.data.AttributeStringValue or 
     *   to access the foreign key attribute GENERICOBJECTOID.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public org.webdocwf.util.genericstore.data.AttributeStringValueDO[] getAttributeStringValueDOArray ( org.webdocwf.dods.access.User usr ) 
    throws DataObjectException, QueryException, AccessException {
	org.webdocwf.util.genericstore.data.AttributeStringValueDO[] ret = null;
	try {
	    org.webdocwf.util.genericstore.data.AttributeStringValueQuery q = new org.webdocwf.util.genericstore.data.AttributeStringValueQuery( usr );
	    q.setQueryGENERICOBJECTOID( this, usr );
	    ret = q.getDOArray();
	} catch ( NonUniqueQueryException e ) { 
	    throw new DataObjectException( 
		"INTERNAL ERROR: unexpected NonUniqueQueryException" );
	} finally {
	    if ( null == ret )
		ret = new org.webdocwf.util.genericstore.data.AttributeStringValueDO[ 0 ];
	}
	return ret;
    }
    /**
     * Get the single AttributeStringValueDO object
     * that refers to this DO.
     *
     * @param usr The user for security checks
     * @return AttributeStringValueDO object.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     * @exception NonUniqueQueryException
     *   If more than one AttributeStringValueDO object was found.
     * @exception AccessException
     *   The user is not allowed to create a query for 
     *   org.webdocwf.util.genericstore.data.AttributeStringValue or 
     *   to access the foreign key attribute GENERICOBJECTOID.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public org.webdocwf.util.genericstore.data.AttributeStringValueDO getAttributeStringValueDO ( org.webdocwf.dods.access.User usr ) 
    throws DataObjectException, QueryException, NonUniqueQueryException, AccessException {
	org.webdocwf.util.genericstore.data.AttributeStringValueQuery q = new org.webdocwf.util.genericstore.data.AttributeStringValueQuery( usr );
	q.setQueryGENERICOBJECTOID( this, usr );
	q.requireUniqueInstance();
	return q.getNextDO();
    }
   /**
     * Add (set & commit) a AttributeStringValueDO object that refers to this DO.
     *
     * @param referrer AttributeStringValueDO to be set to point to this DO and committed.
     * @param usr The user for security checks
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void addAttributeStringValueDO( org.webdocwf.util.genericstore.data.AttributeStringValueDO referrer, org.webdocwf.dods.access.User usr )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException, AccessException {
        addAttributeStringValueDO( referrer, null, usr );
    }
 
 
    /**
     * Add (set & commit) a AttributeStringValueDO object that refers to this DO.
     *
     * @param referrer AttributeStringValueDO to be set to point to this DO and committed.
     * @param usr The user for security checks
     *
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void addAttributeStringValueDO( org.webdocwf.util.genericstore.data.AttributeStringValueDO referrer, DBTransaction tran, org.webdocwf.dods.access.User usr )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException, AccessException {
        referrer.setGENERICOBJECTOID( this, usr );
        referrer.commit( tran );
    }

 
    /**
     * Remove (delete) a AttributeStringValueDO object that refers to this DO.
     *
     * @param referrer AttributeStringValueDO to be deleted.
     * @param usr The user for security checks
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void removeAttributeStringValueDO( org.webdocwf.util.genericstore.data.AttributeStringValueDO referrer, org.webdocwf.dods.access.User usr )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException, AccessException {
        removeAttributeStringValueDO( referrer, null, usr );
    }
 
 
    /**
     * Remove (delete) a AttributeStringValueDO object that refers to this DO.
     *
     * @param referrer AttributeStringValueDO to be deleted.
     * @param usr The user for security checks
     *
     * @param tran The transaction to be used for the commit.
     * If null, a new transaction is created.
     *
     * @exception DatabaseManagerException if could not create a transaction
     * @exception java.sql.SQLException if any SQL errors occur.
     * @exception DataObjectException If object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void removeAttributeStringValueDO( org.webdocwf.util.genericstore.data.AttributeStringValueDO referrer, DBTransaction tran, org.webdocwf.dods.access.User usr )
    throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException, AccessException {
	GenericObjectDO referred = referrer.getGENERICOBJECTOID( usr );
	String referredHandle = referred.getHandle( usr );
	String mydoHandle = this.getHandle( usr );
	if ( null == referredHandle || null == mydoHandle || 
	     ( ! referredHandle.equals( mydoHandle ) ) ) {
	    throw new DataObjectException( "Object " + referrer +
		" does not refer to object " + this +
		", cannot be removed this way." );
	}
        referrer.delete( tran, usr );
    }
// end of WebDocWf extension for DODS row instance security

    // WebDocWF extension for DODS row counter
    // The following lines have been added:
    /**
     * Get the number of AttributeStringValueDOs that refer to this DO.
     * via GENERICOBJECTOID
     *
     * @return The number of objects that refer to this DO
     *
     * @exception DataObjectException If object is not found in the database.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public int getAttributeStringValueDOArrayCount () 
    throws DataObjectException, QueryException, NonUniqueQueryException, SQLException, DatabaseManagerException {
	int ret = 0;
    org.webdocwf.util.genericstore.data.AttributeStringValueQuery q = new org.webdocwf.util.genericstore.data.AttributeStringValueQuery();
    q.setQueryGENERICOBJECTOID( this );
    ret = q.getCount();
	return ret;
    }

    /**
     * Get the number of AttributeStringValueDOs that refer to this DO.
     * via GENERICOBJECTOID
     *
     * @param usr The user for security checks
     * @return The number of objects that refer to this DO
     *
     * @exception DataObjectException If object is not found in the database.
     * @exception AccessException
     *   The user is not allowed to create a query for 
     *   org.webdocwf.util.genericstore.data.AttributeStringValue or 
     *   to access the foreign key attribute GENERICOBJECTOID.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public int getAttributeStringValueDOArrayCount ( org.webdocwf.dods.access.User usr ) 
    throws DataObjectException, QueryException, NonUniqueQueryException, SQLException, DatabaseManagerException, AccessException {
	int ret = 0;
    org.webdocwf.util.genericstore.data.AttributeStringValueQuery q = new org.webdocwf.util.genericstore.data.AttributeStringValueQuery(usr);
    q.setQueryGENERICOBJECTOID( this, usr );
    ret = q.getCount();
	return ret;
    }
    // end of WebDocWF extension for DODS row counter



    /**
     * From the many-to-many relationship expressed by AttributeBoolValueDO,
     * get array of AttributeDefinitionDO objects that indirectly refer
     * to this DO.
     *
     * @return array of AttributeDefinitionDO objects.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public org.webdocwf.util.genericstore.data.AttributeDefinitionDO[] getAttributeDefinitionDOArray_via_AttributeBoolValue () 
    throws DataObjectException {
	org.webdocwf.util.genericstore.data.AttributeDefinitionDO[] ret = null;
	try {
	    org.webdocwf.util.genericstore.data.AttributeBoolValueDO[] arr = getAttributeBoolValueDOArray();
	    ret = new org.webdocwf.util.genericstore.data.AttributeDefinitionDO[ arr.length ];
	    for ( int i = 0; i < arr.length; i++ ) {
		ret[ i ] = arr[ i ].getATTRIBUTEDEFINITIONOID();
	    }
	} catch ( Exception e ) { 
	    throw new DataObjectException( 
		"INTERNAL ERROR: ", e );
	} finally {
	    if ( null == ret )
		ret = new org.webdocwf.util.genericstore.data.AttributeDefinitionDO[ 0 ];
	}
	return ret;
    }

    /**
     * To the many-to-many relationship expressed by AttributeBoolValueDO,
     * add a AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param d The AttributeDefinitionDO to add to the AttributeBoolValueDO mapping
     * for this DO.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public void mapAttributeDefinition_via_AttributeBoolValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException {
// WebDocWf fix, was ambiguous, the following line was changed:
	mapAttributeDefinition_via_AttributeBoolValueDO( d, (DBTransaction)null );
//before: mapAttributeDefinition_via_AttributeBoolValueDO( d, null );
// end of WebDocWf fix
    }

    /**
     * To the many-to-many relationship expressed by AttributeBoolValueDO,
     * add a AttributeDefinitionDO object that indirectly refers to this DO.
     *
     * @param b The AttributeDefinitionDO to add to the AttributeBoolValueDO mapping for this DO.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public void mapAttributeDefinition_via_AttributeBoolValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, DBTransaction tran )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException {
	org.webdocwf.util.genericstore.data.AttributeBoolValueDO m = null;
	try {
	    m = org.webdocwf.util.genericstore.data.AttributeBoolValueDO.createVirgin();
	} catch ( Exception e ) { 
	    throw new DataObjectException( 
		"org.webdocwf.util.genericstore.data.AttributeBoolValueDO.createVirgin failed", e );
	}
	m.setATTRIBUTEDEFINITIONOID( d );
	m.setGENERICOBJECTOID( this );
	m.commit( tran );
    }

    /**
     * From the many-to-many relationship expressed by AttributeBoolValueDO,
     * remove (delete) the AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param d The AttributeDefinitionDO to remove from the AttributeBoolValueDO mapping
     * for this DO.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     */
    public void unmapAttributeDefinition_via_AttributeBoolValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException {
// WebDocWf fix, was ambiguous, the following line was changed:
	unmapAttributeDefinition_via_AttributeBoolValueDO( d, (DBTransaction)null );
//before: unmapAttributeDefinition_via_AttributeBoolValueDO( d, null );
// end of WebDocWf fix
    }

    /**
     * From the many-to-many relationship expressed by AttributeBoolValueDO,
     * remove (delete) the AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param b The AttributeDefinitionDO to remove from the AttributeBoolValueDO mapping
     * for this DO.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     */
    public void unmapAttributeDefinition_via_AttributeBoolValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, DBTransaction tran )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException {
	org.webdocwf.util.genericstore.data.AttributeBoolValueQuery q = new org.webdocwf.util.genericstore.data.AttributeBoolValueQuery();
	q.setQueryGENERICOBJECTOID( this );
	q.setQueryATTRIBUTEDEFINITIONOID( d );
	q.requireUniqueInstance();
	org.webdocwf.util.genericstore.data.AttributeBoolValueDO m = null;
	try {
	    m = q.getNextDO();
	} catch ( NonUniqueQueryException e ) { 
	    throw new DataObjectException( "Multiple mappings for " +
		this + " and " + d + " in org.webdocwf.util.genericstore.data.AttributeBoolValue table." );
	}
	m.delete( tran );
    }
    // WebDocWF extension for DODS row instance security
    // The following lines have been added:
    /**
     * From the many-to-many relationship expressed by AttributeBoolValueDO,
     * get array of AttributeDefinitionDO objects that indirectly refer
     * to this DO.
     *
     * @param usr The user for security checks
     * @return array of AttributeDefinitionDO objects.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception AccessException
     *   The user is not allowed to create a query for 
     *   org.webdocwf.util.genericstore.data.AttributeBoolValue or 
     *   to access the foreign key attribute ATTRIBUTEDEFINITIONOID.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public org.webdocwf.util.genericstore.data.AttributeDefinitionDO[] getAttributeDefinitionDOArray_via_AttributeBoolValue ( org.webdocwf.dods.access.User usr ) 
    throws DataObjectException, AccessException {
	org.webdocwf.util.genericstore.data.AttributeDefinitionDO[] ret = null;
	try {
	    org.webdocwf.util.genericstore.data.AttributeBoolValueDO[] arr = getAttributeBoolValueDOArray( usr );
         Vector tempVector = new Vector();
	    for ( int j = 0; j < arr.length; j++ ) {
		if (((org.webdocwf.util.genericstore.data.AttributeBoolValueDO)tempVector.get(j)).hasDOGetAttrATTRIBUTEDEFINITIONOIDAccess( usr)) tempVector.add(arr[ j ]);
	    }
	    ret = new org.webdocwf.util.genericstore.data.AttributeDefinitionDO[ tempVector.size() ];
	    for ( int i = 0; i < tempVector.size(); i++ ) {
		ret[ i ] = ((org.webdocwf.util.genericstore.data.AttributeBoolValueDO)tempVector.get(i)).getATTRIBUTEDEFINITIONOID( usr );
	    }
	} catch ( QueryException e ) { 
	    throw new DataObjectException( 
		"INTERNAL ERROR: ", e );
	} finally {
	    if ( null == ret )
		ret = new org.webdocwf.util.genericstore.data.AttributeDefinitionDO[ 0 ];
	}
	return ret;
    }
    /**
     * To the many-to-many relationship expressed by AttributeBoolValueDO,
     * add a AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param d The AttributeDefinitionDO to add to the AttributeBoolValueDO mapping
     * for this DO.
     * @param usr The user for security checks
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void mapAttributeDefinition_via_AttributeBoolValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, org.webdocwf.dods.access.User usr )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException, AccessException {
	mapAttributeDefinition_via_AttributeBoolValueDO( d, null, usr );
    }

    /**
     * To the many-to-many relationship expressed by AttributeBoolValueDO,
     * add a AttributeDefinitionDO object that indirectly refers to this DO.
     *
     * @param b The AttributeDefinitionDO to add to the AttributeBoolValueDO mapping for this DO.
     * @param usr The user for security checks
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void mapAttributeDefinition_via_AttributeBoolValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, DBTransaction tran, org.webdocwf.dods.access.User usr )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException, AccessException {
	org.webdocwf.util.genericstore.data.AttributeBoolValueDO m = null;
	try {
	    m = org.webdocwf.util.genericstore.data.AttributeBoolValueDO.createVirgin( usr );
	} catch ( ObjectIdException e ) { 
	    throw new DataObjectException( 
		"org.webdocwf.util.genericstore.data.AttributeBoolValueDO.createVirgin failed", e );
	}
	m.setATTRIBUTEDEFINITIONOID( d, usr );
	m.setGENERICOBJECTOID( this, usr );
	m.commit( tran );
    }

    /**
     * From the many-to-many relationship expressed by AttributeBoolValueDO,
     * remove (delete) the AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param d The AttributeDefinitionDO to remove from the AttributeBoolValueDO mapping
     * for this DO.
     * @param usr The user for security checks
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     *
     * WebDocWf extension
     *
     */
    public void unmapAttributeDefinition_via_AttributeBoolValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, org.webdocwf.dods.access.User usr )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException, AccessException {
	unmapAttributeDefinition_via_AttributeBoolValueDO( d, null, usr );
    }

    /**
     * From the many-to-many relationship expressed by AttributeBoolValueDO,
     * remove (delete) the AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param b The AttributeDefinitionDO to remove from the AttributeBoolValueDO mapping
     * for this DO.
     * @param usr The user for security checks
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     *
     * WebDocWf extension
     *
     */
    public void unmapAttributeDefinition_via_AttributeBoolValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, DBTransaction tran, org.webdocwf.dods.access.User usr )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException, AccessException {
	org.webdocwf.util.genericstore.data.AttributeBoolValueQuery q = new org.webdocwf.util.genericstore.data.AttributeBoolValueQuery( usr );
	q.setQueryGENERICOBJECTOID( this , usr);
	q.setQueryATTRIBUTEDEFINITIONOID( d , usr);
	q.requireUniqueInstance();
	org.webdocwf.util.genericstore.data.AttributeBoolValueDO m = null;
	try {
	    m = q.getNextDO();
	} catch ( NonUniqueQueryException e ) { 
	    throw new DataObjectException( "Multiple mappings for " +
		this + " and " + d + " in org.webdocwf.util.genericstore.data.AttributeBoolValue table." );
	}
	m.delete( tran, usr );
    }
// end of WebDocWf extension for DODS row instance security

    /**
     * From the many-to-many relationship expressed by AttributeDateValueDO,
     * get array of AttributeDefinitionDO objects that indirectly refer
     * to this DO.
     *
     * @return array of AttributeDefinitionDO objects.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public org.webdocwf.util.genericstore.data.AttributeDefinitionDO[] getAttributeDefinitionDOArray_via_AttributeDateValue () 
    throws DataObjectException {
	org.webdocwf.util.genericstore.data.AttributeDefinitionDO[] ret = null;
	try {
	    org.webdocwf.util.genericstore.data.AttributeDateValueDO[] arr = getAttributeDateValueDOArray();
	    ret = new org.webdocwf.util.genericstore.data.AttributeDefinitionDO[ arr.length ];
	    for ( int i = 0; i < arr.length; i++ ) {
		ret[ i ] = arr[ i ].getATTRIBUTEDEFINITIONOID();
	    }
	} catch ( Exception e ) { 
	    throw new DataObjectException( 
		"INTERNAL ERROR: ", e );
	} finally {
	    if ( null == ret )
		ret = new org.webdocwf.util.genericstore.data.AttributeDefinitionDO[ 0 ];
	}
	return ret;
    }

    /**
     * To the many-to-many relationship expressed by AttributeDateValueDO,
     * add a AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param d The AttributeDefinitionDO to add to the AttributeDateValueDO mapping
     * for this DO.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public void mapAttributeDefinition_via_AttributeDateValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException {
// WebDocWf fix, was ambiguous, the following line was changed:
	mapAttributeDefinition_via_AttributeDateValueDO( d, (DBTransaction)null );
//before: mapAttributeDefinition_via_AttributeDateValueDO( d, null );
// end of WebDocWf fix
    }

    /**
     * To the many-to-many relationship expressed by AttributeDateValueDO,
     * add a AttributeDefinitionDO object that indirectly refers to this DO.
     *
     * @param b The AttributeDefinitionDO to add to the AttributeDateValueDO mapping for this DO.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public void mapAttributeDefinition_via_AttributeDateValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, DBTransaction tran )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException {
	org.webdocwf.util.genericstore.data.AttributeDateValueDO m = null;
	try {
	    m = org.webdocwf.util.genericstore.data.AttributeDateValueDO.createVirgin();
	} catch ( Exception e ) { 
	    throw new DataObjectException( 
		"org.webdocwf.util.genericstore.data.AttributeDateValueDO.createVirgin failed", e );
	}
	m.setATTRIBUTEDEFINITIONOID( d );
	m.setGENERICOBJECTOID( this );
	m.commit( tran );
    }

    /**
     * From the many-to-many relationship expressed by AttributeDateValueDO,
     * remove (delete) the AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param d The AttributeDefinitionDO to remove from the AttributeDateValueDO mapping
     * for this DO.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     */
    public void unmapAttributeDefinition_via_AttributeDateValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException {
// WebDocWf fix, was ambiguous, the following line was changed:
	unmapAttributeDefinition_via_AttributeDateValueDO( d, (DBTransaction)null );
//before: unmapAttributeDefinition_via_AttributeDateValueDO( d, null );
// end of WebDocWf fix
    }

    /**
     * From the many-to-many relationship expressed by AttributeDateValueDO,
     * remove (delete) the AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param b The AttributeDefinitionDO to remove from the AttributeDateValueDO mapping
     * for this DO.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     */
    public void unmapAttributeDefinition_via_AttributeDateValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, DBTransaction tran )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException {
	org.webdocwf.util.genericstore.data.AttributeDateValueQuery q = new org.webdocwf.util.genericstore.data.AttributeDateValueQuery();
	q.setQueryGENERICOBJECTOID( this );
	q.setQueryATTRIBUTEDEFINITIONOID( d );
	q.requireUniqueInstance();
	org.webdocwf.util.genericstore.data.AttributeDateValueDO m = null;
	try {
	    m = q.getNextDO();
	} catch ( NonUniqueQueryException e ) { 
	    throw new DataObjectException( "Multiple mappings for " +
		this + " and " + d + " in org.webdocwf.util.genericstore.data.AttributeDateValue table." );
	}
	m.delete( tran );
    }
    // WebDocWF extension for DODS row instance security
    // The following lines have been added:
    /**
     * From the many-to-many relationship expressed by AttributeDateValueDO,
     * get array of AttributeDefinitionDO objects that indirectly refer
     * to this DO.
     *
     * @param usr The user for security checks
     * @return array of AttributeDefinitionDO objects.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception AccessException
     *   The user is not allowed to create a query for 
     *   org.webdocwf.util.genericstore.data.AttributeDateValue or 
     *   to access the foreign key attribute ATTRIBUTEDEFINITIONOID.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public org.webdocwf.util.genericstore.data.AttributeDefinitionDO[] getAttributeDefinitionDOArray_via_AttributeDateValue ( org.webdocwf.dods.access.User usr ) 
    throws DataObjectException, AccessException {
	org.webdocwf.util.genericstore.data.AttributeDefinitionDO[] ret = null;
	try {
	    org.webdocwf.util.genericstore.data.AttributeDateValueDO[] arr = getAttributeDateValueDOArray( usr );
         Vector tempVector = new Vector();
	    for ( int j = 0; j < arr.length; j++ ) {
		if (((org.webdocwf.util.genericstore.data.AttributeDateValueDO)tempVector.get(j)).hasDOGetAttrATTRIBUTEDEFINITIONOIDAccess( usr)) tempVector.add(arr[ j ]);
	    }
	    ret = new org.webdocwf.util.genericstore.data.AttributeDefinitionDO[ tempVector.size() ];
	    for ( int i = 0; i < tempVector.size(); i++ ) {
		ret[ i ] = ((org.webdocwf.util.genericstore.data.AttributeDateValueDO)tempVector.get(i)).getATTRIBUTEDEFINITIONOID( usr );
	    }
	} catch ( QueryException e ) { 
	    throw new DataObjectException( 
		"INTERNAL ERROR: ", e );
	} finally {
	    if ( null == ret )
		ret = new org.webdocwf.util.genericstore.data.AttributeDefinitionDO[ 0 ];
	}
	return ret;
    }
    /**
     * To the many-to-many relationship expressed by AttributeDateValueDO,
     * add a AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param d The AttributeDefinitionDO to add to the AttributeDateValueDO mapping
     * for this DO.
     * @param usr The user for security checks
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void mapAttributeDefinition_via_AttributeDateValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, org.webdocwf.dods.access.User usr )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException, AccessException {
	mapAttributeDefinition_via_AttributeDateValueDO( d, null, usr );
    }

    /**
     * To the many-to-many relationship expressed by AttributeDateValueDO,
     * add a AttributeDefinitionDO object that indirectly refers to this DO.
     *
     * @param b The AttributeDefinitionDO to add to the AttributeDateValueDO mapping for this DO.
     * @param usr The user for security checks
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void mapAttributeDefinition_via_AttributeDateValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, DBTransaction tran, org.webdocwf.dods.access.User usr )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException, AccessException {
	org.webdocwf.util.genericstore.data.AttributeDateValueDO m = null;
	try {
	    m = org.webdocwf.util.genericstore.data.AttributeDateValueDO.createVirgin( usr );
	} catch ( ObjectIdException e ) { 
	    throw new DataObjectException( 
		"org.webdocwf.util.genericstore.data.AttributeDateValueDO.createVirgin failed", e );
	}
	m.setATTRIBUTEDEFINITIONOID( d, usr );
	m.setGENERICOBJECTOID( this, usr );
	m.commit( tran );
    }

    /**
     * From the many-to-many relationship expressed by AttributeDateValueDO,
     * remove (delete) the AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param d The AttributeDefinitionDO to remove from the AttributeDateValueDO mapping
     * for this DO.
     * @param usr The user for security checks
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     *
     * WebDocWf extension
     *
     */
    public void unmapAttributeDefinition_via_AttributeDateValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, org.webdocwf.dods.access.User usr )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException, AccessException {
	unmapAttributeDefinition_via_AttributeDateValueDO( d, null, usr );
    }

    /**
     * From the many-to-many relationship expressed by AttributeDateValueDO,
     * remove (delete) the AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param b The AttributeDefinitionDO to remove from the AttributeDateValueDO mapping
     * for this DO.
     * @param usr The user for security checks
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     *
     * WebDocWf extension
     *
     */
    public void unmapAttributeDefinition_via_AttributeDateValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, DBTransaction tran, org.webdocwf.dods.access.User usr )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException, AccessException {
	org.webdocwf.util.genericstore.data.AttributeDateValueQuery q = new org.webdocwf.util.genericstore.data.AttributeDateValueQuery( usr );
	q.setQueryGENERICOBJECTOID( this , usr);
	q.setQueryATTRIBUTEDEFINITIONOID( d , usr);
	q.requireUniqueInstance();
	org.webdocwf.util.genericstore.data.AttributeDateValueDO m = null;
	try {
	    m = q.getNextDO();
	} catch ( NonUniqueQueryException e ) { 
	    throw new DataObjectException( "Multiple mappings for " +
		this + " and " + d + " in org.webdocwf.util.genericstore.data.AttributeDateValue table." );
	}
	m.delete( tran, usr );
    }
// end of WebDocWf extension for DODS row instance security

    /**
     * From the many-to-many relationship expressed by AttributeNumberValueDO,
     * get array of AttributeDefinitionDO objects that indirectly refer
     * to this DO.
     *
     * @return array of AttributeDefinitionDO objects.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public org.webdocwf.util.genericstore.data.AttributeDefinitionDO[] getAttributeDefinitionDOArray_via_AttributeNumberValue () 
    throws DataObjectException {
	org.webdocwf.util.genericstore.data.AttributeDefinitionDO[] ret = null;
	try {
	    org.webdocwf.util.genericstore.data.AttributeNumberValueDO[] arr = getAttributeNumberValueDOArray();
	    ret = new org.webdocwf.util.genericstore.data.AttributeDefinitionDO[ arr.length ];
	    for ( int i = 0; i < arr.length; i++ ) {
		ret[ i ] = arr[ i ].getATTRIBUTEDEFINITIONOID();
	    }
	} catch ( Exception e ) { 
	    throw new DataObjectException( 
		"INTERNAL ERROR: ", e );
	} finally {
	    if ( null == ret )
		ret = new org.webdocwf.util.genericstore.data.AttributeDefinitionDO[ 0 ];
	}
	return ret;
    }

    /**
     * To the many-to-many relationship expressed by AttributeNumberValueDO,
     * add a AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param d The AttributeDefinitionDO to add to the AttributeNumberValueDO mapping
     * for this DO.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public void mapAttributeDefinition_via_AttributeNumberValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException {
// WebDocWf fix, was ambiguous, the following line was changed:
	mapAttributeDefinition_via_AttributeNumberValueDO( d, (DBTransaction)null );
//before: mapAttributeDefinition_via_AttributeNumberValueDO( d, null );
// end of WebDocWf fix
    }

    /**
     * To the many-to-many relationship expressed by AttributeNumberValueDO,
     * add a AttributeDefinitionDO object that indirectly refers to this DO.
     *
     * @param b The AttributeDefinitionDO to add to the AttributeNumberValueDO mapping for this DO.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public void mapAttributeDefinition_via_AttributeNumberValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, DBTransaction tran )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException {
	org.webdocwf.util.genericstore.data.AttributeNumberValueDO m = null;
	try {
	    m = org.webdocwf.util.genericstore.data.AttributeNumberValueDO.createVirgin();
	} catch ( Exception e ) { 
	    throw new DataObjectException( 
		"org.webdocwf.util.genericstore.data.AttributeNumberValueDO.createVirgin failed", e );
	}
	m.setATTRIBUTEDEFINITIONOID( d );
	m.setGENERICOBJECTOID( this );
	m.commit( tran );
    }

    /**
     * From the many-to-many relationship expressed by AttributeNumberValueDO,
     * remove (delete) the AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param d The AttributeDefinitionDO to remove from the AttributeNumberValueDO mapping
     * for this DO.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     */
    public void unmapAttributeDefinition_via_AttributeNumberValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException {
// WebDocWf fix, was ambiguous, the following line was changed:
	unmapAttributeDefinition_via_AttributeNumberValueDO( d, (DBTransaction)null );
//before: unmapAttributeDefinition_via_AttributeNumberValueDO( d, null );
// end of WebDocWf fix
    }

    /**
     * From the many-to-many relationship expressed by AttributeNumberValueDO,
     * remove (delete) the AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param b The AttributeDefinitionDO to remove from the AttributeNumberValueDO mapping
     * for this DO.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     */
    public void unmapAttributeDefinition_via_AttributeNumberValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, DBTransaction tran )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException {
	org.webdocwf.util.genericstore.data.AttributeNumberValueQuery q = new org.webdocwf.util.genericstore.data.AttributeNumberValueQuery();
	q.setQueryGENERICOBJECTOID( this );
	q.setQueryATTRIBUTEDEFINITIONOID( d );
	q.requireUniqueInstance();
	org.webdocwf.util.genericstore.data.AttributeNumberValueDO m = null;
	try {
	    m = q.getNextDO();
	} catch ( NonUniqueQueryException e ) { 
	    throw new DataObjectException( "Multiple mappings for " +
		this + " and " + d + " in org.webdocwf.util.genericstore.data.AttributeNumberValue table." );
	}
	m.delete( tran );
    }
    // WebDocWF extension for DODS row instance security
    // The following lines have been added:
    /**
     * From the many-to-many relationship expressed by AttributeNumberValueDO,
     * get array of AttributeDefinitionDO objects that indirectly refer
     * to this DO.
     *
     * @param usr The user for security checks
     * @return array of AttributeDefinitionDO objects.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception AccessException
     *   The user is not allowed to create a query for 
     *   org.webdocwf.util.genericstore.data.AttributeNumberValue or 
     *   to access the foreign key attribute ATTRIBUTEDEFINITIONOID.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public org.webdocwf.util.genericstore.data.AttributeDefinitionDO[] getAttributeDefinitionDOArray_via_AttributeNumberValue ( org.webdocwf.dods.access.User usr ) 
    throws DataObjectException, AccessException {
	org.webdocwf.util.genericstore.data.AttributeDefinitionDO[] ret = null;
	try {
	    org.webdocwf.util.genericstore.data.AttributeNumberValueDO[] arr = getAttributeNumberValueDOArray( usr );
         Vector tempVector = new Vector();
	    for ( int j = 0; j < arr.length; j++ ) {
		if (((org.webdocwf.util.genericstore.data.AttributeNumberValueDO)tempVector.get(j)).hasDOGetAttrATTRIBUTEDEFINITIONOIDAccess( usr)) tempVector.add(arr[ j ]);
	    }
	    ret = new org.webdocwf.util.genericstore.data.AttributeDefinitionDO[ tempVector.size() ];
	    for ( int i = 0; i < tempVector.size(); i++ ) {
		ret[ i ] = ((org.webdocwf.util.genericstore.data.AttributeNumberValueDO)tempVector.get(i)).getATTRIBUTEDEFINITIONOID( usr );
	    }
	} catch ( QueryException e ) { 
	    throw new DataObjectException( 
		"INTERNAL ERROR: ", e );
	} finally {
	    if ( null == ret )
		ret = new org.webdocwf.util.genericstore.data.AttributeDefinitionDO[ 0 ];
	}
	return ret;
    }
    /**
     * To the many-to-many relationship expressed by AttributeNumberValueDO,
     * add a AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param d The AttributeDefinitionDO to add to the AttributeNumberValueDO mapping
     * for this DO.
     * @param usr The user for security checks
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void mapAttributeDefinition_via_AttributeNumberValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, org.webdocwf.dods.access.User usr )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException, AccessException {
	mapAttributeDefinition_via_AttributeNumberValueDO( d, null, usr );
    }

    /**
     * To the many-to-many relationship expressed by AttributeNumberValueDO,
     * add a AttributeDefinitionDO object that indirectly refers to this DO.
     *
     * @param b The AttributeDefinitionDO to add to the AttributeNumberValueDO mapping for this DO.
     * @param usr The user for security checks
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void mapAttributeDefinition_via_AttributeNumberValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, DBTransaction tran, org.webdocwf.dods.access.User usr )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException, AccessException {
	org.webdocwf.util.genericstore.data.AttributeNumberValueDO m = null;
	try {
	    m = org.webdocwf.util.genericstore.data.AttributeNumberValueDO.createVirgin( usr );
	} catch ( ObjectIdException e ) { 
	    throw new DataObjectException( 
		"org.webdocwf.util.genericstore.data.AttributeNumberValueDO.createVirgin failed", e );
	}
	m.setATTRIBUTEDEFINITIONOID( d, usr );
	m.setGENERICOBJECTOID( this, usr );
	m.commit( tran );
    }

    /**
     * From the many-to-many relationship expressed by AttributeNumberValueDO,
     * remove (delete) the AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param d The AttributeDefinitionDO to remove from the AttributeNumberValueDO mapping
     * for this DO.
     * @param usr The user for security checks
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     *
     * WebDocWf extension
     *
     */
    public void unmapAttributeDefinition_via_AttributeNumberValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, org.webdocwf.dods.access.User usr )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException, AccessException {
	unmapAttributeDefinition_via_AttributeNumberValueDO( d, null, usr );
    }

    /**
     * From the many-to-many relationship expressed by AttributeNumberValueDO,
     * remove (delete) the AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param b The AttributeDefinitionDO to remove from the AttributeNumberValueDO mapping
     * for this DO.
     * @param usr The user for security checks
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     *
     * WebDocWf extension
     *
     */
    public void unmapAttributeDefinition_via_AttributeNumberValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, DBTransaction tran, org.webdocwf.dods.access.User usr )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException, AccessException {
	org.webdocwf.util.genericstore.data.AttributeNumberValueQuery q = new org.webdocwf.util.genericstore.data.AttributeNumberValueQuery( usr );
	q.setQueryGENERICOBJECTOID( this , usr);
	q.setQueryATTRIBUTEDEFINITIONOID( d , usr);
	q.requireUniqueInstance();
	org.webdocwf.util.genericstore.data.AttributeNumberValueDO m = null;
	try {
	    m = q.getNextDO();
	} catch ( NonUniqueQueryException e ) { 
	    throw new DataObjectException( "Multiple mappings for " +
		this + " and " + d + " in org.webdocwf.util.genericstore.data.AttributeNumberValue table." );
	}
	m.delete( tran, usr );
    }
// end of WebDocWf extension for DODS row instance security

    /**
     * From the many-to-many relationship expressed by AttributeObjectValueDO,
     * get array of AttributeDefinitionDO objects that indirectly refer
     * to this DO.
     *
     * @return array of AttributeDefinitionDO objects.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public org.webdocwf.util.genericstore.data.AttributeDefinitionDO[] getAttributeDefinitionDOArray_via_AttributeObjectValue () 
    throws DataObjectException {
	org.webdocwf.util.genericstore.data.AttributeDefinitionDO[] ret = null;
	try {
	    org.webdocwf.util.genericstore.data.AttributeObjectValueDO[] arr = getAttributeObjectValueDOArray();
	    ret = new org.webdocwf.util.genericstore.data.AttributeDefinitionDO[ arr.length ];
	    for ( int i = 0; i < arr.length; i++ ) {
		ret[ i ] = arr[ i ].getATTRIBUTEDEFINITIONOID();
	    }
	} catch ( Exception e ) { 
	    throw new DataObjectException( 
		"INTERNAL ERROR: ", e );
	} finally {
	    if ( null == ret )
		ret = new org.webdocwf.util.genericstore.data.AttributeDefinitionDO[ 0 ];
	}
	return ret;
    }

    /**
     * To the many-to-many relationship expressed by AttributeObjectValueDO,
     * add a AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param d The AttributeDefinitionDO to add to the AttributeObjectValueDO mapping
     * for this DO.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public void mapAttributeDefinition_via_AttributeObjectValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException {
// WebDocWf fix, was ambiguous, the following line was changed:
	mapAttributeDefinition_via_AttributeObjectValueDO( d, (DBTransaction)null );
//before: mapAttributeDefinition_via_AttributeObjectValueDO( d, null );
// end of WebDocWf fix
    }

    /**
     * To the many-to-many relationship expressed by AttributeObjectValueDO,
     * add a AttributeDefinitionDO object that indirectly refers to this DO.
     *
     * @param b The AttributeDefinitionDO to add to the AttributeObjectValueDO mapping for this DO.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public void mapAttributeDefinition_via_AttributeObjectValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, DBTransaction tran )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException {
	org.webdocwf.util.genericstore.data.AttributeObjectValueDO m = null;
	try {
	    m = org.webdocwf.util.genericstore.data.AttributeObjectValueDO.createVirgin();
	} catch ( Exception e ) { 
	    throw new DataObjectException( 
		"org.webdocwf.util.genericstore.data.AttributeObjectValueDO.createVirgin failed", e );
	}
	m.setATTRIBUTEDEFINITIONOID( d );
	m.setGENERICOBJECTOID( this );
	m.commit( tran );
    }

    /**
     * From the many-to-many relationship expressed by AttributeObjectValueDO,
     * remove (delete) the AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param d The AttributeDefinitionDO to remove from the AttributeObjectValueDO mapping
     * for this DO.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     */
    public void unmapAttributeDefinition_via_AttributeObjectValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException {
// WebDocWf fix, was ambiguous, the following line was changed:
	unmapAttributeDefinition_via_AttributeObjectValueDO( d, (DBTransaction)null );
//before: unmapAttributeDefinition_via_AttributeObjectValueDO( d, null );
// end of WebDocWf fix
    }

    /**
     * From the many-to-many relationship expressed by AttributeObjectValueDO,
     * remove (delete) the AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param b The AttributeDefinitionDO to remove from the AttributeObjectValueDO mapping
     * for this DO.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     */
    public void unmapAttributeDefinition_via_AttributeObjectValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, DBTransaction tran )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException {
	org.webdocwf.util.genericstore.data.AttributeObjectValueQuery q = new org.webdocwf.util.genericstore.data.AttributeObjectValueQuery();
	q.setQueryGENERICOBJECTOID( this );
	q.setQueryATTRIBUTEDEFINITIONOID( d );
	q.requireUniqueInstance();
	org.webdocwf.util.genericstore.data.AttributeObjectValueDO m = null;
	try {
	    m = q.getNextDO();
	} catch ( NonUniqueQueryException e ) { 
	    throw new DataObjectException( "Multiple mappings for " +
		this + " and " + d + " in org.webdocwf.util.genericstore.data.AttributeObjectValue table." );
	}
	m.delete( tran );
    }
    // WebDocWF extension for DODS row instance security
    // The following lines have been added:
    /**
     * From the many-to-many relationship expressed by AttributeObjectValueDO,
     * get array of AttributeDefinitionDO objects that indirectly refer
     * to this DO.
     *
     * @param usr The user for security checks
     * @return array of AttributeDefinitionDO objects.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception AccessException
     *   The user is not allowed to create a query for 
     *   org.webdocwf.util.genericstore.data.AttributeObjectValue or 
     *   to access the foreign key attribute ATTRIBUTEDEFINITIONOID.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public org.webdocwf.util.genericstore.data.AttributeDefinitionDO[] getAttributeDefinitionDOArray_via_AttributeObjectValue ( org.webdocwf.dods.access.User usr ) 
    throws DataObjectException, AccessException {
	org.webdocwf.util.genericstore.data.AttributeDefinitionDO[] ret = null;
	try {
	    org.webdocwf.util.genericstore.data.AttributeObjectValueDO[] arr = getAttributeObjectValueDOArray( usr );
         Vector tempVector = new Vector();
	    for ( int j = 0; j < arr.length; j++ ) {
		if (((org.webdocwf.util.genericstore.data.AttributeObjectValueDO)tempVector.get(j)).hasDOGetAttrATTRIBUTEDEFINITIONOIDAccess( usr)) tempVector.add(arr[ j ]);
	    }
	    ret = new org.webdocwf.util.genericstore.data.AttributeDefinitionDO[ tempVector.size() ];
	    for ( int i = 0; i < tempVector.size(); i++ ) {
		ret[ i ] = ((org.webdocwf.util.genericstore.data.AttributeObjectValueDO)tempVector.get(i)).getATTRIBUTEDEFINITIONOID( usr );
	    }
	} catch ( QueryException e ) { 
	    throw new DataObjectException( 
		"INTERNAL ERROR: ", e );
	} finally {
	    if ( null == ret )
		ret = new org.webdocwf.util.genericstore.data.AttributeDefinitionDO[ 0 ];
	}
	return ret;
    }
    /**
     * To the many-to-many relationship expressed by AttributeObjectValueDO,
     * add a AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param d The AttributeDefinitionDO to add to the AttributeObjectValueDO mapping
     * for this DO.
     * @param usr The user for security checks
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void mapAttributeDefinition_via_AttributeObjectValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, org.webdocwf.dods.access.User usr )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException, AccessException {
	mapAttributeDefinition_via_AttributeObjectValueDO( d, null, usr );
    }

    /**
     * To the many-to-many relationship expressed by AttributeObjectValueDO,
     * add a AttributeDefinitionDO object that indirectly refers to this DO.
     *
     * @param b The AttributeDefinitionDO to add to the AttributeObjectValueDO mapping for this DO.
     * @param usr The user for security checks
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void mapAttributeDefinition_via_AttributeObjectValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, DBTransaction tran, org.webdocwf.dods.access.User usr )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException, AccessException {
	org.webdocwf.util.genericstore.data.AttributeObjectValueDO m = null;
	try {
	    m = org.webdocwf.util.genericstore.data.AttributeObjectValueDO.createVirgin( usr );
	} catch ( ObjectIdException e ) { 
	    throw new DataObjectException( 
		"org.webdocwf.util.genericstore.data.AttributeObjectValueDO.createVirgin failed", e );
	}
	m.setATTRIBUTEDEFINITIONOID( d, usr );
	m.setGENERICOBJECTOID( this, usr );
	m.commit( tran );
    }

    /**
     * From the many-to-many relationship expressed by AttributeObjectValueDO,
     * remove (delete) the AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param d The AttributeDefinitionDO to remove from the AttributeObjectValueDO mapping
     * for this DO.
     * @param usr The user for security checks
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     *
     * WebDocWf extension
     *
     */
    public void unmapAttributeDefinition_via_AttributeObjectValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, org.webdocwf.dods.access.User usr )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException, AccessException {
	unmapAttributeDefinition_via_AttributeObjectValueDO( d, null, usr );
    }

    /**
     * From the many-to-many relationship expressed by AttributeObjectValueDO,
     * remove (delete) the AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param b The AttributeDefinitionDO to remove from the AttributeObjectValueDO mapping
     * for this DO.
     * @param usr The user for security checks
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     *
     * WebDocWf extension
     *
     */
    public void unmapAttributeDefinition_via_AttributeObjectValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, DBTransaction tran, org.webdocwf.dods.access.User usr )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException, AccessException {
	org.webdocwf.util.genericstore.data.AttributeObjectValueQuery q = new org.webdocwf.util.genericstore.data.AttributeObjectValueQuery( usr );
	q.setQueryGENERICOBJECTOID( this , usr);
	q.setQueryATTRIBUTEDEFINITIONOID( d , usr);
	q.requireUniqueInstance();
	org.webdocwf.util.genericstore.data.AttributeObjectValueDO m = null;
	try {
	    m = q.getNextDO();
	} catch ( NonUniqueQueryException e ) { 
	    throw new DataObjectException( "Multiple mappings for " +
		this + " and " + d + " in org.webdocwf.util.genericstore.data.AttributeObjectValue table." );
	}
	m.delete( tran, usr );
    }
// end of WebDocWf extension for DODS row instance security

    /**
     * From the many-to-many relationship expressed by AttributeStringValueDO,
     * get array of AttributeDefinitionDO objects that indirectly refer
     * to this DO.
     *
     * @return array of AttributeDefinitionDO objects.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public org.webdocwf.util.genericstore.data.AttributeDefinitionDO[] getAttributeDefinitionDOArray_via_AttributeStringValue () 
    throws DataObjectException {
	org.webdocwf.util.genericstore.data.AttributeDefinitionDO[] ret = null;
	try {
	    org.webdocwf.util.genericstore.data.AttributeStringValueDO[] arr = getAttributeStringValueDOArray();
	    ret = new org.webdocwf.util.genericstore.data.AttributeDefinitionDO[ arr.length ];
	    for ( int i = 0; i < arr.length; i++ ) {
		ret[ i ] = arr[ i ].getATTRIBUTEDEFINITIONOID();
	    }
	} catch ( Exception e ) { 
	    throw new DataObjectException( 
		"INTERNAL ERROR: ", e );
	} finally {
	    if ( null == ret )
		ret = new org.webdocwf.util.genericstore.data.AttributeDefinitionDO[ 0 ];
	}
	return ret;
    }

    /**
     * To the many-to-many relationship expressed by AttributeStringValueDO,
     * add a AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param d The AttributeDefinitionDO to add to the AttributeStringValueDO mapping
     * for this DO.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public void mapAttributeDefinition_via_AttributeStringValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException {
// WebDocWf fix, was ambiguous, the following line was changed:
	mapAttributeDefinition_via_AttributeStringValueDO( d, (DBTransaction)null );
//before: mapAttributeDefinition_via_AttributeStringValueDO( d, null );
// end of WebDocWf fix
    }

    /**
     * To the many-to-many relationship expressed by AttributeStringValueDO,
     * add a AttributeDefinitionDO object that indirectly refers to this DO.
     *
     * @param b The AttributeDefinitionDO to add to the AttributeStringValueDO mapping for this DO.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     */
    public void mapAttributeDefinition_via_AttributeStringValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, DBTransaction tran )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException {
	org.webdocwf.util.genericstore.data.AttributeStringValueDO m = null;
	try {
	    m = org.webdocwf.util.genericstore.data.AttributeStringValueDO.createVirgin();
	} catch ( Exception e ) { 
	    throw new DataObjectException( 
		"org.webdocwf.util.genericstore.data.AttributeStringValueDO.createVirgin failed", e );
	}
	m.setATTRIBUTEDEFINITIONOID( d );
	m.setGENERICOBJECTOID( this );
	m.commit( tran );
    }

    /**
     * From the many-to-many relationship expressed by AttributeStringValueDO,
     * remove (delete) the AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param d The AttributeDefinitionDO to remove from the AttributeStringValueDO mapping
     * for this DO.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     */
    public void unmapAttributeDefinition_via_AttributeStringValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException {
// WebDocWf fix, was ambiguous, the following line was changed:
	unmapAttributeDefinition_via_AttributeStringValueDO( d, (DBTransaction)null );
//before: unmapAttributeDefinition_via_AttributeStringValueDO( d, null );
// end of WebDocWf fix
    }

    /**
     * From the many-to-many relationship expressed by AttributeStringValueDO,
     * remove (delete) the AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param b The AttributeDefinitionDO to remove from the AttributeStringValueDO mapping
     * for this DO.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     */
    public void unmapAttributeDefinition_via_AttributeStringValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, DBTransaction tran )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException {
	org.webdocwf.util.genericstore.data.AttributeStringValueQuery q = new org.webdocwf.util.genericstore.data.AttributeStringValueQuery();
	q.setQueryGENERICOBJECTOID( this );
	q.setQueryATTRIBUTEDEFINITIONOID( d );
	q.requireUniqueInstance();
	org.webdocwf.util.genericstore.data.AttributeStringValueDO m = null;
	try {
	    m = q.getNextDO();
	} catch ( NonUniqueQueryException e ) { 
	    throw new DataObjectException( "Multiple mappings for " +
		this + " and " + d + " in org.webdocwf.util.genericstore.data.AttributeStringValue table." );
	}
	m.delete( tran );
    }
    // WebDocWF extension for DODS row instance security
    // The following lines have been added:
    /**
     * From the many-to-many relationship expressed by AttributeStringValueDO,
     * get array of AttributeDefinitionDO objects that indirectly refer
     * to this DO.
     *
     * @param usr The user for security checks
     * @return array of AttributeDefinitionDO objects.
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception AccessException
     *   The user is not allowed to create a query for 
     *   org.webdocwf.util.genericstore.data.AttributeStringValue or 
     *   to access the foreign key attribute ATTRIBUTEDEFINITIONOID.
     *
     * This is a WebDocWf extension for DODS row instance security
     *
     */
    public org.webdocwf.util.genericstore.data.AttributeDefinitionDO[] getAttributeDefinitionDOArray_via_AttributeStringValue ( org.webdocwf.dods.access.User usr ) 
    throws DataObjectException, AccessException {
	org.webdocwf.util.genericstore.data.AttributeDefinitionDO[] ret = null;
	try {
	    org.webdocwf.util.genericstore.data.AttributeStringValueDO[] arr = getAttributeStringValueDOArray( usr );
         Vector tempVector = new Vector();
	    for ( int j = 0; j < arr.length; j++ ) {
		if (((org.webdocwf.util.genericstore.data.AttributeStringValueDO)tempVector.get(j)).hasDOGetAttrATTRIBUTEDEFINITIONOIDAccess( usr)) tempVector.add(arr[ j ]);
	    }
	    ret = new org.webdocwf.util.genericstore.data.AttributeDefinitionDO[ tempVector.size() ];
	    for ( int i = 0; i < tempVector.size(); i++ ) {
		ret[ i ] = ((org.webdocwf.util.genericstore.data.AttributeStringValueDO)tempVector.get(i)).getATTRIBUTEDEFINITIONOID( usr );
	    }
	} catch ( QueryException e ) { 
	    throw new DataObjectException( 
		"INTERNAL ERROR: ", e );
	} finally {
	    if ( null == ret )
		ret = new org.webdocwf.util.genericstore.data.AttributeDefinitionDO[ 0 ];
	}
	return ret;
    }
    /**
     * To the many-to-many relationship expressed by AttributeStringValueDO,
     * add a AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param d The AttributeDefinitionDO to add to the AttributeStringValueDO mapping
     * for this DO.
     * @param usr The user for security checks
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void mapAttributeDefinition_via_AttributeStringValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, org.webdocwf.dods.access.User usr )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException, AccessException {
	mapAttributeDefinition_via_AttributeStringValueDO( d, null, usr );
    }

    /**
     * To the many-to-many relationship expressed by AttributeStringValueDO,
     * add a AttributeDefinitionDO object that indirectly refers to this DO.
     *
     * @param b The AttributeDefinitionDO to add to the AttributeStringValueDO mapping for this DO.
     * @param usr The user for security checks
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     *
     * WebDocWf extension
     *
     */
    public void mapAttributeDefinition_via_AttributeStringValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, DBTransaction tran, org.webdocwf.dods.access.User usr )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException, AccessException {
	org.webdocwf.util.genericstore.data.AttributeStringValueDO m = null;
	try {
	    m = org.webdocwf.util.genericstore.data.AttributeStringValueDO.createVirgin( usr );
	} catch ( ObjectIdException e ) { 
	    throw new DataObjectException( 
		"org.webdocwf.util.genericstore.data.AttributeStringValueDO.createVirgin failed", e );
	}
	m.setATTRIBUTEDEFINITIONOID( d, usr );
	m.setGENERICOBJECTOID( this, usr );
	m.commit( tran );
    }

    /**
     * From the many-to-many relationship expressed by AttributeStringValueDO,
     * remove (delete) the AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param d The AttributeDefinitionDO to remove from the AttributeStringValueDO mapping
     * for this DO.
     * @param usr The user for security checks
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     *
     * WebDocWf extension
     *
     */
    public void unmapAttributeDefinition_via_AttributeStringValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, org.webdocwf.dods.access.User usr )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException, AccessException {
	unmapAttributeDefinition_via_AttributeStringValueDO( d, null, usr );
    }

    /**
     * From the many-to-many relationship expressed by AttributeStringValueDO,
     * remove (delete) the AttributeDefinitionDO object that indirectly refers
     * to this DO.
     *
     * @param b The AttributeDefinitionDO to remove from the AttributeStringValueDO mapping
     * for this DO.
     * @param usr The user for security checks
     *
     * @exception DataObjectException
     *   If the object is not found in the database.
     * @exception QueryException
     *   If an error occured while building the query before execution.
     *
     * WebDocWf extension
     *
     */
    public void unmapAttributeDefinition_via_AttributeStringValueDO( org.webdocwf.util.genericstore.data.AttributeDefinitionDO d, DBTransaction tran, org.webdocwf.dods.access.User usr )
    throws DataObjectException, DatabaseManagerException, RefAssertionException, SQLException, DBRowUpdateException, QueryException, AccessException {
	org.webdocwf.util.genericstore.data.AttributeStringValueQuery q = new org.webdocwf.util.genericstore.data.AttributeStringValueQuery( usr );
	q.setQueryGENERICOBJECTOID( this , usr);
	q.setQueryATTRIBUTEDEFINITIONOID( d , usr);
	q.requireUniqueInstance();
	org.webdocwf.util.genericstore.data.AttributeStringValueDO m = null;
	try {
	    m = q.getNextDO();
	} catch ( NonUniqueQueryException e ) { 
	    throw new DataObjectException( "Multiple mappings for " +
		this + " and " + d + " in org.webdocwf.util.genericstore.data.AttributeStringValue table." );
	}
	m.delete( tran, usr );
    }
// end of WebDocWf extension for DODS row instance security


    /**
     * A stub method for implementing pre-commit assertions 
     * for this GenericObjectDO.
     * Implement this stub to throw an RefAssertionException for cases
     * where this object is not valid for writing to the database.
     */
    protected void okToCommit() 
    throws RefAssertionException { }

    /**
     * A stub method for implementing pre-delete assertions 
     * for this GenericObjectDO.
     * Implement this stub to throw an RefAssertionException for cases
     * where this object is not valid for deletion from the database.
     */
    protected void okToDelete() 
    throws RefAssertionException { }

  /**
   * Inserts/Updates the DO into its table.
   *
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   * @deprecated Use save() instead.
   */
  public void commit() 
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
// WebDocWf extension for generic store
// The following line was put under comments
//    modifyDO( null, false );
// The following lines have been inserted:
    DBTransaction dbt = DODS.getDatabaseManager().createTransaction();

    save(dbt);
    dbt.commit();
    dbt.release();
// end of WebDocWf extension for generic store
  }

  /**
   * Inserts/Updates the DO into its table.
   *
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   * 
   * WebDocWf extension
   */
  public void save() 
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
	save(true);
  }

  /**
   * Inserts/Updates the DO into its table.
   *
   * @param references True if references should be saved with this DO
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   * 
   * WebDocWf extension
   */
  public void save(boolean references) 
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
// WebDocWf extension for generic store
// The following line was put under comments
//    modifyDO( null, false, references );
// The following lines have been inserted:
    DBTransaction dbt = DODS.getDatabaseManager().createTransaction();

    save(dbt, references);
    dbt.commit();
    dbt.release();
// end of WebDocWf extension for generic store
  }

  /**
   * Inserts/Updates the DO into its table.
   * The transaction is likely provided by the commit() method of another DO
   * which references this DO.
   * 
   * @param dbt The transaction object to use for this operation.
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception com.lutris.appserver.server.sql.DBRowUpdateException if a version error occurs.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   * @deprecated Use save() instead.
   */
  public void commit(DBTransaction dbt)
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
// WebDocWf extension for generic store
// The following line has been inserted:
    DBTransaction dbtlocal = dbt;
    if (dbtlocal == null) dbtlocal = DODS.getDatabaseManager().createTransaction();
// The following line has been changed:
    modifyDO( dbtlocal, false );
// before: modifyDO( dbt, false );
// The following line has been inserted:
    super.save(dbtlocal);
// end of WebDocWf extension for generic store
  }

  /**
   * Inserts/Updates the DO into its table.
   * The transaction is likely provided by the commit() method of another DO
   * which references this DO.
   * 
   * @param dbt The transaction object to use for this operation.
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception com.lutris.appserver.server.sql.DBRowUpdateException if a version error occurs.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   * 
   * WebDocWf extension
   */
  public void save(DBTransaction dbt)
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
	save(dbt, true);
  }

  /**
   * Inserts/Updates the DO into its table.
   * The transaction is likely provided by the commit() method of another DO
   * which references this DO.
   * 
   * @param dbt The transaction object to use for this operation.
   * @param references True if references of this DO should be saved
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception com.lutris.appserver.server.sql.DBRowUpdateException if a version error occurs.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   * 
   * WebDocWf extension
   */
  public void save(DBTransaction dbt, boolean references)
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
// WebDocWf extension for generic store
// The following line has been inserted:
    DBTransaction dbtlocal = dbt;
    if (dbtlocal == null) dbtlocal = DODS.getDatabaseManager().createTransaction();
// The following line has been changed:
    modifyDO( dbtlocal, false, references );
// before: modifyDO( dbt, false );
// The following line has been inserted:
    super.save(dbtlocal);
// end of WebDocWf extension for generic store
  }

  /**
   * Deletes the DO from its table.
   *
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   */
  public void delete() 
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {

// WebDocWf extension for generic store
// The following line was put under comments
//    modifyDO( null, true );
// The following lines have been inserted:
    DBTransaction dbt = DODS.getDatabaseManager().createTransaction();
    delete(dbt);
    dbt.commit();
    dbt.release();
// end of WebDocWf extension for generic store
  }

  /**
   * Deletes the DO from its table.
   * The transaction is likely provided by the delete() method of another DO
   * which references this DO.
   *
   * @param dbt The transaction object to use for this operation.
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception com.lutris.appserver.server.sql.DBRowUpdateException if a version error occurs.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   */
  public void delete(DBTransaction dbt)
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
// WebDocWf extension for generic store
// The following lines have been inserted:
    DBTransaction dbtlocal = dbt;
    if (dbtlocal == null) dbtlocal = DODS.getDatabaseManager().createTransaction();
    super.delete(dbtlocal);
// The following line has been changed:
    modifyDO( dbtlocal, true );
// before:    modifyDO( dbt, true );
// end of WebDocWf extension for generic store
  }

      /**
     * A stub method for implementing pre-commit assertions 
     * for the TYPEDEFINITIONOID data member.
     * Implement this stub to throw an RefAssertionException for cases
     * where TYPEDEFINITIONOID is not valid for writing to the database.
     */
    protected void okToCommitTYPEDEFINITIONOID( org.webdocwf.util.genericstore.data.TypeDefinitionDO member ) 
    throws RefAssertionException { }

    /**
     * A stub method for implementing pre-delete assertions 
     * for the TYPEDEFINITIONOID data member.
     * Implement this stub to throw an RefAssertionException for cases
     * where TYPEDEFINITIONOID is not valid for deletion from the database.
     */
    protected void okToDeleteTYPEDEFINITIONOID( org.webdocwf.util.genericstore.data.TypeDefinitionDO member ) 
    throws RefAssertionException { }



  /**
   * Modifies the DO within its table.
   * Performs recursive commit/delete on referenced DOs;
   * all operations occur within a single transaction
   * to allow rollback in the event of error.
   * Only the creator of the transaction releases it.
   *
   * @param dbt The transaction object to use for this operation.
   * @param delete True if doing a delete, otherwise doing insert/update.
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception com.lutris.appserver.server.sql.DBRowUpdateException if a version error occurs.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   * 
   * WebDocWf extension
   */
// WebDocWf extension for save without references
// The following line has been modofied
  protected void modifyDO( DBTransaction dbt, boolean delete)
// before:  protected void modifyDO( DBTransaction dbt, boolean delete )
// end of WebDocWf extension for save without references
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
	modifyDO( dbt, delete, true);
  }
  /**
   * Modifies the DO within its table.
   * Performs recursive commit/delete on referenced DOs;
   * all operations occur within a single transaction
   * to allow rollback in the event of error.
   * Only the creator of the transaction releases it.
   *
   * @param dbt The transaction object to use for this operation.
   * @param delete True if doing a delete, otherwise doing insert/update.
   * @param references True if references should be saved
   * @exception com.lutris.appserver.server.sql.DatabaseManagerException if a Transaction can not be created.
   * @exception com.lutris.appserver.server.sql.DBRowUpdateException if a version error occurs.
   * @exception RefAssertionException thrown by okTo method.
   * @exception java.sql.SQLException if any SQL errors occur.
   * 
   * WebDocWf extension
   */
// WebDocWf extension for save without references
// The following line has been modofied
  protected void modifyDO( DBTransaction dbt, boolean delete, boolean references )
// before:  protected void modifyDO( DBTransaction dbt, boolean delete )
// end of WebDocWf extension for save without references
  throws SQLException, DatabaseManagerException, DataObjectException, RefAssertionException, DBRowUpdateException, QueryException {
    if ( delete )
	okToDelete();
    else
	okToCommit();
    boolean ownTransaction = false;
    try {
      if ( null == dbt ) {
	DatabaseManager dbm = DODS.getDatabaseManager();
        dbt = dbm.createTransaction();      // create a transaction
	ownTransaction = true;
      } 
// WebDocWf fix for circular references
// The following lines have been inserted
	  else
      {
        CoreDO foundDO = dbt.getDO(this);
        if (foundDO!=null) return;
      }
// end of WebDocWf fix for circular references
      if ( null == dbt )
	throw new DatabaseManagerException(
		"DatabaseManager.createTransaction returned null." );
      if ( delete ) {
	  // Code to perform cascading deletes is generated here
	  // if cascading deletes are not supported by the database.      
	  	
	{
	    // perform cascading delete on referring table
	    org.webdocwf.util.genericstore.data.AttributeBoolValueDO[] a = getAttributeBoolValueDOArray();
	    for ( int i = 0; i < a.length; i++ ) {
		a[ i ].delete( dbt );
	    }
	}
	
	
	{
	    // perform cascading delete on referring table
	    org.webdocwf.util.genericstore.data.AttributeDateValueDO[] a = getAttributeDateValueDOArray();
	    for ( int i = 0; i < a.length; i++ ) {
		a[ i ].delete( dbt );
	    }
	}
	
	
	{
	    // perform cascading delete on referring table
	    org.webdocwf.util.genericstore.data.AttributeEnumValueDO[] a = getAttributeEnumValueDOArray();
	    for ( int i = 0; i < a.length; i++ ) {
		a[ i ].delete( dbt );
	    }
	}
	
	
	{
	    // perform cascading delete on referring table
	    org.webdocwf.util.genericstore.data.AttributeNumberValueDO[] a = getAttributeNumberValueDOArray();
	    for ( int i = 0; i < a.length; i++ ) {
		a[ i ].delete( dbt );
	    }
	}
	
	
	{
	    // perform cascading delete on referring table
	    org.webdocwf.util.genericstore.data.AttributeObjectValueDO[] a = getAttributeObjectValueDOArray();
	    for ( int i = 0; i < a.length; i++ ) {
		a[ i ].delete( dbt );
	    }
	}
	
	
	{
	    // perform cascading delete on referring table
	    org.webdocwf.util.genericstore.data.AttributeStringValueDO[] a = getAttributeStringValueDOArray();
	    for ( int i = 0; i < a.length; i++ ) {
		a[ i ].delete( dbt );
	    }
	}
	

	  // The following line keeps the compiler happy 
	  // when the CASCADING_DELETES tag is empty.
          if ( false )
	      throw new QueryException("XXX");
      } else {
// WebDocWf extension for save without references
// The following line has been inserted
	  if (references) {
// end of WebDocWf extension for save without references
	  // commit referenced DOs.
	  	org.webdocwf.util.genericstore.data.TypeDefinitionDO TYPEDEFINITIONOID_DO = getTYPEDEFINITIONOID();
	if ( null != TYPEDEFINITIONOID_DO ) {
	    if ( TYPEDEFINITIONOID_DO.isLoaded() ) {
		okToCommitTYPEDEFINITIONOID( TYPEDEFINITIONOID_DO );
		TYPEDEFINITIONOID_DO.commit( dbt );
	    } else {
		// since the referenced DO is not loaded,
		// it cannot be dirty, so there is no need to commit it.
	    }
	} else {
	    if ( ! false )
		throw new RefAssertionException(
		    "Cannot commit GenericObjectDO ( " + toString() +
		    " ) because TYPEDEFINITIONOID is not allowed to be null." );
	}

// WebDocWf extension for save without references
// The following line has been inserted
	  }
// end of WebDocWf extension for save without references
      }
      if ( false ) {
	  // This throw is here to keep the compiler happy
	  // in the case of a DO that does not refer to other DOs.
	  // In that case, the above delete/commit code blocks will be empty
	  // and throw nothing.
	  throw new DataObjectException( "foo" );	  
      }
      if ( delete ) {
          dbt.delete( this );
      } else {
	  if ( isLoaded() )
	      dbt.insert( this );   // dbt.insert() handles insertions and updates
      }
      if (ownTransaction) {
	  dbt.commit(); // commit the transaction
      }
    } catch (SQLException sqle) {
      StringBuffer message = new StringBuffer("Failed to insert/update DO: ");
      message.append(sqle.getMessage());

      // rollback, if necessary
      if (ownTransaction) {
        try {
          dbt.rollback();
        } catch (SQLException sqle2) {
          message.insert(0,"\n");
          message.insert(0,sqle2.getMessage());
          message.insert(0,"Rollback failed: ");
        }
      }
      throw new SQLException(message.toString());
    } finally {
      // release the transaction, if any
      if (ownTransaction) {
        dbt.release();
      }
    }
  }
}
